//
// CCoroutineNT.cpp
//
// NT implementation of Coroutine interface.


#if (!defined(_WINNT) || !defined(_WIN32_WINNT))
#error "This module is only for Windows NT."
#endif

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <assert.h>

#include "CCoroutine.h"
#include "CWinNTCoroutine.h"

///////////////////////////////////////////////////////////

class CCoroutineState
{
public:
	CCoroutineState() : fiber_(0) {}

	LPVOID fiber_;
};

static void WINAPI startCoroutine(PVOID lpParameter)
{
	CCoroutine *c = (CCoroutine *) lpParameter;

	c->go();

	assert(0);
}


///////////////////////////////////////////////////////////

CWinNTCoroutine::CWinNTCoroutine(unsigned stackSizeInBytes) : CCoroutine(INITCOROUTINE)
{

	state_ = new CCoroutineState();

	if (stackSizeInBytes == INITCOROUTINE)
		return;

	state_->fiber_ = CreateFiber(0, startCoroutine, this);
	assert(state_->fiber_);
}


CWinNTCoroutine::~CWinNTCoroutine()
{
	assert(state_->fiber_);
	DeleteFiber(state_->fiber_);
	state_->fiber_ = 0;
	delete state_;
}

void CWinNTCoroutine::resume(CCoroutine *other)
{
	if (other == this) return;

	assert(other->state_->fiber_);
	SwitchToFiber(other->state_->fiber_);
}

/////////////////////////////////////////////////////////
// This can't be pure virtual because it is static

CCoroutine *CCoroutine::initialize()
{
	CCoroutine *thisCoroutine = new CWinNTCoroutine();

	LPVOID thisFiber = ConvertThreadToFiber(thisCoroutine);
	assert(thisFiber != 0);

	thisCoroutine->state_->fiber_ = thisFiber;

	return thisCoroutine;
}

