//  CLASS TCHARTFORM
//  This is the base class for any chart used in
//  the Tiltmeters application. All charting window
//  are derived from this one.
//
// (C) Steve JORDI - 1996-1999  
// steve@sjordi.com - www.sjordi.com
//
// All this code is copyrighted by Steve Jordi and
// cannot be reproduced without the consent of the author.
// The executable may be freely distributed.
//
// The charting tool is GigaSoft's Pro Essential.
// All charting function calls are proprietary of Pro Essential.
// It is represented by the "theChart" variable.
//
// MAKE  SURE  TO  UNDERSTAND  AND  ACCEPT  THE STATEMENTS BELOW
// BEFORE USING THE SOFTWARE:
//
// THIS PROGRAM  IS  PROVIDED  "AS IS"  WITHOUT  WARRANTY OF ANY
// KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED
// TO  THE  IMPLIED  WARRANTIES  OF MERCHANTABILITY  AND FITNESS
// FOR  A  PARTICULAR PURPOSE. THE ENTIRE RISK AS TO THE QUALITY
// AND   PERFORMANCE  OF  THE  PROGRAM  IS  WITH YOU. SHOULD THE
// PROGRAM  PROVE   DEFECTIVE,  YOU   (NOT  STEVE JORDI)  ASSUME
// THE ENTIRE COST OF ALL  NECESSARY   SERVICING,   REPAIR,   OR
// CORRECTION. IN NO EVENT WILL STEVE JORDI BE LIABLE TO YOU FOR
// ANY  DAMAGES ,  INCLUDING ANY LOST PROFITS,  LOST SAVINGS, OR
// OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES  ARISING OUT OF THE
// USE OR INABILITY TO USE  SUCH  PROGRAM  EVEN  IF  STEVE JORDI
// HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES,   OR FOR
// ANY  CLAIM BY ANY OTHER PARTY.
// ALSO, USE IT IN LIFE THREATENING SITUATIONS AT YOUR OWN RISKS
// (SUCH AS VOLCANO MONITORING). STEVE JORDI CANNOT BE ACCOUNTED
// AS RESPONSIBLE FOR ANY LIFE OR PROPERTY LOST IN CASE AN ALARM
// SYSTEM HAD A MALFUNCTION OR WAS NOT ABLE TO CORRECTLY  BEHAVE
// IN A CRISIS ENVIRONMENT.
//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop

#include "Chart.h"
#include "main.h"
#include <fstream.h>
#include <math.h>
//---------------------------------------------------------------------------
#pragma link "Pesgvcl"
#pragma resource "*.dfm"
TChartForm *ChartForm;

const float HIDDEN = -9.99e-90;

// --------------------------------------------------------
// Constructor
// Initialize all variables (if any)
// --------------------------------------------------------
__fastcall TChartForm::TChartForm(TComponent* Owner)
   : TForm(Owner) {
   nbPoints       = 0 ; // Nb points in chart
   pointIndex     = 0 ; // Current point index
   currentSerie   = 0 ; // Current active serie
   nbSeries       = 1 ; // Nb of series in a chart
   defaultX       = 0 ;
   defaultY       = MainToolBar->Top+MainToolBar->Height+1 ;
   defaultWidth   = CHARTRIGHT / 4 -1 ;
   defaultHeight  = CHARTHEIGHT / 3 - 1 ;
   // Store parent window handle
   parent = Owner ;

   maximized = 0 ;
   alreadyDisplayed = false ;
   canUpdateMinimizeButton = true ;

   // Define the parser variables
   vbatPtr  = ucDefineVariable( "vbat" ) ;
   taPtr    = ucDefineVariable( "ta"   ) ;
   utPtr    = ucDefineVariable( "ut"   ) ;
   tf0Ptr   = ucDefineVariable( "tf0"  ) ;
   e1Ptr    = ucDefineVariable( "e1"   ) ;
   po2Ptr   = ucDefineVariable( "po2"  ) ;
   coefPtr  = ucDefineVariable( "coef" ) ;
   inclPtr  = ucDefineVariable( "incl" ) ;

   taPtr2   = ucDefineVariable( "ta2"   ) ;
   utPtr2   = ucDefineVariable( "ut2"   ) ;
   tf0Ptr2  = ucDefineVariable( "tf02"  ) ;
   e1Ptr2   = ucDefineVariable( "e12"   ) ;
   po2Ptr2  = ucDefineVariable( "po22"  ) ;
   coefPtr2 = ucDefineVariable( "coef2" ) ;
   inclPtr2 = ucDefineVariable( "incl2" ) ;


   theChart->MainTitle = "" ;
   theChart->SubTitle  = " " ;
   theChart->XAxisLabel = "" ;
   theChart->YAxisLabel = "" ;
   theChart->UsingXDataII = true ;
   theChart->SubsetByPoint = false ;
   theChart->XAxisScaleControl = sgAuto ;
   theChart->DateTimeMode = sgVBDateTime ;
   theChart->PEactions = sgReinitAndReset ;
   theChart->ShowAnnotations = false ;

   Configure = new TConfigure( Owner ) ;

}


// --------------------------------------------------------
// ADDDATA
// Add a point.
// Note that the point index is automatically handled.
// If you want to modify an existing point, use MODIFYDATA
// --------------------------------------------------------
void TChartForm::AddData(double x, float y) {
   theChart->Points = pointIndex+1  ;

   theChart->XDataII[0][pointIndex  ] = x ;
   theChart->YData[0][pointIndex++] = y ;
}

// --------------------------------------------------------
// ADDDATA
// Add a point in Y and Y2
// Note that the point index is automatically handled.
// If you want to modify an existing point, use MODIFYDATA
// --------------------------------------------------------
void TChartForm::AddData(double x, float y, float y2) {
   theChart->Points = pointIndex+1  ;

   theChart->XDataII[0][pointIndex  ] = x ;
   theChart->YData[0][pointIndex] = y ;
   theChart->XDataII[1][pointIndex  ] = x ;
   theChart->YData[1][pointIndex++] = y2 ;
}

// --------------------------------------------------------
// CHANGEDATA
// Change a point.
// Note that the point index is automatically handled.
// If you want to modify an existing point, use MODIFYDATA
// --------------------------------------------------------
void TChartForm::ChangeData(float y) {
   theChart->YData[0][pointIndex++] = y ;
}

// --------------------------------------------------------
// CHANGEDATA
// Change a point in Y and Y2
// Note that the point index is automatically handled.
// If you want to modify an existing point, use MODIFYDATA
// --------------------------------------------------------
void TChartForm::ChangeData(float y, float y2) {
   theChart->YData[0][pointIndex] = y ;
   theChart->YData[1][pointIndex++] = y2 ;
}

// --------------------------------------------------------
// PREPAREDATAUPDATE
// Tell the chart we are about to change the data...
// It is just a matter of resetting pointIndex to 0
// --------------------------------------------------------
void TChartForm::PrepareDataUpdate() {
   pointIndex = 0 ;
}


// --------------------------------------------------------
// SETPOSITION
// Resize the window
// --------------------------------------------------------
void TChartForm::SetPosition( int x, int y, int w, int h ) {
   Top    = y ;
   Left   = x ;
   Width  = w ; // Right is used as the width
   Height = h ; // Bottom used as Height
   if( alreadyDisplayed ) {
      MainToolBar->MinimizeButton->Enabled = true ;
      MainToolBar->MaximizeButton->Enabled = true ;
      MainToolBar->DataButton->Enabled      = true ;
   }
   else {
      MainToolBar->MinimizeButton->Enabled = false ;
      MainToolBar->MaximizeButton->Enabled = false ;
      MainToolBar->DataButton->Enabled      = false ;
      alreadyDisplayed = true ;
   }
   maximized = 2 ;
}



// --------------------------------------------------------
// SETCAPTION
// Set the window title
// --------------------------------------------------------
void TChartForm::SetCaption( AnsiString str ) {
   Caption = str ;
}


// --------------------------------------------------------
// SETNBPOINTS
// Set the nb of points for the current serie
// --------------------------------------------------------
void TChartForm::SetNbPoints( int nPoints ) {
   nbPoints = nPoints ;
   theChart->Points = nbPoints ;
}


// --------------------------------------------------------
// SETNBSERIES
// Set the nb of series to be displayed
// --------------------------------------------------------
void TChartForm::SetNbSeries( int nb ) {
   nbSeries = nb ;
   theChart->Subsets = nbSeries ;
   theChart->SubsetColors[0] = clNavy ;
   theChart->PlottingMethod = sgLine ;
   theChart->SubsetByPoint = false ;
   theChart->SubsetsToLegend[0] = -1 ;
   theChart->SubsetLineTypes[0] = PELT_THINSOLID ;
   // Pass dummy data to allocate memory by default 15000 points
   theChart->Points = DEFAULTPOINTS ;
   theChart->XDataII[0][DEFAULTPOINTS-1] = 0.0 ;
   theChart->YData[0][DEFAULTPOINTS-1] = 0.0 ;
   if( nbSeries == 2 ) {
      theChart->RYAxisComparisonSubsets = 1 ;
      theChart->PlottingMethodII = sgLine2 ;
      theChart->SubsetsToLegend[1] = -1 ;
      theChart->SubsetColors[1] = clRed ;
      theChart->SubsetLineTypes[1] = PELT_THINSOLID ;
      theChart->XDataII[1][DEFAULTPOINTS-1] = 0.0 ;
      theChart->YData[1][DEFAULTPOINTS-1] = 0.0 ;
   }
}


// --------------------------------------------------------
// SETSERIE
// Set current serie affected by data
// --------------------------------------------------------
void TChartForm::SetSerie( int serie ) {
   if( (serie < 0) | (serie > nbSeries-1 ) )
      MessageBox( this, "Serie out of range", "CHART Error", MB_OK ) ;
   currentSerie = serie ;
}



// --------------------------------------------------------
// ADDHIDDENPOINT
// Add a hidden point (for example when data is missing)
// --------------------------------------------------------
void TChartForm::AddHiddenPoint( int y ) {
   theChart->Points = ++pointIndex  ;

   theChart->XDataII[currentSerie][y] = HIDDEN ;
   theChart->PEactions = sgReinitAndReset ;
}



// --------------------------------------------------------
// SET TOP TITLE
// Add a title on top of the chart
// --------------------------------------------------------
void TChartForm::SetTopTitle( AnsiString str ) {
   theChart->SubTitle = str ;
}


// --------------------------------------------------------
// SET LEFT TITLE
// Add a title next to the first Y axis
// --------------------------------------------------------
void TChartForm::SetLeftTitle( AnsiString str ) {
   theChart->YAxisLabel = str ;
}


// --------------------------------------------------------
// SET Right TITLE
// Add a title right of secondary axis
// --------------------------------------------------------
void TChartForm::SetRightTitle( AnsiString str ) {
   theChart->RYAxisLabel = str ;
}


// --------------------------------------------------------
// SET BOTTOM TITLE
// Add a title under the chart (under x-axis)
// --------------------------------------------------------
void TChartForm::SetBottomTitle( AnsiString str ) {
   theChart->XAxisLabel = str ;
}


// --------------------------------------------------------
// SETGAPS
// Adjust the margins around the chart to fit its hosting window
// --------------------------------------------------------
void TChartForm::SetGaps( int top, int left , int bottom , int right ) {

}


// --------------------------------------------------------
// SETDEFAULTPOSITION
// Set the default size and position of the form itself
// This is used when the chart is called to Minimize.
// --------------------------------------------------------
void TChartForm::SetDefaultPosition( int x, int y, int w, int h ) {
   defaultWidth = w ;
   defaultHeight = h ;
   defaultX = x ;
   defaultY = y ;
   SetPosition( x, y, w, h ) ;
}


// --------------------------------------------------------
// SELFPOSITION
// This is called to the chart to readjust itself.
// For example, when the user wants to display only tiltmeters
// this will be called to resize accordingly the window as
// well as tiling it or stacking it up.
// index means the window index (1 to 4) and kind means
// it is tiled or stacked up.
// --------------------------------------------------------
void TChartForm::SelfPosition( int index, bool kind ) {

   int offsetX = MainToolBar->Left ;
   int offsetY = MainToolBar->Top+MainToolBar->Height ;
   canUpdateMinimizeButton = false ;

   switch( index ) {
      case TILT1:
         Top    = offsetY+1 ;
         Left   = offsetX ;
         break ;

      case TILT2:
         if( kind == TILE )
            Top    = offsetY+1 ;
         else
            Top    = offsetY + CHARTHEIGHT / 4 + 1 ;

         if( kind == TILE )
            Left   = offsetX + CHARTRIGHT / 2 ;
         else
            Left   = offsetX ;
         break ;

      case TILT1TAN:
         Top    = offsetY + CHARTHEIGHT / 2 + 1 ;
         Left   = offsetX ;
         break ;

      case TILT2TAN:
         if( kind == TILE )
            Top    = offsetY + CHARTHEIGHT / 2 + 1 ;
         else
            Top    = offsetY + 3*CHARTHEIGHT / 4 + 1 ;

         if( kind == TILE )
            Left   = offsetX + CHARTRIGHT / 2  ;
         else
            Left   = offsetX ;
         break ;
   }

   if( kind == TILE )
      Width  = CHARTRIGHT / 2 - 1 ;
   else
      Width  = CHARTRIGHT ;

   if( kind == TILE )
      Height = CHARTHEIGHT / 2 - 1 ;
   else
      Height = CHARTHEIGHT / 4 - 1 ;

   SetFocus() ;
   canUpdateMinimizeButton = true ;
}


// --------------------------------------------------------
// MINIMIZE
// This means the window returns to its default size and position
// --------------------------------------------------------
void TChartForm::Minimize() {
   Width  = defaultWidth ;
   Height = defaultHeight ;
   Top    = defaultY ;
   Left   = defaultX ;
   maximized = 0 ;
   MainToolBar->MinimizeButton->Enabled = false ;
   MainToolBar->MaximizeButton->Enabled = true  ;
}


// --------------------------------------------------------
// MAXIMIZE
// Maximize the window to maximum screen area
// --------------------------------------------------------
void TChartForm::Maximize() {
   Top    = MainToolBar->Top+MainToolBar->Height+1 ;
   Left   = MainToolBar->Left ;
   Width  = CHARTRIGHT ;
   Height = CHARTHEIGHT ;
   maximized = 1 ;
   MainToolBar->MinimizeButton->Enabled = true  ;
   MainToolBar->MaximizeButton->Enabled = false ;
   SetFocus() ;
}


// --------------------------------------------------------
// HIDECHART
// Hide this window
// --------------------------------------------------------
void TChartForm::HideChart() {
   Visible = false ;
}


// --------------------------------------------------------
// SHOWCHART
// Makes this window visible
// --------------------------------------------------------
void TChartForm::ShowChart() {
   Visible = true ;
}


// --------------------------------------------------------
// SETID
// Gives this chart an ID
// --------------------------------------------------------
void TChartForm::SetID( int id ) {
   chartID = id ;
}


// --------------------------------------------------------
// FORMACTIVATE
// This is caleed when the user clicks on this
// form window. This is absolutely required if using the
// Maximize and Minimize buttons from the main tool bar.
// --------------------------------------------------------
void __fastcall TChartForm::FormActivate(TObject *Sender) {
   if( alreadyDisplayed )
      MainToolBar->SetCurrentWindow( chartID ) ;
}


// --------------------------------------------------------
// SETRANGES
// Set both axis extremums.
// First X and then Y and eventuelly RY if subsets = 2
// --------------------------------------------------------
void TChartForm::SetRanges(float xmin,  float xmax,
                           float ymin , float ymax,
                           float y2min, float y2max ) {
   theChart->ManualMinX = xmin ;
   theChart->ManualMaxX = xmax ;
   theChart->ManualMinY = ymin ;
   theChart->ManualMaxY = ymax ;
   if( nbSeries == 2 ){
      theChart->ManualMinRY = y2min ;
      theChart->ManualMaxRY = y2max ;
   }
   theChart->PEactions = sgReinitAndReset ;
}


// --------------------------------------------------------
// SetAxisTitle
// Set both axis title at once
// --------------------------------------------------------
void TChartForm::SetAxisTitle(char* xlabel, char* ylabel, char* y2label ) {
   SetLeftTitle( ylabel ) ;
   SetBottomTitle( xlabel ) ;
   if( nbSeries == 2 )
      SetRightTitle( y2label ) ;
   theChart->PEactions = sgReinitAndReset ;
}


// --------------------------------------------------------
// FORMRESIZE
// Respond to a resize event by updating the maximize
// and minimize buttons.
// --------------------------------------------------------
void __fastcall TChartForm::FormResize(TObject *Sender) {
   // Adjust chart size
   if( alreadyDisplayed  ) {
      if( canUpdateMinimizeButton )
         MainToolBar->MinimizeButton->Enabled = true ;
      MainToolBar->MaximizeButton->Enabled = true ;
   }
   else {
      MainToolBar->MinimizeButton->Enabled = false ;
      alreadyDisplayed = true ;
   }


   // Also update the font sizes of the chart
   // It will be medium and Zoom factor 1 if larger than 400x300
   // or large and zoom factor 1.5 if smaller
   if( (Width>=(CHARTRIGHT / 2 - 1)) &&
       (Height>=(CHARTHEIGHT / 2 - 1)) )
      theChart->FontSizeGlobalCntl  = 1 ;
   else
      theChart->FontSizeGlobalCntl  = 1.5 ;

   // Set window status
   maximized = 2 ;

}


// --------------------------------------------------------
// UPDATECHART
// Called to force a chart refresh
// --------------------------------------------------------
void TChartForm::UpdateChart() {
   theChart->PEactions = sgReinitAndReset ;
}



// --------------------------------------------------------
// UPDATEDATA
// Called to read in data from files and update the chart
// content.
// If xRef is true, we add both X and Y. If not, only Y.
// If newdata is true, we process all the files, otherwise
// only the new*.dat ones.
// --------------------------------------------------------
void TChartForm::UpdateData( bool xRef, bool newData ) {
   bool   y2Axis ;
   bool   readSomething = false ;

   AnsiString fileName ;

   // The file itself
   fstream inFile ;
   AnsiString mainPath ;
   mainPath = ExtractFilePath( Application->ExeName ) ;
   // Tell the chart that the first value added is the ManualMinX
   if( newData )
      firstPoint = false ;
   else
      firstPoint = true ;
   // Open the file
   switch( chartID ) {
      case TILT1:
      case TILT1TAN:
      case TILT1BATT:
         if( newData )
            fileName =  mainPath+"NewT1.dat" ;
         else
            fileName =  mainPath+"Tilt1.dat" ;
         break ;
      case TILT2:
      case TILT2TAN:
      case TILT2BATT:
         if( newData )
            fileName =  mainPath+"NewT2.dat" ;
         else
            fileName =  mainPath+"Tilt2.dat" ;
         break ;
      default:
         if( newData )
            fileName =  mainPath+"NewCra.dat" ;
         else
            fileName =  mainPath+"Crater.dat" ;
         break ;
   }
   inFile.open( fileName.c_str(), ios::in ) ;


   // Read file
   bool setFormula = true ;
   Configure->RegWorkWith( "Formulas" ) ;

   // Get the formulas from the dialog box
   vbatF  = Configure->GetRegString( "VBat", VBATFORMULA ) ;
   taF    = Configure->GetRegString( "Ta",  TAFORMULA   ) ;
   taF2   = Configure->GetRegString( "Ta2",  TAFORMULA   ) ;
   utF    = Configure->GetRegString( "Ut", UTFORMULA ) ;
   utF2   = Configure->GetRegString( "Ut2", UTFORMULA ) ;
   tf0F   = Configure->GetRegString( "TF0", TF0FORMULA ) ;
   tf0F2  = Configure->GetRegString( "TF02", TF0FORMULA ) ;
   e1F    = Configure->GetRegString( "E1", E1FORMULA ) ;
   e1F2   = Configure->GetRegString( "E2", E2FORMULA ) ;
   po2F   = Configure->GetRegString( "PO2", PO2FORMULA ) ;
   po2F2  = Configure->GetRegString( "PO22", PO2FORMULA ) ;
   coefF  = Configure->GetRegString( "Coef", COEFFORMULA ) ;
   coefF2 = Configure->GetRegString( "Coef2", COEFFORMULA ) ;
   inclF  = Configure->GetRegString( "Incl", INCLFORMULA ) ;
   inclF2 = Configure->GetRegString( "Incl2", INCLFORMULA ) ;

   // Prepare the parser for each formula
   vbatFPtr  = ucParse( vbatF.c_str() ) ;
   taFPtr    = ucParse( taF.c_str()   ) ;
   taFPtr2   = ucParse( taF2.c_str()   ) ;
   utFPtr    = ucParse( utF.c_str()   ) ;
   utFPtr2   = ucParse( utF2.c_str()   ) ;
   tf0FPtr   = ucParse( tf0F.c_str()  ) ;
   tf0FPtr2  = ucParse( tf0F2.c_str()  ) ;
   e1FPtr    = ucParse( e1F.c_str()   ) ;
   e1FPtr2   = ucParse( e1F2.c_str()   ) ;
   po2FPtr   = ucParse( po2F.c_str()  ) ;
   po2FPtr2  = ucParse( po2F2.c_str()  ) ;
   coefFPtr  = ucParse( coefF.c_str() ) ;
   coefFPtr2 = ucParse( coefF2.c_str() ) ;
   inclFPtr  = ucParse( inclF.c_str() ) ;
   inclFPtr2 = ucParse( inclF2.c_str() ) ;


   // Now the parsers are intialized and we can enter the loop
   // in order to read the available data and process them!
   char  theLine[255] ;
   while( !inFile.eof() && inFile.good() ) {
      inFile.getline( theLine, 255 ) ;
      if( strlen( theLine ) == 0 )
         break ;
      readSomething = true ;
      if( chartID == TILT1     || chartID == TILT2 ||
          chartID == TILT1TAN  || chartID == TILT2TAN ||
          chartID == TILT1BATT || chartID == TILT2BATT  )
         sscanf( theLine, "%d %d %d %d %d %d %d %d %d %d %d %d %d %d", &val1, &val2, &val3,
                  &val4, &val5, &val6, &val7, &val8,&val9, &Hour, &Minutes, &Day, &Month, &Year ) ;
      else
         sscanf( theLine, "%d %d %d %d %d %d %d %d %d %d %d %d %d", &val1, &val2, &val3,
                  &val4, &val5, &val6, &val7, &val8, &Hour, &Minutes, &Day, &Month, &Year ) ;

      // Now process the data
      // First create the time stuff. Then pass the data

      // Set a time X value
      TDateTime date( (unsigned short)Year, (unsigned short)Month, (unsigned short)Day ) ;
      TDateTime time( (unsigned short)Hour, (unsigned short)Minutes, 0, 0 ) ;
      xPoint = date+time ;
      y2Axis=false ;
      // Now calculate the points accordingly to chartType
      switch( chartID ) {

         case TILT1: // X Radial - Tilt files
            ucSetVariableValue( inclPtr, (float)val8 ) ;
            yPoint = ucEvaluate( inclFPtr ) ;
            break ;

         case TILT2: // X Radial - Tilt Files
            ucSetVariableValue( inclPtr2, (float)val8 ) ;
            yPoint = ucEvaluate( inclFPtr2 ) ;
            break ;



         case TILT1BATT: // Batt + Temp - Tilt Files
            ucSetVariableValue( vbatPtr, (float)val2 ) ;
            ucSetVariableValue( taPtr,  (float)val5 ) ;

            yPoint  = ucEvaluate( vbatFPtr ) ;
            yPoint2 = ucEvaluate( taFPtr ) ;
            y2Axis = true ;
            break ;


         case TILT2BATT: // Batt + Temp - Tilt Files
            ucSetVariableValue( vbatPtr, (float)val2 ) ;
            ucSetVariableValue( taPtr2, (float)val5 ) ;

            yPoint  = ucEvaluate( vbatFPtr ) ;
            yPoint2 = ucEvaluate( taFPtr2 ) ;
            y2Axis = true ;
            break ;



         case TILT1TAN: // Y Tan - Tilt Files
            ucSetVariableValue( inclPtr, (float)val6 ) ;
            yPoint  = ucEvaluate( inclFPtr ) ;
            break ;



         case TILT2TAN: // Y Tan - Tilt Files
            ucSetVariableValue( inclPtr2, (float)val6 ) ;
            yPoint  = ucEvaluate( inclFPtr2 ) ;
            break ;




         case TEMPFUM1: // TAmb + TFum - CRA Files
            ucSetVariableValue( taPtr,  (float)val4 ) ;
            yPoint = ucEvaluate( taFPtr ) ; // = Ta

            ucSetVariableValue( utPtr,  (float)val8 ) ;
            float ta = ucEvaluate( utFPtr ) ;

            ucSetVariableValue( tf0Ptr, ta ) ;
            y2Point = ucEvaluate( tf0Fptr ) ;
            y2Axis = true ;
            break ;

         case TEMPFUM2: // TAmb + TFum - CRA Files
            ucSetVariableValue( taPtr2,  (float)val4 ) ;
            yPoint = ucEvaluate( taFPtr2 ) ; // = Ta

            ucSetVariableValue( utPtr2,  (float)val6 ) ;
            float ta = ucEvaluate( utF2 ) ;

            ucSetVariableValue( tf0Ptr2, ta ) ;
            y2Point = ucEvaluate( tf0FPtr2 ) ;
            y2Axis = true ;
            break ;

         case PO2S1WIN: // PO2 - CRA Files
         case OXIDS1: // Batt + PO2 - CRA Files
            ucSetVariableValue( taPtr,  (float)val4 ) ;
            ucSetVariableValue( utPtr,  ucEvaluate( taFPtr ) ) ;
            ucSetVariableValue( tf0Ptr,  ucEvaluate( utFPtr ) ) ;
            ucSetVariableValue( e1Ptr,  (float)val7 ) ;
            yPoint = ucEvaluate( po2FPtr ) ; // = Ta
            y2Point = ucEvaluate( e1FPtr )  ;
            y2Axis = true ;
            break ;


         case PO2S2WIN: // PO2 - CRA Files
         case OXIDS2: // Batt + PO2 - CRA Files
            ucSetVariableValue( taPtr2,  (float)val4 ) ;
            ucSetVariableValue( utPtr2,  ucEvaluate( taFPtr2 ) ) ;
            ucSetVariableValue( tf0Ptr2,  ucEvaluate( utFPtr2 ) ) ;
            ucSetVariableValue( e1Ptr2,  (float)val7 ) ;
            yPoint = ucEvaluate( po2FPtr2 ) ; // = Ta
            y2Point = ucEvaluate( e1FPtr2 )  ;
            y2Axis = true ;
            break ;
      }


      // If firstpoint, keep it as the ManualMinX
      if( firstPoint ){
         theChart->ManualMinX =xPoint ;
         firstPoint = false ;
      }
      // Add the point now
      if( xRef ) {
         if( y2Axis )
            AddData( xPoint, (float)yPoint, (float)y2Point ) ;
         else
            AddData( xPoint, (float)yPoint ) ;
      }
      else {
         if( y2Axis )
            ChangeData( (float)yPoint, (float)y2Point ) ;
         else
            ChangeData( (float)yPoint ) ;
      }

   }
   inFile.close() ;
   if( readSomething ) {
      theChart->Points = pointIndex  ;
      // Update max X
      theChart->ManualMaxX = xPoint ;
      theChart->PEactions =  sgReinitAndReset ;
   }
}



// --------------------------------------------------------
// PRINTCHART
// Print the currently selected chart.
// First display the setup dialog from which the user will
// print.
// Also don't forget to set the background colors to white
// and reset them to original colors once done.
// This is to avoid switching to monochrome (white back-
// ground instead of gray one)... this way we keep the
// line colors available.
// --------------------------------------------------------
void TChartForm::PrintChart() {
   // Store colors
   TColor backColor = theChart->GraphBackColor ;
   TColor foreColor = theChart->GraphForeColor ;
   TColor deskColor = theChart->DeskColor ;
   TColor shadowColor = theChart->ShadowColor ;
   // Set background to white
   theChart->GraphBackColor = clWhite ;
   theChart->GraphForeColor = clBlack ;
   theChart->DeskColor      = clWhite ;
   theChart->ShadowColor    = clWhite ;
   PEnset((HWND)theChart->hObject, PEP_dwGRAPHBACKCOLOR, clWhite);
   PEnset((HWND)theChart->hObject, PEP_dwGRAPHFORECOLOR, clBlack);
   PEnset((HWND)theChart->hObject, PEP_dwDESKCOLOR, clWhite);
   PEnset((HWND)theChart->hObject, PEP_dwSHADOWCOLOR, clWhite);
   // ...and...
   PEnset((HWND)theChart->hObject, PEP_dwCGRAPHBACKCOLOR, clWhite);
   PEnset((HWND)theChart->hObject, PEP_dwCGRAPHFORECOLOR, clBlack);
   PEnset((HWND)theChart->hObject, PEP_dwCDESKCOLOR, clWhite);
   PEnset((HWND)theChart->hObject, PEP_dwCSHADOWCOLOR, clWhite);
   // Set title to printed chart
   theChart->SubTitle = Caption ;
   theChart->FontSize = sgSmall ;
   theChart->PEactions = sgReinitAndReset ;

   // Print it
   theChart->PEactions = sgPrintDialog ;

   // Remove title
   theChart->SubTitle = " " ;
   theChart->FontSize = sgMedium ;

   // Restore original colors
   theChart->GraphBackColor = backColor ;
   theChart->GraphForeColor = foreColor ;
   theChart->DeskColor = deskColor ;
   theChart->ShadowColor = shadowColor ;
   if( PEnget( (HWND)theChart->hObject, PEP_bCUSTOM ) == true ) {
      PEnset((HWND)theChart->hObject, PEP_dwCGRAPHBACKCOLOR, backColor  );
      PEnset((HWND)theChart->hObject, PEP_dwCGRAPHFORECOLOR, foreColor  );
      PEnset((HWND)theChart->hObject, PEP_dwCDESKCOLOR,      deskColor  );
      PEnset((HWND)theChart->hObject, PEP_dwCSHADOWCOLOR,    shadowColor);
   }

   PEreinitializecustoms( theChart ) ;
   theChart->PEactions = sgReinitAndReset ;
}


// --------------------------------------------------------
// CHECKXAXISLOG
// This is called when exiting the Configuration dialog box.
// It checks if the LOG check box has been changed and
// update the Axes accordingly if this is required.
// --------------------------------------------------------
void TChartForm::CheckAxisLog() {
   bool logStatus ;

   switch( chartID ) {
      case TILT1:
         Configure->RegWorkWith( "Scales\\Tilt1" ) ;
         break ;
      case TILT2:
         Configure->RegWorkWith( "Scales\\Tilt2" ) ;
         break ;
      case TILT1BATT:
         Configure->RegWorkWith( "Scales\\Tilt1Batt" ) ;
         break ;
      case TILT2BATT:
         Configure->RegWorkWith( "Scales\\Tilt2Batt" ) ;
         break ;
      case TILT1TAN:
         Configure->RegWorkWith( "Scales\\Tilt1Tan" ) ;
         break ;
      case TILT2TAN:
         Configure->RegWorkWith( "Scales\\Tilt1tan" ) ;
         break ;
      case TEMPFUM1:
         Configure->RegWorkWith( "Scales\\TempFum1" ) ;
         break ;
      case TEMPFUM2:
         Configure->RegWorkWith( "Scales\\TempFum2" ) ;
         break ;
      case PO2S1WIN:
         Configure->RegWorkWith( "Scales\\PO21" ) ;
         break ;
      case PO2S2WIN:
         Configure->RegWorkWith( "Scales\\PO22" ) ;
         break ;
      case OXIDS1:
         Configure->RegWorkWith( "Scales\\Oxid1" ) ;
         break ;
      case OXIDS2:
         Configure->RegWorkWith( "Scales\\Oxdi2" ) ;
         break ;
   }

   // Check X Axis
   logStatus = Configure->GetRegBool( "XLog", true ) ;
   if( ( (theChart->XAxisScaleControl == sgAuto) && logStatus ) ||
       ( (theChart->XAxisScaleControl == sgLog) && !logStatus   )    )
      theChart->XAxisScaleControl = logStatus ? sgLog : sgAuto ;

   // Check now Y
   logStatus = Configure->GetRegBool( "YLog", true ) ;
   if( ( (theChart->YAxisScaleControl == sgAuto) && logStatus ) ||
       ( (theChart->YAxisScaleControl == sgLog) && !logStatus   )    )
      theChart->YAxisScaleControl = logStatus ? sgLog : sgAuto ;

   // Finally check Y2
   if( ( chartID == TILT1BATT ) || ( chartID == TILT2BATT ) ||
       ( chartID == TEMPFUM1  ) || ( chartID == TEMPFUM2  ) ||
       ( chartID == PO2S1WIN  ) || ( chartID == PO2S2WIN  ) ||
       ( chartID == OXIDS1    ) || ( chartID == OXIDS2    )    ) {

      logStatus = Configure->GetRegBool( "Y2Log", true ) ;
      if( ( (theChart->RYAxisScaleControl == sgAuto) && logStatus ) ||
          ( (theChart->RYAxisScaleControl == sgLog) && !logStatus   )    )
         theChart->RYAxisScaleControl = logStatus ? sgLog : sgAuto ;
   }

}
