//  CLASS TCHARTFORM
//  This is the base class for any chart used in
//  the Tiltmeters application. All charting window
//  are derived from this one.
//
// (C) Steve JORDI - 1996-1999  
// steve@sjordi.com - www.sjordi.com
//
//  All this code is copyrighted by Steve Jordi and
//  cannot be reproduced without consent of the author.
//  The executable may be freely distributed.
//
//  THIS PROGRAM  IS  PROVIDED  "AS IS"  WITHOUT  WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED
//  TO  THE  IMPLIED  WARRANTIES  OF MERCHANTABILITY  AND FITNESS
//  FOR  A  PARTICULAR PURPOSE. THE ENTIRE RISK AS TO THE QUALITY
//  AND   PERFORMANCE  OF  THE  PROGRAM  IS  WITH YOU. SHOULD THE
//  PROGRAM  PROVE   DEFECTIVE,  YOU   (NOT  STEVE JORDI)  ASSUME
//  THE ENTIRE COST OF ALL  NECESSARY   SERVICING,   REPAIR,   OR
//  CORRECTION. IN NO EVENT WILL STEVE JORDI BE LIABLE TO YOU FOR
//  ANY  DAMAGES ,  INCLUDING ANY LOST PROFITS,  LOST SAVINGS, OR
//  OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES  ARISING OUT OF THE
//  USE OR INABILITY TO USE  SUCH  PROGRAM  EVEN  IF  STEVE JORDI
//  HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES,  OR FOR
//  ANY  CLAIM BY ANY OTHER PARTY.
//---------------------------------------------------------------------------
#ifndef ChartH
#define ChartH
//---------------------------------------------------------------------------
#include <vcl\Classes.hpp>
#include <vcl\Controls.hpp>
#include <vcl\StdCtrls.hpp>
#include <vcl\Forms.hpp>
#include <vcl\OleCtrls.hpp>

// Include the Gigasoft Charting tool definitions
#include "Pesgvcl.hpp"

// Include the configuration class
#include "Config.h"

// State of the charts
#define  TILE        true
#define  HORIZ       false

// Define screen area available to charts
#define  CHARTHEIGHT 489
#define  CHARTBOTTOM 600
#define  CHARTRIGHT  800

#define  DEFAULTPOINTS  15000
//---------------------------------------------------------------------------
class TChartForm : public TForm
{
   __published:   // IDE-managed Components
   PESGraph *theChart;

   // variables to be used
   float vbat, ta, ut, tf0, e1, po2, coef, incl ;
   long vbatPtr, taPtr, utPtr, tf0Ptr, e1Ptr, po2Ptr, coefPtr, inclPtr ;

   // variables holding the formulas
   String vbatF, taF, utF, tf0F, e1F, po2F, coefF, inclF ;
   String vbatF2, taF2, utF2, tf0F2, e1F2, po2F2, coefF2, inclF2 ;
   long vbatFPtr, taFPtr, utFPtr, tf0FPtr, e1FPtr, po2FPtr, coefFPtr, inclFPtr ;
   long vbatFPtr2, taFPtr2, utFPtr2, tf0FPtr2, e1FPtr2, po2FPtr2, coefFPtr2, inclFPtr2 ;
   long  timeVar = 0 ;
   float dataVBat, dataTa, dataUt, dataTf0, dataE1, dataPo2, dataCoef, dataIncl ;



   // Events
   void __fastcall FormActivate(TObject *Sender);
   void __fastcall FormResize(TObject *Sender);


   private: // User declarations
      int   nbPoints ;     // Nb points in chart
      int   pointIndex ;   // Current point index
      int   currentSerie ; // Current active serie
      int   nbSeries ;     // Nb of series in a chart
      int   defaultX,
            defaultY,
            defaultWidth,
            defaultHeight ; // Default window position
      TComponent* parent ;
      int   chartID ;
      int   maximized ; // 0=minimized, 1=maximized, 2=custom size
      bool  alreadyDisplayed ;
      bool  canUpdateMinimizeButton ;
      double xPoint ;
      double yPoint, y2Point ;
      TConfigure* Configure ;


      bool  firstPoint ;
      // Now declare dummy variables which will be used
      // to read data from files
      // We name them val1, val2, etc... and they will just
      // be equivalent to
      // Tilt: vbat2, vbat1, tElec, TJct, ...
      // REL : vbat, H, M, DD, MM, YY
      // CRA : vbat2, vbat1, tElec, TJct, ...
      int   val1, val2, val3, val4, val5, val6, val7, val8, val9 ;
      int   Hour, Minutes, Day, Month, Year ;

   public:     // User declarations
      __fastcall TChartForm(TComponent* Owner);

      void  SetCaption( AnsiString ) ;
      void  SetPosition( int, int, int, int ) ;
      void  SetNbPoints( int ) ;
      void  SetNbSeries( int ) ;
      void  SetSerie(    int ) ;
      void  AddHiddenPoint( int ) ;
      void  AddData( double, float ) ;
      void  AddData( double, float, float ) ;
      void  ChangeData( float ) ;
      void  ChangeData( float, float ) ;
      void  PrepareDataUpdate() ;
      void  UpdateChart() ;
      void  SetTopTitle( AnsiString ) ;
      void  SetLeftTitle( AnsiString ) ;
      void  SetRightTitle( AnsiString ) ;
      void  SetBottomTitle( AnsiString ) ;
      void  SetGaps( int, int, int, int ) ;
      void  SetDefaultPosition( int, int, int, int ) ;
      void  SelfPosition( int, bool ) ; // false = tile, true = horiz
      void  Minimize() ;
      void  Maximize() ;
      void  HideChart() ;
      void  ShowChart() ;
      void  SetID( int ) ;
      void  SetRanges(float, float, float, float, float y2min=0, float y2max = 0 ) ;
      void  SetAxisTitle(char*, char*, char* rtitle = "" ) ;
      void  UpdateData( bool, bool newData = false ) ;
      void  PrintChart() ;
      void  CheckAxisLog() ;
};
//---------------------------------------------------------------------------
extern TChartForm *ChartForm;
//---------------------------------------------------------------------------
#endif
