//  CLASS TCONFIGURE
//  This is the base class configuring Titlmeters.
//  It handles everything the user has control on, such as the
//  scales for each chart axis, alarm levels for battery and
//  temperatures, as well as the formulas used in the calculation,
//  and when the program should pass the hand to the acquisition
//  one.
//
// (C) Steve JORDI - 1996-1999  
// steve@sjordi.com - www.sjordi.com
//
// All this code is copyrighted by Steve Jordi and
// cannot be reproduced without the consent of the author.
// The executable may be freely distributed.
//
// MAKE  SURE  TO  UNDERSTAND  AND  ACCEPT  THE STATEMENTS BELOW
// BEFORE USING THE SOFTWARE:
//
// THIS PROGRAM  IS  PROVIDED  "AS IS"  WITHOUT  WARRANTY OF ANY
// KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED
// TO  THE  IMPLIED  WARRANTIES  OF MERCHANTABILITY  AND FITNESS
// FOR  A  PARTICULAR PURPOSE. THE ENTIRE RISK AS TO THE QUALITY
// AND   PERFORMANCE  OF  THE  PROGRAM  IS  WITH YOU. SHOULD THE
// PROGRAM  PROVE   DEFECTIVE,  YOU   (NOT  STEVE JORDI)  ASSUME
// THE ENTIRE COST OF ALL  NECESSARY   SERVICING,   REPAIR,   OR
// CORRECTION. IN NO EVENT WILL STEVE JORDI BE LIABLE TO YOU FOR
// ANY  DAMAGES ,  INCLUDING ANY LOST PROFITS,  LOST SAVINGS, OR
// OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES  ARISING OUT OF THE
// USE OR INABILITY TO USE  SUCH  PROGRAM  EVEN  IF  STEVE JORDI
// HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES,   OR FOR
// ANY  CLAIM BY ANY OTHER PARTY.
// ALSO, USE IT IN LIFE THREATENING SITUATIONS AT YOUR OWN RISKS
// (SUCH AS VOLCANO MONITORING). STEVE JORDI CANNOT BE ACCOUNTED
// AS RESPONSIBLE FOR ANY LIFE OR PROPERTY LOST IN CASE AN ALARM
// SYSTEM HAD A MALFUNCTION OR WAS NOT ABLE TO CORRECTLY  BEHAVE
// IN A CRISIS ENVIRONMENT.
//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop

#include "Config.h"
#include "DirSel.h"
#include "Password.h"
#include "ChPwd.h"
#include "Emails.h"
#include <stdio.h>
//---------------------------------------------------------------------------
#pragma link "Parser10"
#pragma link "marsCap"
#pragma resource "*.dfm"
TConfigure *Configure;

// --------------------------------------------------------
// CONSTRUCTOR
// Initialize variables here
// --------------------------------------------------------
__fastcall TConfigure::TConfigure(TComponent* Owner)
   : TForm(Owner) {
   passwordOK = false ;
   theRegistry = new TRegistry() ;
   theRegistry->RootKey = HKEY_LOCAL_MACHINE ;

   RegWorkWith( "Updating" ) ;

   theRegistry->OpenKey("\\Software\\SteveWare\\Tiltmeters\\Directories", false ) ;
   CRADirButton->Caption   = GetRegString( "CRA", "Default (current dir)" ) ;
   Tilt1DirButton->Caption = GetRegString( "TILT1", "Default (current dir)" ) ;
   Tilt2DirButton->Caption = GetRegString( "TILT2", "Default (current dir)" ) ;
   CraterPrefix->Text = GetRegString( "CraterPrefix", "CR" ) ;
   Tilt1Prefix->Text  = GetRegString( "Tilt1Prefix", "T1" ) ;
   Tilt2Prefix->Text  = GetRegString( "Tilt2Prefix", "T2" ) ;

   // Retrieve updating info
   theRegistry->OpenKey("\\Software\\SteveWare\\Tiltmeters\\Updating", false ) ;
   char toto[255] ;
   strcpy( toto, IntToStr( GetRegInteger( "Hour", 1 ) ).c_str() ) ;
   HourEdit->Text = IntToStr( GetRegInteger( "Hour", 1 ) ) ;
   MinuteEdit->Text = IntToStr( GetRegInteger( "Minute", 0 ) ) ;
//   OffsetEdit->Text = IntToStr( GetRegInteger( "Offset", 0 ) ) ;
   ExternalUpdateCheck->Checked = GetRegBool( "UseExternal", false ) ;
   if( ExternalUpdateCheck->Checked )
      UpdateButton->Caption = GetRegString( "ExternalApplication", "none" ) ;

   // Retrieve Alarms information
   // Temp check boxes
   theRegistry->OpenKey("\\Software\\SteveWare\\Tiltmeters\\Alarms", false ) ;
   TempFum1Chk->Checked  = GetRegBool( "TempFum1",  false ) ;
   TempFum2Chk->Checked  = GetRegBool( "TempFum2",  false ) ;
   TempAmb1Chk->Checked  = GetRegBool( "TempAmb1",  false ) ;
   TempAmb2Chk->Checked  = GetRegBool( "TempAmb2",  false ) ;
   TempTilt1Chk->Checked = GetRegBool( "TempTilt1", false ) ;
   TempTilt2Chk->Checked = GetRegBool( "TempTilt2", false ) ;
   // Maximum temperature values
   MaxTempFum1->Text  = GetRegString( "MaxTempFum1",  "100" ) ;
   MaxTempFum2->Text  = GetRegString( "MaxTempFum2",  "100" ) ;
   MaxTempAmb1->Text  = GetRegString( "MaxTempAmb1",  "100" ) ;
   MaxTempAmb2->Text  = GetRegString( "MaxTempAmb2",  "100" ) ;
   MaxTempTilt1->Text = GetRegString( "MaxTempTilt1", "100" ) ;
   MaxTempTilt2->Text = GetRegString( "MaxTempTilt2", "100" ) ;
   // Batteries check box
   BattTilt1Chk->Checked = GetRegBool( "BattTilt1", false ) ;
   BattTilt2Chk->Checked = GetRegBool( "BattTilt2", false ) ;
   BattOxid1Chk->Checked = GetRegBool( "BattOxid1", false ) ;
   BattOxid2Chk->Checked = GetRegBool( "BattOxid2", false ) ;
   // Batteries minimum levels
   MinBattTilt1->Text = GetRegString( "MinBattTilt1", "11" ) ;
   MinBattTilt2->Text = GetRegString( "MinBattTilt2", "11" ) ;
   MinBattOxid1->Text = GetRegString( "MinBattOxid1", "11" ) ;
   MinBattOxid2->Text = GetRegString( "MinBattOxid2", "11" ) ;
   MaxBattTilt1->Text = GetRegString( "MaxBattTilt1", "18" ) ;
   MaxBattTilt2->Text = GetRegString( "MaxBattTilt2", "18" ) ;
   MaxBattOxid1->Text = GetRegString( "MaxBattOxid1", "18" ) ;
   MaxBattOxid2->Text = GetRegString( "MaxBattOxid2", "18" ) ;
   // Email notifications?
   SendEmailCheckBox->Checked = GetRegBool( "EmailNotification", false ) ;


   // Read formulas back
   theRegistry->OpenKey("\\Software\\SteveWare\\Tiltmeters\\Formulas", false ) ;
   // Drill 1
   VBATFormula->Text = GetRegString( "VBat", VBATFORMULA ) ;
   TAFormula->Text   = GetRegString( "Ta",   TAFORMULA   ) ;
   UTFormula->Text   = GetRegString( "Ut",   UTFORMULA   ) ;
   TFOFormula->Text  = GetRegString( "TF0",  TF0FORMULA  ) ;
   EFormula->Text    = GetRegString( "E",    E1FORMULA   ) ;
   PO2Formula->Text  = GetRegString( "PO2",  PO2FORMULA  ) ;
   InclFormula->Text = GetRegString( "Incl", INCLFORMULA ) ;
   CoefFormula->Text = GetRegString( "Coef", COEFFORMULA ) ;
   // Drill 2
   VBATFormula2->Text = GetRegString( "VBat2", VBATFORMULA ) ;
   TAFormula2->Text   = GetRegString( "Ta2",   TAFORMULA   ) ;
   UTFormula2->Text   = GetRegString( "Ut2",   UTFORMULA   ) ;
   TFOFormula2->Text  = GetRegString( "TF02",  TF0FORMULA  ) ;
   EFormula2->Text    = GetRegString( "E2",    E2FORMULA   ) ;
   PO2Formula2->Text  = GetRegString( "PO22",  PO2FORMULA  ) ;
   InclFormula2->Text = GetRegString( "Incl2", INCLFORMULA ) ;
   CoefFormula2->Text = GetRegString( "Coef2", COEFFORMULA ) ;

   // Store scales by windows
   // Get scale button selection
   theRegistry->OpenKey("\\Software\\SteveWare\\Tiltmeters\\Scales\\", false ) ;
   currentScaleSelection = GetRegInteger( "CurrentSelection", TILT1 ) ;
   GetOptions( currentScaleSelection ) ;
   UpdateXScaleControls() ;
   UpdateYScaleControls() ;
   UpdateY2ScaleControls() ;
   // We just make the last button pushed look down.
   switch( currentScaleSelection ) {
      case TILT1:
         Tilt1RadCfg->Down = true ;
         break ;
      case TILT2:
         Tilt2RadCfg->Down = true ;
         break ;
      case TILT1BATT:
         Tilt1TempCfg->Down = true ;
         EnableY2Axis() ;
         break ;
      case TILT2BATT:
         Tilt2TempCfg->Down = true ;
         EnableY2Axis() ;
         break ;
      case TILT1TAN:
         Tilt1TanCfg->Down = true ;
         break ;
      case TILT2TAN:
         Tilt2TanCfg->Down = true ;
         break ;
      case TEMPFUM1:
         Fum1Cfg->Down = true ;
         EnableY2Axis() ;
         break ;
      case TEMPFUM2:
         Fum2Cfg->Down = true ;
         EnableY2Axis() ;
         break ;
      case PO2S1WIN:
         PO21Cfg->Down = true ;
         break ;
      case PO2S2WIN:
         PO22Cfg->Down = true ;
         break ;
      case OXIDS1:
         Oxid1Cfg->Down = true ;
         EnableY2Axis() ;
         break ;
      case OXIDS2:
         Oxid2Cfg->Down = true ;
         EnableY2Axis() ;
         break ;
   }
}



// --------------------------------------------------------
// CRADIRBUTTONCLICK
// Respond to a click on the CRA station directory button
// --------------------------------------------------------
void __fastcall TConfigure::CRADirButtonClick(TObject *Sender) {
   // Display dialog box
   DirSelectForm->ShowModal() ;
   // Respond to an OK by getting the directory and displaying
   // it into the button's caption
   if(DirSelectForm->ModalResult == mrOk)
      CRADirButton->Caption = DirSelectForm->DirectoryListBox->Directory ;
}



// --------------------------------------------------------
// TILT1DIRBUTTONCLICK
// Respond to a click on the TILT1 station directory button
// --------------------------------------------------------
void __fastcall TConfigure::Tilt1DirButtonClick(TObject *Sender) {
   // Display the dialog box
   DirSelectForm->ShowModal() ;
   // Respond to an OK by getting the directory and displaying
   // it into the button's caption
   if(DirSelectForm->ModalResult == mrOk)
      Tilt1DirButton->Caption = DirSelectForm->DirectoryListBox->Directory ;
}


// --------------------------------------------------------
// TILT2DIRBUTTONCLICK
// Respond to a click on the TILT2 station directory button
// --------------------------------------------------------
void __fastcall TConfigure::Tilt2DirButtonClick(TObject *Sender) {
   // Display dialog box
   DirSelectForm->ShowModal() ;
   // Respond to an OK by getting the directory and displaying
   // it into the button's caption
   if(DirSelectForm->ModalResult == mrOk)
      Tilt2DirButton->Caption = DirSelectForm->DirectoryListBox->Directory ;
}


// --------------------------------------------------------
// CHANGEPASSWORDBUTTONCLICK
// The user wants to change the password to access the
// formula page.
// --------------------------------------------------------
void __fastcall TConfigure::ChangePasswordButtonClick(TObject *Sender) {
   AnsiString pwd ;
   // Pass old password to the dialog for comparison
   AnsiString comparePwd ;
   comparePwd = GetRegString( "password", "TECH" ) ;
   ChPwdDialog->SetPreviousPassword( comparePwd ) ;
   // Display change password dialog box
   ChPwdDialog->ShowModal() ;
   // Process result
   if(ChPwdDialog->ModalResult == mrOk) {
      // Get new password and store it
      pwd = ChPwdDialog->GetNewPassword() ;
      if( pwd != "" ) {
         theRegistry->OpenKey("\\Software\\SteveWare\\Tiltmeters\\Password", true ) ;
         SetRegString( "password", pwd  ) ;
         MessageBox( 0, "Password succesfully changed", "Password OK", MB_OK ) ;
      }
   }
}

// --------------------------------------------------------
// CONFIGPAGECONTROLCHANGE
// This is called when the user clicks on the Formula
// sheet. It requires a password. If not already done,
// ask for it and let the user acces it only if password
// is correct. If not, switch to the DirectorySheet
// --------------------------------------------------------
void __fastcall TConfigure::ConfigPageControlChange(TObject *Sender) {
   // UPdate the TUpDown size
   HourUpDown->Width = 25 ;
   MinuteUpDown->Width = 25 ;
// OffsetUpDown->Width = 25 ;
   
   // Get current tab sheet in case the user doesn't know the password
   // to access the formulas
   TTabSheet *current = ConfigPageControl->ActivePage ;
   // If we try to access the formula sheet...
   if( ( current == FormulaSheet ) && !passwordOK ) {
      // Ask for password
      PasswordAccess->ShowModal() ;
      // Is password correct? Yes, display the dialog
      if(PasswordAccess->ModalResult == mrOk) {
         // Compare password to the one from registry
         theRegistry->OpenKey("\\Software\\SteveWare\\Tiltmeters\\Password", false ) ;
         AnsiString password = GetRegString( "password", "TECH" ) ;
         if( PasswordAccess->PwdEdit->Text != password )
            ConfigPageControl->ActivePage = DirectorySheet ;
         else
            passwordOK = true ;
      }
      else {
         // Password is wrong, switch back to current sheet.
         passwordOK = false ;
         ConfigPageControl->ActivePage = DirectorySheet ;
      }
   }
}



// --------------------------------------------------------
// FORMSHOW
// Initialize stuff when the form is displayed.
// --------------------------------------------------------
void __fastcall TConfigure::FormShow(TObject *Sender) {
   passwordOK  = false ;
   VBat1Dirty  = false ;
   VBat2Dirty  = false ;
   TA1Dirty    = false ;
   TA2Dirty    = false ;
   UT1Dirty    = false ;
   UT2Dirty    = false ;
   TF01Dirty   = false ;
   TF02Dirty   = false ;
   E1Dirty     = false ;
   E2Dirty     = false ;
   PO21Dirty   = false ;
   PO22Dirty   = false ;
   Coeff1Dirty = false ;
   Coeff2Dirty = false ;
   Incl1Dirty  = false ;
   Incl2Dirty  = false ;
   UpdateMinutes() ;
   UpdateHours() ;
}


// --------------------------------------------------------
// FORMCLOSE
// Handle the close message by just switching back to
// the directory sheet to be sure that the next time you
// display the configuration dialog box, you won't find
// yourself on the formula sheet if that was the last
// one you accessed, leaving it without password protection.
// --------------------------------------------------------
void __fastcall TConfigure::FormClose(TObject *Sender, TCloseAction &Action) {
   ConfigPageControl->ActivePage = DirectorySheet ;
}



// --------------------------------------------------------
// UPDOWN1CLICK
// User modifees the hour field for automation
// --------------------------------------------------------
void __fastcall TConfigure::HourUpDownClick(TObject *Sender, TUDBtnType Button) {
   UpdateHours() ;
}


// --------------------------------------------------------
// UPDOWN2CLICK
// User modifies the minutes field for automation
// --------------------------------------------------------
void __fastcall TConfigure::MinuteUpDownClick(TObject *Sender, TUDBtnType Button) {
   UpdateMinutes() ;
}



// --------------------------------------------------------
// UPDATEHOURS
// User has changed the hour. Update the info in the
// explanation of when the updates will take place
// --------------------------------------------------------
void  TConfigure::UpdateHours() {
   char buffer[4] ;
   AnsiString hourStr = HourEdit->Text ;
   int hour = hourStr.ToInt() ; // + OffsetEdit->Text.ToInt() ;
   Hour1->Caption = "00" ;
   sprintf( buffer, "%02d", hour%24 ) ;
   Hour2->Caption = buffer ;
   sprintf( buffer, "%02d", (2*hour)%24 ) ;
   Hour3->Caption = buffer ;
   sprintf( buffer, "%02d", (3*hour)%24 ) ;
   Hour4->Caption = buffer ;
}

// --------------------------------------------------------
// UPDATEMINUTES
// User has changed the minutes. Update the info in the
// explanation of when the updates will take place
// --------------------------------------------------------
void  TConfigure::UpdateMinutes() {
   char buffer[4] ;
   AnsiString minuteStr = MinuteEdit->Text ;
   int minute = minuteStr.ToInt()  ;
   sprintf( buffer, "%02d", minute ) ;
   Minute1->Caption = buffer ;
   Minute2->Caption = buffer ;
   Minute3->Caption = buffer ;
   Minute4->Caption = buffer ;
}


// --------------------------------------------------------
// TEMPFUM1CHKCLICK
// The user has clicked on the TEMPFUM1 icon. Display
// appropriate information on the right panel of the scale
// dialog box.
// --------------------------------------------------------
void __fastcall TConfigure::TempFum1ChkClick(TObject *Sender) {
   if( TempFum1Chk->Checked )
      MaxTempFum1->Enabled = true ;
   else
      MaxTempFum1->Enabled = false ;
}


// --------------------------------------------------------
// TEMPFUM2CHKCLICK
// The user has clicked on the TEMPFUM2 icon. Display
// appropriate information on the right panel of the scale
// dialog box.
// --------------------------------------------------------
void __fastcall TConfigure::TempFum2ChkClick(TObject *Sender) {
   if( TempFum2Chk->Checked )
      MaxTempFum2->Enabled = true ;
   else
      MaxTempFum2->Enabled = false ;
}


// --------------------------------------------------------
// TEMPAMB1CHKCLICK
// The user has clicked on the TEMPAMB1 icon. Display
// appropriate information on the right panel of the scale
// dialog box.
// --------------------------------------------------------
void __fastcall TConfigure::TempAmb1ChkClick(TObject *Sender) {
   if( TempAmb1Chk->Checked )
      MaxTempAmb1->Enabled = true ;
   else
      MaxTempAmb1->Enabled = false ;
}


// --------------------------------------------------------
// TEMPAMB2CHKCLICK
// The user has clicked on the TEMPAMB2 icon. Display
// appropriate information on the right panel of the scale
// dialog box.
// --------------------------------------------------------
void __fastcall TConfigure::TempAmb2ChkClick(TObject *Sender) {
   if( TempAmb2Chk->Checked )
      MaxTempAmb2->Enabled = true ;
   else
      MaxTempAmb2->Enabled = false ;
}


// --------------------------------------------------------
// TEMPTILT1CHKCLICK
// The user has clicked on the TEMPTILT1 icon. Display
// appropriate information on the right panel of the scale
// dialog box.
// --------------------------------------------------------
void __fastcall TConfigure::TempTilt1ChkClick(TObject *Sender) {
   if( TempTilt1Chk->Checked )
      MaxTempTilt1->Enabled = true ;
   else
      MaxTempTilt1->Enabled = false ;
}


// --------------------------------------------------------
// TEMPTILT2CHKCLICK
// The user has clicked on the TEMPTILT2 icon. Display
// appropriate information on the right panel of the scale
// dialog box.
// --------------------------------------------------------
void __fastcall TConfigure::TempTilt2ChkClick(TObject *Sender) {
   if( TempTilt2Chk->Checked )
      MaxTempTilt2->Enabled = true ;
   else
      MaxTempTilt2->Enabled = false ;
}


// --------------------------------------------------------
// BATTTILT1CHKCLICK
// The user has clicked on the BATTTILT1 icon. Display
// appropriate information on the right panel of the scale
// dialog box.
// --------------------------------------------------------
void __fastcall TConfigure::BattTilt1ChkClick(TObject *Sender) {
   if( BattTilt1Chk->Checked )
      MinBattTilt1->Enabled = true ;
   else
      MinBattTilt1->Enabled = false ;
}


// --------------------------------------------------------
// BATTTILT2CHKCLICK
// The user has clicked on the BATTTILT2 icon. Display
// appropriate information on the right panel of the scale
// dialog box.
// --------------------------------------------------------
void __fastcall TConfigure::BattTilt2ChkClick(TObject *Sender) {
   if( BattTilt2Chk->Checked )
      MinBattTilt2->Enabled = true ;
   else
      MinBattTilt2->Enabled = false ;
}


// --------------------------------------------------------
// BATTOXID1CHKCLICK
// The user has clicked on the BATTOXID1 icon. Display
// appropriate information on the right panel of the scale
// dialog box.
// --------------------------------------------------------
void __fastcall TConfigure::BattOxid1ChkClick(TObject *Sender) {
   if( BattOxid1Chk->Checked )
      MinBattOxid1->Enabled = true ;
   else
      MinBattOxid1->Enabled = false ;
}


// --------------------------------------------------------
// BATTOXID2CHKCLICK
// The user has clicked on the BATTOXID2 icon. Display
// appropriate information on the right panel of the scale
// dialog box.
// --------------------------------------------------------
void __fastcall TConfigure::BattOxid2ChkClick(TObject *Sender) {
   if( BattOxid2Chk->Checked )
      MinBattOxid2->Enabled = true ;
   else
      MinBattOxid2->Enabled = false ;
}


// --------------------------------------------------------
// TEMPALLONCLICK
// Enable all controls associated to the temperature windows.
// --------------------------------------------------------
void __fastcall TConfigure::TempAllOnClick(TObject *Sender) {
   TempFum1Chk->Checked = true  ;
   MaxTempFum1->Enabled = true  ;
   TempFum2Chk->Checked = true  ;
   MaxTempFum2->Enabled = true  ;
   TempAmb1Chk->Checked = true  ;
   MaxTempAmb1->Enabled = true  ;
   TempAmb2Chk->Checked = true  ;
   MaxTempAmb2->Enabled = true  ;
   TempTilt1Chk->Checked = true ;
   MaxTempTilt1->Enabled = true ;
   TempTilt2Chk->Checked = true ;
   MaxTempTilt2->Enabled = true ;
}


// --------------------------------------------------------
// TEMPALLONCLICK
// Disable all controls associated to the temperature windows.
// --------------------------------------------------------
void __fastcall TConfigure::TempAllOffClick(TObject *Sender) {
   TempFum1Chk->Checked = false  ;
   MaxTempFum1->Enabled = false  ;
   TempFum2Chk->Checked = false  ;
   MaxTempFum2->Enabled = false  ;
   TempAmb1Chk->Checked = false  ;
   MaxTempAmb1->Enabled = false  ;
   TempAmb2Chk->Checked = false  ;
   MaxTempAmb2->Enabled = false  ;
   TempTilt1Chk->Checked = false ;
   MaxTempTilt1->Enabled = false ;
   TempTilt2Chk->Checked = false ;
   MaxTempTilt2->Enabled = false ;
}


// --------------------------------------------------------
// BATTALLONCLICK
// Enable all controls associated to the batteries windows.
// --------------------------------------------------------
void __fastcall TConfigure::BattAllOnClick(TObject *Sender) {
   BattTilt1Chk->Checked = true ;
   MinBattTilt1->Enabled = true ;
   BattTilt2Chk->Checked = true ;
   MinBattTilt2->Enabled = true ;
   BattOxid1Chk->Checked = true ;
   MinBattOxid1->Enabled = true ;
   BattOxid2Chk->Checked = true ;
   MaxBattOxid2->Enabled = true ;
   MaxBattTilt1->Enabled = true ;
   MaxBattTilt2->Enabled = true ;
   MaxBattOxid1->Enabled = true ;
   MaxBattOxid2->Enabled = true ;
}

// --------------------------------------------------------
// BATTALLONCLICK
// Disable all controls associated to the batteries windows.
// --------------------------------------------------------
void __fastcall TConfigure::BattAllOffClick(TObject *Sender) {
   BattTilt1Chk->Checked = false ;
   MinBattTilt1->Enabled = false ;
   BattTilt2Chk->Checked = false ;
   MinBattTilt2->Enabled = false ;
   BattOxid1Chk->Checked = false ;
   MinBattOxid1->Enabled = false ;
   BattOxid2Chk->Checked = false ;
   MinBattOxid2->Enabled = false ;
   MaxBattOxid2->Enabled = false ;
   MaxBattTilt1->Enabled = false ;
   MaxBattTilt2->Enabled = false ;
   MaxBattOxid1->Enabled = false ;
   MaxBattOxid2->Enabled = false ;
}


// --------------------------------------------------------
// AUTOXAXISCLICK
// Display the X axis min and max or hide them depending
// on the status of current checkbox
// --------------------------------------------------------
void __fastcall TConfigure::AutoXAxisClick(TObject *Sender) {
      UpdateXScaleControls() ;
}

// --------------------------------------------------------
// AUTOYAXISCLICK
// Display the Y axis min and max or hide them depending
// on the status of current checkbox
// --------------------------------------------------------
void __fastcall TConfigure::AutoYAxisClick(TObject *Sender) {
      UpdateYScaleControls() ;
}


// --------------------------------------------------------
// AUTOZAXISCLICK
// Display the Z axis min and max or hide them depending
// on the status of current checkbox
// --------------------------------------------------------
void __fastcall TConfigure::AutoY2AxisClick(TObject *Sender) {
      UpdateY2ScaleControls() ;
}

// --------------------------------------------------------
// UPDATEXSCALECONTROLS
// This currently hide/display the controls for X axis
// --------------------------------------------------------
void TConfigure::UpdateXScaleControls() {
   // Enable/Disable controls
   LastEdit->Enabled       = !AutoXAxis->Checked ;
   RangeDropDown->Enabled  = !AutoXAxis->Checked ;
   LastEdit->Visible       = !AutoXAxis->Checked ;
   RangeDropDown->Visible  = !AutoXAxis->Checked ;

   LastLabel->Visible      = !AutoXAxis->Checked ;
   XUnits->Visible         = !AutoXAxis->Checked ;
}

// --------------------------------------------------------
// UPDATEYSCALECONTROLS
// This currently hide/display the controls for Y axis
// --------------------------------------------------------
void TConfigure::UpdateYScaleControls() {
   // Enable/Disable controls
   MinYScale->Enabled   = !AutoYAxis->Checked ;
   MaxYScale->Enabled   = !AutoYAxis->Checked ;
   // Show/Hide controls
   MinYScale->Visible   = !AutoYAxis->Checked ;
   MaxYScale->Visible   = !AutoYAxis->Checked ;
   YUnits->Visible      = !AutoYAxis->Checked ;
   MinYLegend->Visible  = !AutoYAxis->Checked ;
   MaxYLegend->Visible  = !AutoYAxis->Checked ;
}

// --------------------------------------------------------
// UPDATEY2SCALECONTROLS
// This currently hide/display the controls for secondary Y
// axis
// --------------------------------------------------------
void TConfigure::UpdateY2ScaleControls() {
   // Enable/Disable controls
   MinY2Scale->Enabled  = !AutoY2Axis->Checked ;
   MaxY2Scale->Enabled  = !AutoY2Axis->Checked ;
   // Show/Hide controls
   MinY2Scale->Visible  = !AutoY2Axis->Checked ;
   MaxY2Scale->Visible  = !AutoY2Axis->Checked ;
   Y2Units->Visible     = !AutoY2Axis->Checked ;
   MinY2Legend->Visible = !AutoY2Axis->Checked ;
   MaxY2Legend->Visible = !AutoY2Axis->Checked ;
}



// --------------------------------------------------------
// TILT1RADCFGCLICK
// When the user clicks on TILT1 RADIAL, show check boxes
// for both X and Y axis and hide Y2
// --------------------------------------------------------
void __fastcall TConfigure::Tilt1RadCfgClick(TObject *Sender) {
   // If we move from a different scale button, save it first
   if( currentScaleSelection != TILT1 )
      SaveOptions( currentScaleSelection ) ;

   // Now get configuration for current scale button.
   // Default values are Automatic scales for X and Y
   // and no log. Hide Y2
   GetOptions( TILT1 ) ;
   DisableY2Axis() ;
   UpdateXScaleControls() ;
   UpdateYScaleControls() ;
   currentScaleSelection      = TILT1 ;
   YUnits->Caption = "[-rad]" ;
}



// --------------------------------------------------------
// TILT1TEMPCFGCLICK
// When the user clicks on TILT1 TEMP, show check boxes
// for both X and Y axis and hide Y2
// --------------------------------------------------------
void __fastcall TConfigure::Tilt1TempCfgClick(TObject *Sender) {
   if( currentScaleSelection != TILT1BATT )
      SaveOptions( currentScaleSelection ) ;

   // Now get configuration for current scale button.
   // Default values are Automatic scales for X and Y
   // and no log. Hide Y2
   GetOptions( TILT1BATT ) ;
   EnableY2Axis() ;
   UpdateXScaleControls() ;
   UpdateYScaleControls() ;
   UpdateY2ScaleControls() ;
   YUnits->Caption = "[V]" ;
   Y2Units->Caption = "[C]" ;
   currentScaleSelection      = TILT1BATT ;
}


// --------------------------------------------------------
// AUTOMATIONSHEETENTER
// Called when the user enter the automation window
// Doesn't work. Tried to resize the <- and -> buttons.
// --------------------------------------------------------
void __fastcall TConfigure::AutomationSheetEnter(TObject *Sender) {
   HourUpDown->Width = 25 ;
   MinuteUpDown->Width = 25 ;
//   OffsetUpDown->Width = 25 ;
}



// --------------------------------------------------------
// OKBUTTONCLICK
// When the user quits the configuration window, store
// current parameters to the registry.
// --------------------------------------------------------
void __fastcall TConfigure::OKButtonClick(TObject *Sender) {
   // -- Directories --
   // Store the directories into registry
   theRegistry->OpenKey("\\Software\\SteveWare\\Tiltmeters\\Directories", true ) ;
   SetRegString( "CRA",   CRADirButton->Caption  ) ;
   SetRegString( "TILT1", Tilt1DirButton->Caption ) ;
   SetRegString( "TILT2", Tilt2DirButton->Caption ) ;
   SetRegString( "CraterPrefix", CraterPrefix->Text ) ;
   SetRegString( "Tilt1Prefix", Tilt1Prefix->Text ) ;
   SetRegString( "Tilt2Prefix", Tilt2Prefix->Text ) ;

   // -- Updating info --
   // Store updating info
   theRegistry->OpenKey("\\Software\\SteveWare\\Tiltmeters\\Updating", true ) ;
   SetRegInteger( "Hour", StrToInt(HourEdit->Text) ) ;
   SetRegInteger( "Minute", StrToInt(MinuteEdit->Text) ) ;
//   SetRegInteger( "Offset", StrToInt(OffsetEdit->Text) ) ;

   // Store external reference
   theRegistry->WriteBool( "UseExternal", ExternalUpdateCheck->Checked ) ;
   if( ExternalUpdateCheck->Checked )
      SetRegString( "ExternalApplication", UpdateButton->Caption ) ;

   // -- Alarms --
   // Temp check boxes
   theRegistry->OpenKey("\\Software\\SteveWare\\Tiltmeters\\Alarms", true ) ;
   theRegistry->WriteBool( "TempFum1",  TempFum1Chk->Checked  ) ;
   theRegistry->WriteBool( "TempFum2",  TempFum2Chk->Checked  ) ;
   theRegistry->WriteBool( "TempAmb1",  TempAmb1Chk->Checked  ) ;
   theRegistry->WriteBool( "TempAmb2",  TempAmb2Chk->Checked  ) ;
   theRegistry->WriteBool( "TempTilt1", TempTilt1Chk->Checked ) ;
   theRegistry->WriteBool( "TempTilt2", TempTilt2Chk->Checked ) ;
   // Maximum temperature values
   SetRegString( "MaxTempFum1",  MaxTempFum1->Text ) ;
   SetRegString( "MaxTempFum2",  MaxTempFum2->Text ) ;
   SetRegString( "MaxTempAmb1",  MaxTempAmb1->Text ) ;
   SetRegString( "MaxTempAmb2",  MaxTempAmb2->Text ) ;
   SetRegString( "MaxTempTilt1", MaxTempTilt1->Text ) ;
   SetRegString( "MaxTempTilt2", MaxTempTilt2->Text ) ;
   // Batteries check box
   theRegistry->WriteBool( "BattTilt1", BattTilt1Chk->Checked ) ;
   theRegistry->WriteBool( "BattTilt2", BattTilt2Chk->Checked ) ;
   theRegistry->WriteBool( "BattOxid1", BattOxid1Chk->Checked ) ;
   theRegistry->WriteBool( "BattOxid2", BattOxid2Chk->Checked ) ;
   // Batteries minimum levels
   SetRegString( "MinBattTilt1", MinBattTilt1->Text ) ;
   SetRegString( "MinBattTilt2", MinBattTilt2->Text ) ;
   SetRegString( "MinBattOxid1", MinBattOxid1->Text ) ;
   SetRegString( "MinBattOxid2", MinBattOxid2->Text ) ;
   SetRegString( "MaxBattTilt1", MaxBattTilt1->Text ) ;
   SetRegString( "MaxBattTilt2", MaxBattTilt2->Text ) ;
   SetRegString( "MaxBattOxid1", MaxBattOxid1->Text ) ;
   SetRegString( "MaxBattOxid2", MaxBattOxid2->Text ) ;
   // Email notifications
   theRegistry->WriteBool( "EmailNotification", SendEmailCheckBox->Checked ) ;

   // -- Formulas --
   // First trim all blanks from the formula as this is not
   // allowed in the component.
   // Drill 1
   AnsiString  VBat  = VBATFormula->Text ;
   AnsiString  Ta    = TAFormula->Text ;
   AnsiString  Ut    = UTFormula->Text ;
   AnsiString  TF0   = TFOFormula->Text ;
   AnsiString  E     = EFormula->Text ;
   AnsiString  PO2   = PO2Formula->Text ;
   AnsiString  Incl  = InclFormula->Text ;
   AnsiString  Coef  = CoefFormula->Text ;
   // Drill 2
   AnsiString  VBat2 = VBATFormula2->Text ;
   AnsiString  Ta2   = TAFormula2->Text ;
   AnsiString  Ut2   = UTFormula2->Text ;
   AnsiString  TF02  = TFOFormula2->Text ;
   AnsiString  E2    = EFormula2->Text ;
   AnsiString  PO22  = PO2Formula2->Text ;
   AnsiString  Incl2 = InclFormula2->Text ;
   AnsiString  Coef2 = CoefFormula2->Text ;
   // Trim Drill 1
   TrimBlanks( VBat ) ;
   TrimBlanks( Ta   ) ;
   TrimBlanks( Ut   ) ;
   TrimBlanks( TF0  ) ;
   TrimBlanks( E    ) ;
   TrimBlanks( PO2  ) ;
   TrimBlanks( Incl ) ;
   TrimBlanks( Coef ) ;
   // Trim Drill 2
   TrimBlanks( VBat2 ) ;
   TrimBlanks( Ta2   ) ;
   TrimBlanks( Ut2   ) ;
   TrimBlanks( TF02  ) ;
   TrimBlanks( E2    ) ;
   TrimBlanks( PO22  ) ;
   TrimBlanks( Incl2 ) ;
   TrimBlanks( Coef2 ) ;

   // Now store those formulas into the registy
   theRegistry->OpenKey("\\Software\\SteveWare\\Tiltmeters\\Formulas", true ) ;
   // Drill 1
   SetRegString( "VBat", VBat ) ;
   SetRegString( "Ta",   Ta   ) ;
   SetRegString( "Ut",   Ut   ) ;
   SetRegString( "TF0",  TF0  ) ;
   SetRegString( "E",    E    ) ;
   SetRegString( "PO2",  PO2  ) ;
   SetRegString( "Incl", Incl ) ;
   SetRegString( "Coef", Coef ) ;
   // Drill 2
   SetRegString( "VBat2", VBat2 ) ;
   SetRegString( "Ta2",   Ta2   ) ;
   SetRegString( "Ut2",   Ut2   ) ;
   SetRegString( "TF02",  TF02  ) ;
   SetRegString( "E2",    E2    ) ;
   SetRegString( "PO22",  PO22  ) ;
   SetRegString( "Incl2", Incl2 ) ;
   SetRegString( "Coef2", Coef2 ) ;

   // -- Scales --
   theRegistry->OpenKey("\\Software\\SteveWare\\Tiltmeters\\Scales\\", true ) ;
   SetRegInteger( "CurrentSelection", currentScaleSelection ) ;
   SaveOptions( currentScaleSelection ) ;
   ModalResult = mrOk ;
// Close() ;
}


// --------------------------------------------------------
// DEFINEEMAILBUTTONCLICK
// Open the email configuration window in order to let the
// operator create the list of emails to be sent by alarms.
// If the user clicks on OK, store the list of emails.
// Also load them before displaying data.
// --------------------------------------------------------
void __fastcall TConfigure::DefineEmailButtonClick(TObject *Sender) {
   int count ;
   AnsiString emailAddress ;
   // Retrieve the email list from the registry and prepare
   // the dialog box
   theRegistry->OpenKey("\\Software\\SteveWare\\Tiltmeters\\Emails", false ) ;
   count = GetRegInteger( "NbAddresses", 0 ) ;
   EmailForm->EmailList->Clear() ;
   if( count )
      for( int i=0; i<count; i++ ) {
         emailAddress = GetRegString( "Email"+IntToStr(i), "" ) ;
         EmailForm->EmailList->Items->Add( emailAddress ) ;
      }
   EmailForm->SMTPField->Clear() ;
   emailAddress = GetRegString( "SMTP", "" ) ;
   EmailForm->SMTPField->Text = emailAddress ;

   // Display the window
   EmailForm->ShowModal() ;
   if(EmailForm->ModalResult == mrOk) {
      count = EmailForm->EmailList->Items->Count ;
      // Save smtp
      theRegistry->OpenKey("\\Software\\SteveWare\\Tiltmeters\\Emails", true ) ;
      SetRegString( "SMTP", EmailForm->SMTPField->Text ) ;
      // Save addresses
      if( count != 0 ) {
         SetRegInteger( "NbAddresses", count  ) ;
         for( int i=0; i<count; i++ ) {
            emailAddress = EmailForm->EmailList->Items->Strings[i] ;
            SetRegString( "Email"+IntToStr(i), emailAddress  ) ;
         }
      }
   }
}


// --------------------------------------------------------
// CraterPrefixChange
// This is called when the user enters a new character into
// the CRATER Prefix field to update the example supplied
// to the right of that field.
// --------------------------------------------------------
void __fastcall TConfigure::CraterPrefixChange(TObject *Sender) {
   CraterPrefixEx->Caption = CraterPrefix->Text + "_07-28.03" ;
}



// --------------------------------------------------------
// Tilt1PrefixChange
// This is called when the user enters a new character into
// the TILT1 Prefix field to update the example supplied
// to the right of that field.
// --------------------------------------------------------
void __fastcall TConfigure::Tilt1PrefixChange(TObject *Sender) {
   Tilt1PrefixEx->Caption = Tilt1Prefix->Text + "_07-28.03" ;
}


// --------------------------------------------------------
// Tilt2PrefixChange
// This is called when the user enters a new character into
// the TILT2 Prefix field to update the example supplied
// to the right of that field.
// --------------------------------------------------------
void __fastcall TConfigure::Tilt2PrefixChange(TObject *Sender) {
   Tilt2PrefixEx->Caption = Tilt2Prefix->Text + "_07-28.03" ;
}


// GetRegBool
// Used to retrieve a bool value from the registry.
// If key doesn't exist, return the default value.
// --------------------------------------------------------
bool TConfigure::GetRegBool( AnsiString str, bool defValue ) {
   bool value ;
   try {
      value = theRegistry->ReadBool( str ) ;
   }
   catch ( ERegistryException* error ) {
      value = defValue ;
   }
   return value ;
}

// --------------------------------------------------------
// GetRegFloat
// Used to retrieve a float value from the registry.
// If key doesn't exist, return the default value.
// --------------------------------------------------------
float TConfigure::GetRegFloat( AnsiString str, float defValue ) {
   float value ;
   try {
      value = theRegistry->ReadFloat( str ) ;
   }
   catch ( ERegistryException* error ) {
      value = defValue ;
   }
   return value ;
}


// --------------------------------------------------------
// GetRegString
// Used to retrieve a string value from the registry.
// If key doesn't exist, return the default value.
// --------------------------------------------------------
AnsiString TConfigure::GetRegString( AnsiString str, AnsiString defValue ) {
   AnsiString value ;
   try {
      value = theRegistry->ReadString( str ) ;
   }
   catch( ERegistryException* error ) {
      value = defValue ;
   }
   if( value == "" )
      value = defValue ;
   return value ;
}

// --------------------------------------------------------
// SetRegString
// Used to write a string value to the registry.
// --------------------------------------------------------
void TConfigure::SetRegString( AnsiString str, AnsiString value) {
   theRegistry->WriteString( str, value ) ;
}

// --------------------------------------------------------
// SetRegInteger
// Used to write a string value to the registry.
// --------------------------------------------------------
void TConfigure::SetRegInteger( AnsiString str, int value) {
   theRegistry->WriteInteger( str, value ) ;
}

// --------------------------------------------------------
// SetRegFloat
// Used to write a sloat value to the registry.
// --------------------------------------------------------
void TConfigure::SetRegFloat( AnsiString str, float value) {
   theRegistry->WriteFloat( str, value ) ;
}

// --------------------------------------------------------
// GetRegInteger
// Used to retrieve an integer value from the registry.
// If key doesn't exist, return the default value.
// --------------------------------------------------------
int TConfigure::GetRegInteger( AnsiString str, int defValue ) {
   int value ;
   try {
      value = theRegistry->ReadInteger( str ) ;
   }
//   __finally { // ( ERegistryException* error )
   catch( ERegistryException* error ) {
      value = defValue ;
   }
   return value ;
}


// --------------------------------------------------------
// TrimBlanks
// Used to remove all blanks from formulas as blanks are
// not understood by the parser.
// --------------------------------------------------------
void TConfigure::TrimBlanks( AnsiString &str ) {
   int i = 1 ;
   if( str != "" ) {
      while( i<str.Length() ) {
         if( str.SubString(i, 1 ) == " " ) {
            str.Delete( i, 1 ) ;
            --i ; // To compensate with immediate i++
         }
         i++ ;
      }
   }
   str = str.TrimRight() ;
}



// --------------------------------------------------------
// TILT2RADCFGCLICK
// When the user clicks on TILT2 RADIAL, show check boxes
// for both X and Y axis and hide Y2
// --------------------------------------------------------
void __fastcall TConfigure::Tilt2RadCfgClick(TObject *Sender) {
   if( currentScaleSelection != TILT2 )
      SaveOptions( currentScaleSelection ) ;

   // Now get configuration for current scale button.
   // Default values are Automatic scales for X and Y
   // and no log. Hide Y2
   GetOptions( TILT2 ) ;
   DisableY2Axis() ;
   UpdateXScaleControls() ;
   UpdateYScaleControls() ;
   currentScaleSelection      = TILT2 ;
   YUnits->Caption = "[-rad]" ;
}


// --------------------------------------------------------
// TILT2TEMPCFGCLICK
// When the user clicks on TILT2 BATT, show check boxes
// for both X and Y axis and hide Y2
// --------------------------------------------------------
void __fastcall TConfigure::Tilt2TempCfgClick(TObject *Sender) {
   if( currentScaleSelection != TILT2BATT )
      SaveOptions( currentScaleSelection ) ;

   // Now get configuration for current scale button.
   // Default values are Automatic scales for X and Y
   // and no log. Hide Y2
   GetOptions( TILT2BATT ) ;
   EnableY2Axis() ;
   UpdateXScaleControls() ;
   UpdateYScaleControls() ;
   UpdateY2ScaleControls() ;
   YUnits->Caption = "[V]" ;
   Y2Units->Caption = "[C]" ;
   currentScaleSelection      = TILT2BATT ;
}


// --------------------------------------------------------
// TILT1TANCFGCLICK
// When the user clicks on TILT1 TANG, show check boxes
// for both X and Y axis and hide Y2
// --------------------------------------------------------
void __fastcall TConfigure::Tilt1TanCfgClick(TObject *Sender)
{
   if( currentScaleSelection != TILT1TAN )
      SaveOptions( currentScaleSelection ) ;

   // Now get configuration for current scale button.
   // Default values are Automatic scales for X and Y
   // and no log. Hide Y2
   GetOptions( TILT1TAN ) ;
   DisableY2Axis() ;
   UpdateXScaleControls() ;
   UpdateYScaleControls() ;
   UpdateY2ScaleControls() ;
   YUnits->Caption = "[-rad]" ;
   currentScaleSelection      = TILT1TAN ;
}



// --------------------------------------------------------
// TILT2TANCFGCLICK
// When the user clicks on TILT2 TANG, show check boxes
// for both X and Y axis and hide Y2
// --------------------------------------------------------
void __fastcall TConfigure::Tilt2TanCfgClick(TObject *Sender)
{
   if( currentScaleSelection != TILT2TAN )
      SaveOptions( currentScaleSelection ) ;

   // Now get configuration for current scale button.
   // Default values are Automatic scales for X and Y
   // and no log. Hide Y2
   GetOptions( TILT2TAN ) ;
   DisableY2Axis() ;
   UpdateXScaleControls() ;
   UpdateYScaleControls() ;
   YUnits->Caption = "[-rad]" ;
   currentScaleSelection      = TILT2TAN ;
}


// --------------------------------------------------------
// FUM1CFGCLICK
// When the user clicks on FUM1, show check boxes
// for both X and Y axis and hide Y2
// --------------------------------------------------------
void __fastcall TConfigure::Fum1CfgClick(TObject *Sender) {
   if( currentScaleSelection != TEMPFUM1 )
      SaveOptions( currentScaleSelection ) ;

   // Now get configuration for current scale button.
   // Default values are Automatic scales for X and Y
   // and no log. Hide Y2
   GetOptions( TEMPFUM1 ) ;
   EnableY2Axis() ;
   UpdateXScaleControls() ;
   UpdateYScaleControls() ;
   UpdateY2ScaleControls() ;
   YUnits->Caption = "[C]" ;
   Y2Units->Caption = "[C]" ;
   currentScaleSelection      = TEMPFUM1 ;
}



// --------------------------------------------------------
// FUM2RADCFGCLICK
// When the user clicks on FUM2, show check boxes
// for both X and Y axis and hide Y2
// --------------------------------------------------------
void __fastcall TConfigure::Fum2CfgClick(TObject *Sender) {
   if( currentScaleSelection != TEMPFUM2 )
      SaveOptions( currentScaleSelection ) ;

   // Now get configuration for current scale button.
   // Default values are Automatic scales for X and Y
   // and no log. Hide Y2
   GetOptions( TEMPFUM2 ) ;
   EnableY2Axis() ;
   UpdateXScaleControls() ;
   UpdateYScaleControls() ;
   UpdateY2ScaleControls() ;
   YUnits->Caption = "[C]" ;
   Y2Units->Caption = "[C]" ;
   currentScaleSelection      = TEMPFUM2 ;
}


// --------------------------------------------------------
// PO21CFGCLICK
// When the user clicks on P(O2) 1, show check boxes
// for both X and Y axis and hide Y2
// --------------------------------------------------------
void __fastcall TConfigure::PO21CfgClick(TObject *Sender) {
   if( currentScaleSelection != PO2S1WIN )
      SaveOptions( currentScaleSelection ) ;

   // Now get configuration for current scale button.
   // Default values are Automatic scales for X and Y
   // and no log. Hide Y2
   GetOptions( PO2S1WIN ) ;
   DisableY2Axis() ;
   UpdateXScaleControls() ;
   UpdateYScaleControls() ;
   YUnits->Caption = "[atm*1E-21]" ;
   currentScaleSelection      = PO2S1WIN ;
}


// --------------------------------------------------------
// PO22CFGCLICK
// When the user clicks on P(O2) 2, show check boxes
// for both X and Y axis and hide Y2
// --------------------------------------------------------
void __fastcall TConfigure::PO22CfgClick(TObject *Sender) {
   if( currentScaleSelection != PO2S2WIN )
      SaveOptions( currentScaleSelection ) ;

   // Now get configuration for current scale button.
   // Default values are Automatic scales for X and Y
   // and no log. Hide Y2
   GetOptions( PO2S2WIN ) ;
   DisableY2Axis() ;
   UpdateXScaleControls() ;
   UpdateYScaleControls() ;
   YUnits->Caption = "[atm*1E-21]" ;
   currentScaleSelection      = PO2S2WIN ;
}


// --------------------------------------------------------
// OXID1CFGCLICK
// When the user clicks on TILT1 RADIAL, show check boxes
// for both X and Y axis and hide Y2
// --------------------------------------------------------
void __fastcall TConfigure::Oxid1CfgClick(TObject *Sender) {
   if( currentScaleSelection != OXIDS1 )
      SaveOptions( currentScaleSelection ) ;

   // Now get configuration for current scale button.
   // Default values are Automatic scales for X and Y
   // and no log. Hide Y2
   GetOptions( OXIDS1 ) ;
   EnableY2Axis() ;
   UpdateXScaleControls() ;
   UpdateYScaleControls() ;
   UpdateY2ScaleControls() ;
   YUnits->Caption = "[V]" ;
   Y2Units->Caption = "oxid" ;
   currentScaleSelection      = OXIDS1 ;
}


// --------------------------------------------------------
// OXID2CFGCLICK
// When the user clicks on TILT1 RADIAL, show check boxes
// for both X and Y axis and hide Y2
// --------------------------------------------------------
void __fastcall TConfigure::Oxid2CfgClick(TObject *Sender) {
   if( currentScaleSelection != OXIDS2 )
      SaveOptions( currentScaleSelection ) ;

   // Now get configuration for current scale button.
   // Default values are Automatic scales for X and Y
   // and no log. Hide Y2
   GetOptions( OXIDS2 ) ;
   EnableY2Axis() ;
   UpdateXScaleControls() ;
   UpdateYScaleControls() ;
   UpdateY2ScaleControls() ;
   YUnits->Caption = "[V]" ;
   Y2Units->Caption = "oxid" ;
   currentScaleSelection      = OXIDS2 ;
}

// --------------------------------------------------------
// DISABLEY2AXIS
// Hide the controls for the 2nd Y axis
// --------------------------------------------------------
void TConfigure::DisableY2Axis() {
   RightYAxisLabel->Visible   = false ;
   RightYAxisLabel->Enabled   = false ;
   AutoY2Axis->Visible        = false ;
   AutoY2Axis->Enabled        = false ;
   Y2LogScaleChk->Visible     = false ;
   Y2LogScaleChk->Enabled     = false ;
}



// --------------------------------------------------------
// ENABLEY2AXIS
// Show the controls for the 2nd Y axis
// --------------------------------------------------------
void TConfigure::EnableY2Axis() {
   RightYAxisLabel->Visible   = true ;
   RightYAxisLabel->Enabled   = true ;
   AutoY2Axis->Visible        = true ;
   AutoY2Axis->Enabled        = true ;
   Y2LogScaleChk->Visible     = true ;
   Y2LogScaleChk->Enabled     = true ;
}


// --------------------------------------------------------
// SAVEOPTIONS
// Save current configuration for specified SCALE button
// --------------------------------------------------------
void TConfigure::SaveOptions( int id  ) {
   AnsiString prefix ;

   switch( id ) {
      case TILT1:
         prefix = "Tilt1" ;
         break ;
      case TILT2:
         prefix = "Tilt2" ;
         break ;
      case TILT1BATT:
         prefix = "Tilt1Batt" ;
         break ;
      case TILT2BATT:
         prefix = "Tilt2Batt" ;
         break ;
      case TILT1TAN:
         prefix = "Tilt1Tan" ;
         break ;
      case TILT2TAN:
         prefix = "Tilt2Tan" ;
         break ;
      case TEMPFUM1:
         prefix = "TempFum1" ;
         break ;
      case TEMPFUM2:
         prefix = "TempFum2" ;
         break ;
      case PO2S1WIN:
         prefix = "PO21" ;
         break ;
      case PO2S2WIN:
         prefix = "PO22" ;
         break ;
      case OXIDS1:
         prefix = "Oxid1" ;
         break ;
      case OXIDS2:
         prefix = "Oxid2" ;
         break ;
   }

   //theRegistry->WriteBool...
   theRegistry->OpenKey("\\Software\\SteveWare\\Tiltmeters\\Scales\\"+prefix, true ) ;
   theRegistry->WriteBool( "XAuto",  AutoXAxis->Checked     ) ;
   theRegistry->WriteBool( "YAuto",  AutoYAxis->Checked     ) ;
   theRegistry->WriteBool( "Y2Auto", AutoY2Axis->Checked    ) ;
   theRegistry->WriteBool( "XLog",   XLogScaleChk->Checked  ) ;
   theRegistry->WriteBool( "YLog",   YLogScaleChk->Checked  ) ;
   theRegistry->WriteBool( "Y2Log",  Y2LogScaleChk->Checked ) ;
//   SetRegFloat( "MaxXScale",  StrToFloat( MaxXScale->Text  ) ) ;
//   SetRegFloat( "MinXScale",  StrToFloat( MinXScale->Text  ) ) ;
   SetRegFloat( "MaxYScale",  StrToFloat( MaxYScale->Text  ) ) ;
   SetRegFloat( "MinYScale",  StrToFloat( MinYScale->Text  ) ) ;
   SetRegFloat( "MaxY2Scale", StrToFloat( MaxY2Scale->Text ) ) ;
   SetRegFloat( "MinY2Scale", StrToFloat( MinY2Scale->Text ) ) ;
}



// --------------------------------------------------------
// GETOPTIONS
// Get current configuration for specified SCALE button
// --------------------------------------------------------
void TConfigure::GetOptions( int id  ) {
   AnsiString prefix ;

   switch( id ) {
      case TILT1:
         prefix = "Tilt1" ;
         break ;
      case TILT2:
         prefix = "Tilt2" ;
         break ;
      case TILT1BATT:
         prefix = "Tilt1Batt" ;
         break ;
      case TILT2BATT:
         prefix = "Tilt2Batt" ;
         break ;
      case TILT1TAN:
         prefix = "Tilt1Tan" ;
         break ;
      case TILT2TAN:
         prefix = "Tilt2Tan" ;
         break ;
      case TEMPFUM1:
         prefix = "TempFum1" ;
         break ;
      case TEMPFUM2:
         prefix = "TempFum2" ;
         break ;
      case PO2S1WIN:
         prefix = "PO21" ;
         break ;
      case PO2S2WIN:
         prefix = "PO22" ;
         break ;
      case OXIDS1:
         prefix = "Oxid1" ;
         break ;
      case OXIDS2:
         prefix = "Oxid2" ;
         break ;
   }

   //theRegistry->WriteBool...
   theRegistry->OpenKey("\\Software\\SteveWare\\Tiltmeters\\Scales\\"+prefix, false ) ;
   AutoXAxis->Checked  = GetRegBool( "XAuto",  true ) ;
   AutoYAxis->Checked  = GetRegBool( "YAuto",  true ) ;
   AutoY2Axis->Checked = GetRegBool( "Y2Auto", true ) ;
   XLogScaleChk->Checked  = GetRegBool( "XLog",  false ) ;
   YLogScaleChk->Checked  = GetRegBool( "YLog",  false ) ;
   Y2LogScaleChk->Checked = GetRegBool( "Y2Log", false ) ;
//   MaxXScale->Text  = FloatToStr( GetRegFloat( "MaxXScale",  180 ) ) ;
//   MinXScale->Text  = FloatToStr( GetRegFloat( "MinXScale",  0   ) ) ;
   MaxYScale->Text  = FloatToStr( GetRegFloat( "MaxYScale",  5   ) ) ;
   MinYScale->Text  = FloatToStr( GetRegFloat( "MinYScale",  0   ) ) ;
   MaxY2Scale->Text = FloatToStr( GetRegFloat( "MaxY2Scale", 12  ) ) ;
   MinY2Scale->Text = FloatToStr( GetRegFloat( "MinY2Scale", 0   ) ) ;
}

// --------------------------------------------------------
// REGWORKWITH
// Tells the confiuration class on which subkey to work in
// the Win95 registry.
// --------------------------------------------------------
void TConfigure::RegWorkWith( AnsiString key, bool create ) {
   theRegistry->RootKey = HKEY_LOCAL_MACHINE ;
   theRegistry->OpenKey("\\Software\\SteveWare\\Tiltmeters\\"+key, create ) ;
}


// --------------------------------------------------------
// EXTERNALUPDATECHECKCLICK
// The user has checked/unchecked the external tool option.
// Enable or disable the selection button.
// --------------------------------------------------------
void __fastcall TConfigure::ExternalUpdateCheckClick(TObject *Sender) {
   if( ExternalUpdateCheck->Checked )
      UpdateButton->Enabled = true ;
   else
      UpdateButton->Enabled = false ;
}


// --------------------------------------------------------
// UpdateButtonClick
// Open the file dialog box to select the executable to
// run each time the software should update the data.
// --------------------------------------------------------
void __fastcall TConfigure::UpdateButtonClick(TObject *Sender) {
   AnsiString ExecName ;
   if( OpenExternal->Execute() ) {
      ExecName = OpenExternal->FileName ;
      UpdateButton->Caption = ExecName ;
   }
}


// --------------------------------------------------------
// Incl1Change
// This is called when Incl1 formula is changed.
// --------------------------------------------------------
void __fastcall TConfigure::Incl1Change(TObject *Sender)
{
   Incl1Dirty = true ;
}


// --------------------------------------------------------
// Incl2Change
// This is called when Incl2 formula is changed.
// --------------------------------------------------------
void __fastcall TConfigure::Incl2Change(TObject *Sender)
{
   Incl2Dirty = true ;
}

// --------------------------------------------------------
// VBat1Change
// This is called when VBat1 formula is changed.
// --------------------------------------------------------
void __fastcall TConfigure::VBat1Change(TObject *Sender)
{
   VBat1Dirty = true ;
}



// --------------------------------------------------------
// TA1Change
// This is called when TA1 formula is changed.
// --------------------------------------------------------
void __fastcall TConfigure::TA1Change(TObject *Sender)
{
   TA1Dirty = true ;
}


// --------------------------------------------------------
// UT1Change
// This is called when UT1 formula is changed.
// --------------------------------------------------------
void __fastcall TConfigure::UT1Change(TObject *Sender)
{
   UT1Dirty = true ;
}


// --------------------------------------------------------
// TF01Change
// This is called when TF01 formula is changed.
// --------------------------------------------------------
void __fastcall TConfigure::TF01Change(TObject *Sender)
{
   TF01Dirty = true ;
}



// --------------------------------------------------------
// E1Change
// This is called when E1 formula is changed.
// --------------------------------------------------------
void __fastcall TConfigure::E1Change(TObject *Sender)
{
   E1Dirty = true ;
}



// --------------------------------------------------------
// PO21Change
// This is called when PO21 formula is changed.
// --------------------------------------------------------
void __fastcall TConfigure::PO21Change(TObject *Sender)
{
   PO21Dirty = true ;
}



// --------------------------------------------------------
// Coeff1Change
// This is called when Coeff1 formula is changed.
// --------------------------------------------------------
void __fastcall TConfigure::Coeff1Change(TObject *Sender)
{
   Coeff1Dirty = true ;
}


// --------------------------------------------------------
// VBat2Change
// This is called when VBat2 formula is changed.
// --------------------------------------------------------
void __fastcall TConfigure::VBat2Change(TObject *Sender)
{
   VBat2Dirty = true ;
}



// --------------------------------------------------------
// TA2Change
// This is called when TA2 formula is changed.
// --------------------------------------------------------
void __fastcall TConfigure::TA2Change(TObject *Sender)
{
   TA2Dirty = true ;
}


// --------------------------------------------------------
// Ut2Change
// This is called when UT2 formula is changed.
// --------------------------------------------------------
void __fastcall TConfigure::UT2Change(TObject *Sender)
{
   UT2Dirty = true ;
}



// --------------------------------------------------------
// TF02Change
// This is called when TF02 formula is changed.
// --------------------------------------------------------
void __fastcall TConfigure::TF02Change(TObject *Sender)
{
   TF02Dirty = true ;
}


// --------------------------------------------------------
// E2Change
// This is called when E2 formula is changed.
// --------------------------------------------------------
void __fastcall TConfigure::E2Change(TObject *Sender)
{
   E2Dirty = true ;
}



// --------------------------------------------------------
// PO22Change
// This is called when PO22 formula is changed.
// --------------------------------------------------------
void __fastcall TConfigure::PO22Change(TObject *Sender)
{
   PO22Dirty = true ;
}



// --------------------------------------------------------
// Coeff2Change
// This is called when Coeff2 formula is changed.
// --------------------------------------------------------
void __fastcall TConfigure::Coeff2Change( TObject *Sender)
{
   Coeff2Dirty = true ;
}

// --------------------------------------------------------
// IsVBat1Dirty
// Return formula field state (dirty means we need to
// update the chart with the new formula)
// --------------------------------------------------------
bool TConfigure::IsVBat1Dirty() {
   return VBat1Dirty ;
}

// --------------------------------------------------------
// IsVBat2Dirty
// Return formula field state (dirty means we need to
// update the chart with the new formula)
// --------------------------------------------------------
bool TConfigure::IsVBat2Dirty() {
   return VBat2Dirty ;
}


// --------------------------------------------------------
// IsTA1Dirty
// Return formula field state (dirty means we need to
// update the chart with the new formula)
// --------------------------------------------------------
bool TConfigure::IsTA1Dirty() {
   return TA1Dirty ;
}


// --------------------------------------------------------
// IsTA2Dirty
// Return formula field state (dirty means we need to
// update the chart with the new formula)
// --------------------------------------------------------
bool TConfigure::IsTA2Dirty() {
   return TA2Dirty ;
}


// --------------------------------------------------------
// IsUT1Dirty
// Return formula field state (dirty means we need to
// update the chart with the new formula)
// --------------------------------------------------------
bool TConfigure::IsUT1Dirty() {
   return UT1Dirty ;
}


// --------------------------------------------------------
// IsUT2Dirty
// Return formula field state (dirty means we need to
// update the chart with the new formula)
// --------------------------------------------------------
bool TConfigure::IsUT2Dirty() {
   return UT2Dirty ;
}


// --------------------------------------------------------
// IsTF01Dirty
// Return formula field state (dirty means we need to
// update the chart with the new formula)
// --------------------------------------------------------
bool TConfigure::IsTF01Dirty() {
   return TF01Dirty ;
}


// --------------------------------------------------------
// IsTF02Dirty
// Return formula field state (dirty means we need to
// update the chart with the new formula)
// --------------------------------------------------------
bool TConfigure::IsTF02Dirty() {
   return TF02Dirty ;
}


// --------------------------------------------------------
// IsE1Dirty
// Return formula field state (dirty means we need to
// update the chart with the new formula)
// --------------------------------------------------------
bool TConfigure::IsE1Dirty() {
   return E1Dirty ;
}


// --------------------------------------------------------
// IsE2Dirty
// Return formula field state (dirty means we need to
// update the chart with the new formula)
// --------------------------------------------------------
bool TConfigure::IsE2Dirty() {
   return E2Dirty ;
}


// --------------------------------------------------------
// IsPO21Dirty
// Return formula field state (dirty means we need to
// update the chart with the new formula)
// --------------------------------------------------------
bool TConfigure::IsPO21Dirty() {
   return PO21Dirty ;
}


// --------------------------------------------------------
// IsPO22Dirty
// Return formula field state (dirty means we need to
// update the chart with the new formula)
// --------------------------------------------------------
bool TConfigure::IsPO22Dirty() {
   return PO22Dirty ;
}


// --------------------------------------------------------
// IsCoeff1Dirty
// Return formula field state (dirty means we need to
// update the chart with the new formula)
// --------------------------------------------------------
bool TConfigure::IsCoeff1Dirty() {
   return Coeff1Dirty ;
}


// --------------------------------------------------------
// IsCoeff2Dirty
// Return formula field state (dirty means we need to
// update the chart with the new formula)
// --------------------------------------------------------
bool TConfigure::IsCoeff2Dirty() {
   return Coeff2Dirty ;
}


// --------------------------------------------------------
// IsIncl1Dirty
// Return formula field state (dirty means we need to
// update the chart with the new formula)
// --------------------------------------------------------
bool TConfigure::IsIncl1Dirty() {
   return Incl1Dirty ;
}


// --------------------------------------------------------
// IsIncl2Dirty
// Return formula field state (dirty means we need to
// update the chart with the new formula)
// --------------------------------------------------------
bool TConfigure::IsIncl2Dirty() {
   return Incl2Dirty ;
}



// --------------------------------------------------------
// DEFAULTFORMULACLICK
// We enter this function when the user choses "Default
// formula" on the popup menu available over each formula
// field.
// In order to know which one has the focus, we manage the
// "OnMouseMove" event of each field. Then we can know if we
// are on the Drill1 or Drill2 pages and act accordingly.
// --------------------------------------------------------
void __fastcall TConfigure::DefaultFormulaClick(TObject *Sender)
{
   // Insert default formula back into the field.
   bool page1 = (FormulaTab->ActivePage == Drill1Tab) ? true : false ;


   switch( currentFormulaEdit ) {
      case VBATFIELD:
         if( page1 )
            VBATFormula->Text  = VBATFORMULA ;
         else
            VBATFormula2->Text = VBATFORMULA ;
         break ;

      case TAFIELD:
         if( page1 )
            TAFormula->Text  = TAFORMULA ;
         else
            TAFormula2->Text = TAFORMULA ;
         break ;

      case UTFIELD:
         if( page1 )
            UTFormula->Text  = UTFORMULA ;
         else
            UTFormula2->Text = UTFORMULA ;
         break ;

      case TF0FIELD:
         if( page1 )
            TFOFormula->Text  = TF0FORMULA ;
         else
            TFOFormula2->Text = TF0FORMULA ;
         break ;

      case EFIELD:
         if( page1 )
            EFormula->Text  = E1FORMULA ;
         else
            EFormula2->Text = E2FORMULA ;
         break ;

      case PO2FIELD:
         if( page1 )
            PO2Formula->Text  = PO2FORMULA ;
         else
            PO2Formula2->Text = PO2FORMULA ;
         break ;

      case COEFFIELD:
         if( page1 )
            CoefFormula->Text  = COEFFORMULA ;
         else
            CoefFormula2->Text = COEFFORMULA ;
         break ;

      case INCLFIELD:
         if( page1 )
            InclFormula->Text  = INCLFORMULA ;
         else
            InclFormula2->Text = INCLFORMULA ;
         break ;

   }
}


// ------------------------------------------------------
// INCLFORMULAENTER
// The mouse enters the INCL Formula.
// --------------------------------------------------------
void __fastcall TConfigure::InclFormulaMouseMove(TObject *Sender,
   TShiftState Shift, int X, int Y)
{
   currentFormulaEdit = INCLFIELD ;
}


// --------------------------------------------------------
// COEFFFORMULAENTER
// The mouse enters the COEFF Formula.
// --------------------------------------------------------
void __fastcall TConfigure::CoefFormulaMouseMove(TObject *Sender,
   TShiftState Shift, int X, int Y)
{
   currentFormulaEdit = COEFFIELD ;
}


// --------------------------------------------------------
// PO2FORMULAENTER
// The mouse enters the PO2 Formula.
// --------------------------------------------------------
void __fastcall TConfigure::PO2FormulaMouseMove(TObject *Sender,
   TShiftState Shift, int X, int Y)
{
   currentFormulaEdit = PO2FIELD ;
}



// --------------------------------------------------------
// EFORMULAENTER
// The mouse enters the E Formula.
// --------------------------------------------------------
void __fastcall TConfigure::EFormulaMouseMove(TObject *Sender,
   TShiftState Shift, int X, int Y)
{
   currentFormulaEdit = EFIELD ;
}


// ------------------------------------------------------
// TF0FORMULAENTER
// The mouse enters the TF0 Formula.
// --------------------------------------------------------
void __fastcall TConfigure::TFOFormulaMouseMove(TObject *Sender,
   TShiftState Shift, int X, int Y)
{
   currentFormulaEdit = TF0FIELD ;
}


// --------------------------------------------------------
// UTFORMULAENTER
// The mouse enters the UT Formula.
// --------------------------------------------------------
void __fastcall TConfigure::UTFormulaMouseMove(TObject *Sender,
   TShiftState Shift, int X, int Y)
{
   currentFormulaEdit = UTFIELD ;
}


// ------------------------------------------------------
// TAFORMULAENTER
// The mouse enters the TA Formula.
// ------------------------------------------------------
void __fastcall TConfigure::TAFormulaMouseMove(TObject *Sender,
   TShiftState Shift, int X, int Y)
{
   currentFormulaEdit = TAFIELD ;
}



// --------------------------------------------------------
// VBATFORMULAENTER
// The mouse enters the VBAT Formula.
// --------------------------------------------------------
void __fastcall TConfigure::VBATFormulaMouseMove(TObject *Sender,
   TShiftState Shift, int X, int Y)
{
   currentFormulaEdit = VBATFIELD ;
}

