//  CLASS TCONFIGURE
//  This is the base class configuring Titlmeters.
//  It handles everything the user has control on, such as the
//  scales for each chart axis, alarm levels for battery and
//  temperatures, as well as the formulas used in the calculation,
//  and when the program should pass the hand to the acquisition
//  one.
//
// (C) Steve JORDI - 1996-1999  
// steve@sjordi.com - www.sjordi.com
//
// All this code is copyrighted by Steve Jordi and
// cannot be reproduced without the consent of the author.
// The executable may be freely distributed.
//
// MAKE  SURE  TO  UNDERSTAND  AND  ACCEPT  THE STATEMENTS BELOW
// BEFORE USING THE SOFTWARE:
//
// THIS PROGRAM  IS  PROVIDED  "AS IS"  WITHOUT  WARRANTY OF ANY
// KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED
// TO  THE  IMPLIED  WARRANTIES  OF MERCHANTABILITY  AND FITNESS
// FOR  A  PARTICULAR PURPOSE. THE ENTIRE RISK AS TO THE QUALITY
// AND   PERFORMANCE  OF  THE  PROGRAM  IS  WITH YOU. SHOULD THE
// PROGRAM  PROVE   DEFECTIVE,  YOU   (NOT  STEVE JORDI)  ASSUME
// THE ENTIRE COST OF ALL  NECESSARY   SERVICING,   REPAIR,   OR
// CORRECTION. IN NO EVENT WILL STEVE JORDI BE LIABLE TO YOU FOR
// ANY  DAMAGES ,  INCLUDING ANY LOST PROFITS,  LOST SAVINGS, OR
// OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES  ARISING OUT OF THE
// USE OR INABILITY TO USE  SUCH  PROGRAM  EVEN  IF  STEVE JORDI
// HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES,   OR FOR
// ANY  CLAIM BY ANY OTHER PARTY.
// ALSO, USE IT IN LIFE THREATENING SITUATIONS AT YOUR OWN RISKS
// (SUCH AS VOLCANO MONITORING). STEVE JORDI CANNOT BE ACCOUNTED
// AS RESPONSIBLE FOR ANY LIFE OR PROPERTY LOST IN CASE AN ALARM
// SYSTEM HAD A MALFUNCTION OR WAS NOT ABLE TO CORRECTLY  BEHAVE
// IN A CRISIS ENVIRONMENT.
//---------------------------------------------------------------------------
#ifndef ConfigH
#define ConfigH
//---------------------------------------------------------------------------
#include <vcl\Classes.hpp>
#include <vcl\Controls.hpp>
#include <vcl\StdCtrls.hpp>
#include <vcl\Forms.hpp>
#include <vcl\ComCtrls.hpp>
#include <vcl\ExtCtrls.hpp>
#include <vcl\Buttons.hpp>
#include <vcl\Registry.hpp>
#include "Parser10.hpp"
#include <vcl\Dialogs.hpp>
#include <vcl\Menus.hpp>
#include "marsCap.hpp"
#include <Graphics.hpp>

// Report the following to MAIN.H if modified
#define  TILT1          1
#define  TILT2          2
#define  TILT1BATT      3
#define  TILT2BATT      4
#define  TILT1TAN       5
#define  TILT2TAN       6
#define  TEMPFUM1       7
#define  TEMPFUM2       8
#define  PO2S1WIN       9
#define  PO2S2WIN       10
#define  OXIDS1         11
#define  OXIDS2         12

// Edit field constants
#define  VBATFIELD      1
#define  TAFIELD        2
#define  UTFIELD        3
#define  TF0FIELD       4
#define  EFIELD         5
#define  PO2FIELD       6
#define  COEFFIELD      7
#define  INCLFIELD      8

// FORMULAES TO BE USED
#define  TILTFORMULA    "X*5.26/6553.4"
#define  TAFORMULA      "X*20/6553.4"
#define  UTFORMULA      "-.05+4.48*TA+.01*TA*TA+X*2000/6553.4"
#define  E1FORMULA      "(0.497*TF0-431)-X*200/6553.4"
#define  E2FORMULA      "(0.444*TF0-490)-X*200/6553.4"
#define  PO2FORMULA     "EXP(E/0.021544/(TF0+273))"
#define  INCLFORMULA    "X/6553.4*8725.4"
#define  COEFFORMULA    "LOG10(PO2)/0.65+38000/(TF0+273)-12"
#define  VBATFORMULA    "X*5.26/6553.4"
#define  TAFORMULA      "X*20/6553.4"
#define  TF0FORMULA     "37+.1164*UT-1.7E-6*UT*UT"

//---------------------------------------------------------------------------
class TConfigure : public TForm
{
__published:   // IDE-managed Components
   TButton *Button2;
   TPageControl *ConfigPageControl;
   TTabSheet *DirectorySheet;
   TTabSheet *ScaleSheet;
   TTabSheet *FormulaSheet;
   TTabSheet *AutomationSheet;
   TLabel *Label11;
   TEdit *MaxYScale;
   TEdit *MinYScale;
   TLabel *MinYLegend;
   TLabel *MaxYLegend;
   TCheckBox *XLogScaleChk;
   TCheckBox *AutoXAxis;
   TEdit *MaxY2Scale;
   TEdit *MinY2Scale;
   TLabel *RightYAxisLabel;
   TLabel *Label4;
   TBevel *Bevel3;
   TLabel *Label5;
   TGroupBox *GroupBox5;
   TLabel *Label44;
   TUpDown *HourUpDown;
   TEdit *HourEdit;
   TLabel *Label45;
   TEdit *MinuteEdit;
   TUpDown *MinuteUpDown;
   TBevel *Bevel4;
   TTabSheet *AlarmSheet;
   TGroupBox *GroupBox6;
   TLabel *Label49;
   TLabel *Label50;
   TLabel *Label51;
   TLabel *Label52;
   TEdit *MinBattTilt1;
   TEdit *MinBattTilt2;
   TEdit *MinBattOxid1;
   TEdit *MinBattOxid2;
   TLabel *Label53;
   TLabel *Label54;
   TLabel *Label55;
   TLabel *Label56;
   TLabel *Label57;
   TLabel *Label58;
   TCheckBox *BattTilt1Chk;
   TCheckBox *BattTilt2Chk;
   TCheckBox *BattOxid1Chk;
   TCheckBox *BattOxid2Chk;
   TGroupBox *GroupBox7;
   TLabel *Label59;
   TLabel *Label60;
   TLabel *Label61;
   TLabel *Label62;
   TLabel *Label63;
   TLabel *Label64;
   TLabel *Label65;
   TLabel *Label66;
   TLabel *Label67;
   TLabel *Label68;
   TEdit *MaxTempFum1;
   TEdit *MaxTempFum2;
   TEdit *MaxTempAmb1;
   TEdit *MaxTempAmb2;
   TCheckBox *TempFum1Chk;
   TCheckBox *TempFum2Chk;
   TCheckBox *TempAmb1Chk;
   TCheckBox *TempAmb2Chk;
   TLabel *Label69;
   TCheckBox *TempTilt1Chk;
   TEdit *MaxTempTilt1;
   TLabel *Label70;
   TLabel *Label71;
   TCheckBox *TempTilt2Chk;
   TEdit *MaxTempTilt2;
   TLabel *Label72;
   TGroupBox *GroupBox8;
   TLabel *Label1;
   TButton *CRADirButton;
   TLabel *Label2;
   TButton *Tilt1DirButton;
   TLabel *Label3;
   TButton *Tilt2DirButton;
   TLabel *Label46;
   TLabel *Hour1;
   TLabel *Label48;
   TLabel *Minute1;
   TLabel *Hour2;
   TLabel *Label73;
   TLabel *Minute2;
   TLabel *Hour3;
   TLabel *Label76;
   TLabel *Minute3;
   TLabel *Hour4;
   TLabel *Label74;
   TLabel *Minute4;
   TLabel *Label47;
   TButton *TempAllOn;
   TButton *TempAllOff;
   TButton *BattAllOn;
   TButton *BattAllOff;
   TSpeedButton *Tilt1RadCfg;
   TSpeedButton *Tilt2RadCfg;
   TSpeedButton *Tilt1TempCfg;
   TSpeedButton *Tilt2TempCfg;
   TSpeedButton *Tilt1TanCfg;
   TSpeedButton *Tilt2TanCfg;
   TSpeedButton *Fum1Cfg;
   TSpeedButton *Fum2Cfg;
   TSpeedButton *PO21Cfg;
   TSpeedButton *PO22Cfg;
   TImage *Image1;
   TLabel *Label6;
   TCheckBox *AutoYAxis;
   TCheckBox *YLogScaleChk;
   TCheckBox *AutoY2Axis;
   TCheckBox *Y2LogScaleChk;
   TLabel *MaxY2Legend;
   TLabel *MinY2Legend;
   TBevel *Bevel1;
   TLabel *Label14;
   TLabel *Label15;
   TLabel *Label16;
   TLabel *Label17;
   TLabel *Label18;
   TLabel *Label19;
   TLabel *YUnits;
   TLabel *XUnits;
   TLabel *Y2Units;
   TGroupBox *GroupBox9;
   TCheckBox *SendEmailCheckBox;
   TButton *DefineEmailButton;
   TGroupBox *GroupBox10;
   TLabel *Label7;
   TLabel *Label8;
   TLabel *Label9;
   TLabel *Label10;
   TLabel *Label12;
   TLabel *Label13;
   TBevel *Bevel2;
   TEdit *CraterPrefix;
   TEdit *Tilt1Prefix;
   TEdit *Tilt2Prefix;
   TLabel *Label20;
   TLabel *CraterPrefixEx;
   TLabel *Label22;
   TLabel *Tilt1PrefixEx;
   TLabel *Label24;
   TLabel *Tilt2PrefixEx;
   TSpeedButton *Oxid1Cfg;
   TSpeedButton *Oxid2Cfg;
   TGroupBox *GroupBox11;
   TLabel *Label21;
   TLabel *Label23;
   TLabel *Label25;
   TLabel *Label26;
   TLabel *Label27;
   TLabel *Label28;
   TBevel *Bevel5;
   TCheckBox *ExternalUpdateCheck;
   TButton *UpdateButton;
   TOpenDialog *OpenExternal;
   TLabel *Label32;
   TEdit *MaxBattTilt1;
   TEdit *MaxBattTilt2;
   TEdit *MaxBattOxid1;
   TEdit *MaxBattOxid2;
   TLabel *Label75;
   TLabel *Label77;
   TLabel *Label78;
   TLabel *Label79;
   TLabel *Label81;
   TEdit *OffsetEdit;
   TUpDown *OffsetUpDown;
   TPageControl *FormulaTab;
   TTabSheet *Drill1Tab;
   TTabSheet *Drill2Tab;
   TGroupBox *GroupBox1;
   TLabel *Label33;
   TLabel *Label34;
   TEdit *VBATFormula;
   TGroupBox *GroupBox2;
   TLabel *Label35;
   TLabel *Label36;
   TEdit *TAFormula;
   TGroupBox *GroupBox3;
   TLabel *Label37;
   TLabel *Label38;
   TLabel *Label39;
   TLabel *Label40;
   TEdit *UTFormula;
   TEdit *TFOFormula;
   TGroupBox *GroupBox4;
   TLabel *Label41;
   TLabel *Label42;
   TLabel *Label43;
   TLabel *Label80;
   TEdit *EFormula;
   TEdit *PO2Formula;
   TGroupBox *GroupBox13;
   TLabel *Label31;
   TEdit *CoefFormula;
   TGroupBox *GroupBox12;
   TLabel *Label29;
   TLabel *Label30;
   TEdit *InclFormula;
   TGroupBox *GroupBox14;
   TLabel *Label82;
   TLabel *Label83;
   TEdit *VBATFormula2;
   TGroupBox *GroupBox15;
   TLabel *Label84;
   TLabel *Label85;
   TEdit *TAFormula2;
   TGroupBox *GroupBox16;
   TLabel *Label86;
   TLabel *Label87;
   TLabel *Label88;
   TLabel *Label89;
   TEdit *UTFormula2;
   TEdit *TFOFormula2;
   TGroupBox *GroupBox17;
   TLabel *Label90;
   TLabel *Label91;
   TLabel *Label92;
   TLabel *Label93;
   TEdit *EFormula2;
   TEdit *PO2Formula2;
   TGroupBox *GroupBox18;
   TLabel *Label94;
   TEdit *CoefFormula2;
   TGroupBox *GroupBox19;
   TLabel *Label95;
   TLabel *Label96;
   TEdit *InclFormula2;
   TTabSheet *TabSheet3;
   TButton *ChangePasswordButton;
   TButton *OKButton;
   TPopupMenu *FormulaPopUp;
   TMenuItem *DefaultFormula;
   TEdit *LastEdit;
   TLabel *LastLabel;
   TComboBox *RangeDropDown;
   TMarsCaption *MarsCaption1;
   void __fastcall CRADirButtonClick(TObject *Sender);
   void __fastcall Tilt1DirButtonClick(TObject *Sender);
   void __fastcall Tilt2DirButtonClick(TObject *Sender);
   void __fastcall ChangePasswordButtonClick(TObject *Sender);
   void __fastcall ConfigPageControlChange(TObject *Sender);

   void __fastcall FormShow(TObject *Sender);
   void __fastcall FormClose(TObject *Sender, TCloseAction &Action);


   void __fastcall HourUpDownClick(TObject *Sender, TUDBtnType Button);
   void __fastcall MinuteUpDownClick(TObject *Sender, TUDBtnType Button);
   
   
   void __fastcall TempFum1ChkClick(TObject *Sender);
   void __fastcall TempFum2ChkClick(TObject *Sender);
   void __fastcall TempAmb1ChkClick(TObject *Sender);
   void __fastcall TempAmb2ChkClick(TObject *Sender);
   void __fastcall TempTilt1ChkClick(TObject *Sender);
   void __fastcall TempTilt2ChkClick(TObject *Sender);
   void __fastcall BattTilt1ChkClick(TObject *Sender);
   void __fastcall BattTilt2ChkClick(TObject *Sender);
   void __fastcall BattOxid1ChkClick(TObject *Sender);
   void __fastcall BattOxid2ChkClick(TObject *Sender);
   void __fastcall TempAllOnClick(TObject *Sender);
   
   void __fastcall TempAllOffClick(TObject *Sender);
   void __fastcall BattAllOnClick(TObject *Sender);
   void __fastcall BattAllOffClick(TObject *Sender);
   void __fastcall AutoXAxisClick(TObject *Sender);
   void __fastcall AutoYAxisClick(TObject *Sender);
   void __fastcall AutoY2AxisClick(TObject *Sender);
   void __fastcall Tilt1RadCfgClick(TObject *Sender);

   void __fastcall Tilt1TempCfgClick(TObject *Sender);
   void __fastcall AutomationSheetEnter(TObject *Sender);

   void __fastcall OKButtonClick(TObject *Sender);
   
   
   void __fastcall DefineEmailButtonClick(TObject *Sender);
   void __fastcall CraterPrefixChange(TObject *Sender);
   void __fastcall Tilt1PrefixChange(TObject *Sender);
   void __fastcall Tilt2PrefixChange(TObject *Sender);
   void __fastcall Tilt2RadCfgClick(TObject *Sender);
   void __fastcall Tilt2TempCfgClick(TObject *Sender);
   void __fastcall Tilt1TanCfgClick(TObject *Sender);
   void __fastcall Tilt2TanCfgClick(TObject *Sender);
   void __fastcall Fum1CfgClick(TObject *Sender);
   void __fastcall Fum2CfgClick(TObject *Sender);
   void __fastcall PO21CfgClick(TObject *Sender);
   void __fastcall PO22CfgClick(TObject *Sender);
   void __fastcall Oxid1CfgClick(TObject *Sender);
   void __fastcall Oxid2CfgClick(TObject *Sender);

   void __fastcall ExternalUpdateCheckClick(TObject *Sender);
   void __fastcall UpdateButtonClick(TObject *Sender);
   void __fastcall Incl1Change(TObject *Sender);
   void __fastcall Incl2Change(TObject *Sender);
   void __fastcall VBat1Change(TObject *Sender);
   void __fastcall TA1Change(TObject *Sender);
   void __fastcall UT1Change(TObject *Sender);
   void __fastcall TF01Change(TObject *Sender);
   void __fastcall E1Change(TObject *Sender);
   void __fastcall PO21Change(TObject *Sender);
   void __fastcall Coeff1Change(TObject *Sender);
   void __fastcall VBat2Change(TObject *Sender);
   void __fastcall TA2Change(TObject *Sender);
   void __fastcall UT2Change(TObject *Sender);
   void __fastcall TF02Change(TObject *Sender);
   void __fastcall E2Change(TObject *Sender);
   void __fastcall PO22Change(TObject *Sender);
   void __fastcall Coeff2Change(TObject *Sender);
   void __fastcall DefaultFormulaClick(TObject *Sender);
   
   
   void __fastcall InclFormulaMouseMove(TObject *Sender, TShiftState Shift, int X,
   int Y);
   void __fastcall CoefFormulaMouseMove(TObject *Sender, TShiftState Shift, int X,
   int Y);
   void __fastcall PO2FormulaMouseMove(TObject *Sender, TShiftState Shift, int X,
   int Y);
   void __fastcall EFormulaMouseMove(TObject *Sender, TShiftState Shift, int X,
   int Y);
   void __fastcall TFOFormulaMouseMove(TObject *Sender, TShiftState Shift, int X,
   int Y);
   void __fastcall UTFormulaMouseMove(TObject *Sender, TShiftState Shift, int X,
   int Y);
   void __fastcall TAFormulaMouseMove(TObject *Sender, TShiftState Shift, int X,
   int Y);
   void __fastcall VBATFormulaMouseMove(TObject *Sender, TShiftState Shift, int X,
   int Y);

private: // User declarations
   bool  passwordOK ;
   void  UpdateHours() ;
   void  UpdateMinutes() ;
   void  HideAllScaleControls() ;
   void  UpdateXScaleControls() ;
   void  UpdateYScaleControls() ;
   void  UpdateY2ScaleControls() ;
   TRegistry* theRegistry ;
   int   currentScaleSelection ;
   void  DisableY2Axis() ;
   void  EnableY2Axis() ;
   void  SaveOptions( int ) ;
   void  GetOptions( int ) ;

   // Formula fields change status
   bool  VBat1Dirty ;
   bool  VBat2Dirty ;
   bool  TA1Dirty ;
   bool  TA2Dirty ;
   bool  UT1Dirty ;
   bool  UT2Dirty ;
   bool  TF01Dirty ;
   bool  TF02Dirty ;
   bool  E1Dirty ;
   bool  E2Dirty ;
   bool  PO21Dirty ;
   bool  PO22Dirty ;
   bool  Coeff1Dirty ;
   bool  Coeff2Dirty ;
   bool  Incl1Dirty ;
   bool  Incl2Dirty ;

   // Formula fields identificator
   short currentFormulaEdit ;

public:     // User declarations
   __fastcall TConfigure(TComponent* Owner);
   void  TrimBlanks( AnsiString& ) ;
   bool  GetRegBool( AnsiString, bool ) ;
   float GetRegFloat( AnsiString, float ) ;
   AnsiString  GetRegString( AnsiString, AnsiString ) ;
   int   GetRegInteger( AnsiString, int ) ;
   void  SetRegString( AnsiString, AnsiString ) ;
   void  SetRegInteger( AnsiString, int ) ;
   void  SetRegFloat( AnsiString, float ) ;
   void  RegWorkWith( AnsiString, bool create = false ) ;
   bool  IsVBat1Dirty() ;
   bool  IsVBat2Dirty() ;
   bool  IsTA1Dirty() ;
   bool  IsTA2Dirty() ;
   bool  IsUT1Dirty() ;
   bool  IsUT2Dirty() ;
   bool  IsTF01Dirty() ;
   bool  IsTF02Dirty() ;
   bool  IsE1Dirty() ;
   bool  IsE2Dirty() ;
   bool  IsPO21Dirty() ;
   bool  IsPO22Dirty() ;
   bool  IsCoeff1Dirty() ;
   bool  IsCoeff2Dirty() ;
   bool  IsIncl1Dirty() ;
   bool  IsIncl2Dirty() ;

};
//---------------------------------------------------------------------------
extern TConfigure *Configure;
//---------------------------------------------------------------------------
#endif
