/************************************************************
*          This is a part of the VideoClick project.
*               Copyright (C) 2000 Vsoft Ltd.
*                   All rights reserved.
* NAME: DbgCunknown.cpp
* DESCRIPTION:
* debug functions added to CUnknown
* AUTHOR: Noam Cohen July 2000
*
* $Archive:  will be filled by sourceSafe  $
* $Revision:  will be filled by sourceSafe $
* $Author:  will be filled by sourceSafe    $
* $JustDate:   will be filled by sourceSafe $
************************************************************/


#include "DbgUtils.h"
#include "CUNKNOWN.h"
#include "RefCountingRot.h"

#include "SymbolHandler.h"
#include <stdio.h>

// These objects are here for historical reasons: The original 
// idea was to have singeltons, but linking issues are a burden 
// to big for me, so I let each COM object have its own set of helper
// objects.
RefCountingRot gRot; // running object table
SymbolHandler ghSym; // The symbols API wrapper 


///////////////////////////////////////////////////////////
//
// Count of active objects
//   - Use to determine if we can unload the DLL.
//
void CUnknown::RegisterAddRef(DWORD _ebp)
{
	ULONG line;
	ULONG displacement;
	char szSymName[255], szFile[255];
	ObjParams record;

	HRESULT hr = ParamsOfCaller(_ebp, line,szSymName,255, szFile, 255,  displacement);
	if(SUCCEEDED(hr))
	{
		record.functionName = strdup(szSymName);
		record.line = line;
		record.szFile = strdup(szFile);
	}
	else
	{
		TRACE("RegisterAddRef: Failed getting params of caller. code 0x%X\n", hr);
	}

	gRot.Register(this, record);
}

void CUnknown::RegisterRelease(DWORD _ebp)
{
	ULONG line;
	ULONG displacement;
	char szSymName[255], szFile[255];
	ObjParams record;

	HRESULT hr = ParamsOfCaller(_ebp, line,szSymName,255, szFile, 255,  displacement);
	if(SUCCEEDED(hr))
	{
		record.functionName = strdup(szSymName);
		record.line = line;
		record.szFile = strdup(szFile);
	}
	else
	{
		TRACE("RegisterRelease: Failed getting params of caller. code 0x%X\n", hr);
	}
	gRot.UnRegister(this, record);

}

HRESULT CUnknown::ParamsOfCaller(IN DWORD _ebp ,
					OUT ULONG &line,
					OUT char* szSymbolName, 
					IN	UINT	nSymNameLen,
					OUT	char* szFileName,
					INT /*nFileNameLen*/,
					OUT ULONG &displacement)
{
	// to get the address of the caller:
	UINT * pAddr = (UINT*)_ebp + 1; // note the pointer arithmetics: 1 == 4 bytes !!
	ULONG addr = *pAddr;

	HRESULT hr;
	hr = ghSym.GetLineByAddr(addr, &displacement, &line, szFileName);
	if(FAILED(hr))
		return hr;

	hr = ghSym.GetSymbolByAddr(addr,(char*)szSymbolName,nSymNameLen,&displacement);
	if(FAILED(hr))
		return hr;
	return S_OK;
}

