//  CLASS TTOOLBAR
//  Main class for the application. Everything starts here.
//  We create all of the 12 windows (derived from TChartForm)
//  and pass them the data, as well as maintain everything
//  affetcting those datas such as reading new incoming files, etc.
//
// (C) Steve JORDI - 1996-1999  
// steve@sjordi.com - www.sjordi.com
//
// All this code is copyrighted by Steve Jordi and
// cannot be reproduced without the consent of the author.
// The executable may be freely distributed.
//
// MAKE  SURE  TO  UNDERSTAND  AND  ACCEPT  THE STATEMENTS BELOW
// BEFORE USING THE SOFTWARE:
//
// THIS PROGRAM  IS  PROVIDED  "AS IS"  WITHOUT  WARRANTY OF ANY
// KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED
// TO  THE  IMPLIED  WARRANTIES  OF MERCHANTABILITY  AND FITNESS
// FOR  A  PARTICULAR PURPOSE. THE ENTIRE RISK AS TO THE QUALITY
// AND   PERFORMANCE  OF  THE  PROGRAM  IS  WITH YOU. SHOULD THE
// PROGRAM  PROVE   DEFECTIVE,  YOU   (NOT  STEVE JORDI)  ASSUME
// THE ENTIRE COST OF ALL  NECESSARY   SERVICING,   REPAIR,   OR
// CORRECTION. IN NO EVENT WILL STEVE JORDI BE LIABLE TO YOU FOR
// ANY  DAMAGES ,  INCLUDING ANY LOST PROFITS,  LOST SAVINGS, OR
// OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES  ARISING OUT OF THE
// USE OR INABILITY TO USE  SUCH  PROGRAM  EVEN  IF  STEVE JORDI
// HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES,   OR FOR
// ANY  CLAIM BY ANY OTHER PARTY.
// ALSO, USE IT IN LIFE THREATENING SITUATIONS AT YOUR OWN RISKS
// (SUCH AS VOLCANO MONITORING). STEVE JORDI CANNOT BE ACCOUNTED
// AS RESPONSIBLE FOR ANY LIFE OR PROPERTY LOST IN CASE AN ALARM
// SYSTEM HAD A MALFUNCTION OR WAS NOT ABLE TO CORRECTLY  BEHAVE
// IN A CRISIS ENVIRONMENT.
//---------------------------------------------------------------------------

#define __TRACE

#include <vcl\vcl.h>
#pragma hdrstop

#include "About.h"
#include "Chart.h"
#include "Main.h"
#include "UpdateInfo.h"
#include "Config.h"
#include "Spread.h"
//---------------------------------------------------------------------------
#pragma link "explbtn"
#pragma link "marsCap"

#pragma link "ExplBtn"
#pragma resource "*.dfm"
TMainToolBar *MainToolBar;
extern TMainSplash*  MainSplash ;


// --------------------------------------------------------
// CONSTRUCTOR
// Initialize all variables/components here
// --------------------------------------------------------
__fastcall TMainToolBar::TMainToolBar(TComponent* Owner)
   : TForm(Owner) {

   // Get a tool to read/write current options!
   Configure = new TConfigure( Owner ) ;


   // Initialize all chart instances
   Tilt1    = new TChartForm( this ) ; // Tiltmeter 1
   Tilt1Tan = new TChartForm( this ) ; // Tangential Tiltmeter 1
   Tilt2    = new TChartForm( this ) ; // Tiltmeter 2
   Tilt2Tan = new TChartForm( this ) ; // Tangential Tiltmeter 2
   PO2S1    = new TChartForm( this ) ; // O2 Pressure Sulfur 1
   PO2S2    = new TChartForm( this ) ; // O2 Pressure Sulfur 2
   TempFum1 = new TChartForm( this ) ; // Fumarol Temp 1
   TempFum2 = new TChartForm( this ) ; // Fumarol Temp 2
   Tilt1Batt= new TChartForm( this ) ; // Battery Level of Tiltmeter 1
   Tilt2Batt= new TChartForm( this ) ; // Battery Level of Tiltmeter 2
   OxidS1   = new TChartForm( this ) ; // Oxid Sulfur 1
   OxidS2   = new TChartForm( this ) ; // Oxid Sulfur 2
   BatteryForm = new TBatteryForm( this ) ; // All batteries at once
   clockMode = true ;
   newData   = false ;
   initDone = false ;


   // Create the charts
   // Initialize Tilt1
   MainSplash->InitStatus->Caption = "Initializing Radial Tiltmeters Charts" ;
   MainSplash->InitStatus->Repaint() ;
   Tilt1->SetDefaultPosition( Left, Top+Height+1, DEFAULTWIDTH, DEFAULTHEIGHT ) ;
   Tilt1->SetCaption( "TILT 1 - Radial" ) ;
   Tilt1->SetAxisTitle("Time", "-rad") ;
   Tilt1->SetNbSeries( 1 ) ;
   Tilt1->SetID( TILT1 ) ;
   MainSplash->SplashProgress->StepIt() ;

   // Initialize Tilt2
   Tilt2->SetDefaultPosition( Left+DEFAULTWIDTH+1, Top+Height+1, DEFAULTWIDTH, DEFAULTHEIGHT ) ;
   Tilt2->SetCaption( "TILT 2 - Radial" ) ;
   Tilt2->SetAxisTitle("Time", "-rad") ;
   Tilt2->SetNbSeries( 1 ) ;
   Tilt2->SetID( TILT2 ) ;

   // Initialize Tilt1Batt with 2 Y axis and 2 lines
   MainSplash->InitStatus->Caption = "Initializing Tiltmeters Batteries Charts" ;
   MainSplash->InitStatus->Repaint() ;
   Tilt1Batt->SetDefaultPosition( Left+2*DEFAULTWIDTH+1, Top+Height+1, DEFAULTWIDTH, DEFAULTHEIGHT ) ;
   Tilt1Batt->SetCaption( "Temp. & Batt. - Tilt 1" ) ;
   Tilt1Batt->SetNbSeries( 2 ) ;
   Tilt1Batt->SetAxisTitle("Time", "Batt [V]", "T Tilt[C]") ;
   Tilt1Batt->SetID( TILT1BATT ) ;

   // Initialize Tilt2Batt with 2 Y axis and 2 lines
   Tilt2Batt->SetDefaultPosition( Left+3*DEFAULTWIDTH+1, Top+Height+1, DEFAULTWIDTH, DEFAULTHEIGHT ) ;
   Tilt2Batt->SetCaption( "Temp. & Batt. - Tilt 2" ) ;
   Tilt2Batt->SetNbSeries( 2 ) ;
   Tilt2Batt->SetAxisTitle("Time", "Batt [V]", "T Tilt[C]") ;
   Tilt2Batt->SetID( TILT2BATT ) ;
   MainSplash->SplashProgress->StepIt() ;

   // Initialize Tilt1Tan
   MainSplash->InitStatus->Caption = "Initializing Tangential Tilts Charts" ;
   MainSplash->InitStatus->Repaint() ;
   Tilt1Tan->SetDefaultPosition( Left, Top+Height+1+DEFAULTHEIGHT, DEFAULTWIDTH, DEFAULTHEIGHT ) ;
   Tilt1Tan->SetCaption( "TILT 1 - Tang." ) ;
   Tilt1Tan->SetAxisTitle("Time", "-rad") ;
   Tilt1Tan->SetID( TILT1TAN ) ;
   Tilt1Tan->SetNbSeries( 1 ) ;
   MainSplash->SplashProgress->StepIt() ;

   // Initialize Tilt2Tan
   Tilt2Tan->SetDefaultPosition( Left+DEFAULTWIDTH+1, Top+Height+1+DEFAULTHEIGHT, DEFAULTWIDTH, DEFAULTHEIGHT ) ;
   Tilt2Tan->SetCaption( "TILT 2 - Tang." ) ;
   Tilt2Tan->SetAxisTitle("Time", "-rad") ;
   Tilt2Tan->SetID( TILT2TAN ) ;
   Tilt2Tan->SetNbSeries( 1 ) ;
   MainSplash->SplashProgress->StepIt() ;

   // Initialize Temp1Fum with 2 Y axis and 2 lines
   MainSplash->InitStatus->Caption = "Initializing Temperature Charts" ;
   MainSplash->InitStatus->Repaint() ;
   TempFum1->SetDefaultPosition( Left+2*DEFAULTWIDTH+1, Top+Height+1+DEFAULTHEIGHT, DEFAULTWIDTH, DEFAULTHEIGHT ) ;
   TempFum1->SetCaption( "Fum. Temp. - Drill 1" ) ;
   TempFum1->SetNbSeries( 2 ) ;
   TempFum1->SetAxisTitle("Time", "T Amb [C]", "T Fum [C]") ;
   TempFum1->SetID( TEMPFUM1 ) ;
   MainSplash->SplashProgress->StepIt() ;

   // Initialize Temp2Fum with 2 Y axis and 2 lines
   TempFum2->SetDefaultPosition( Left+3*DEFAULTWIDTH+1, Top+Height+1+DEFAULTHEIGHT, DEFAULTWIDTH, DEFAULTHEIGHT ) ;
   TempFum2->SetCaption( "Fum. Temp. - Drill 2" ) ;
   TempFum2->SetNbSeries( 2 ) ;
   TempFum2->SetAxisTitle("Time", "T Amb [C]", "T Fum [C]") ;
   TempFum2->SetID( TEMPFUM2 ) ;


   // Initialize PO2S1
   MainSplash->InitStatus->Caption = "Initializing P(O2) Charts" ;
   MainSplash->InitStatus->Repaint() ;
   PO2S1->SetDefaultPosition( Left, Top+Height+1+2*DEFAULTHEIGHT,  DEFAULTWIDTH, DEFAULTHEIGHT ) ;
   PO2S1->SetCaption( "P(O2) - Drill 1" ) ;
   PO2S1->SetAxisTitle("Time", "[atm]") ;
   PO2S1->SetID( PO2S1WIN ) ;
   PO2S1->SetNbSeries( 1 ) ;
   MainSplash->SplashProgress->StepIt() ;

   // Initialize PO2S2
   PO2S2->SetDefaultPosition( Left + DEFAULTWIDTH+1, Top+Height+1+2*DEFAULTHEIGHT,  DEFAULTWIDTH, DEFAULTHEIGHT ) ;
   PO2S2->SetCaption( "P(O2) - Drill 2" ) ;
   PO2S2->SetAxisTitle("Time", "[atm]") ;
   PO2S2->SetID( PO2S2WIN ) ;
   PO2S2->SetNbSeries( 1 ) ;
   MainSplash->SplashProgress->StepIt() ;

   // Initialize OxidS1 with 2 Y axis and 2 lines
   MainSplash->InitStatus->Caption = "Initializing Oxidation Charts" ;
   MainSplash->InitStatus->Repaint() ;
   OxidS1->SetDefaultPosition( Left+2*DEFAULTWIDTH+1, Top+Height+1+2*DEFAULTHEIGHT,  DEFAULTWIDTH, DEFAULTHEIGHT ) ;
   OxidS1->SetCaption( "Oxidation - Drill 1" ) ;
   OxidS1->SetNbSeries( 2 ) ;
   OxidS1->SetAxisTitle("Time", "Batt1 [V]", "C.Oxid") ;
   OxidS1->SetID( OXIDS1 ) ;

   // Initialize OxidS2 with 2 Y axis and 2 lines
   OxidS2->SetDefaultPosition( Left+3*DEFAULTWIDTH+1, Top+Height+1+2*DEFAULTHEIGHT,  DEFAULTWIDTH, DEFAULTHEIGHT ) ;
   OxidS2->SetCaption( "Oxidation - Drill 2" ) ;
   OxidS2->SetNbSeries( 2 ) ;
   OxidS2->SetAxisTitle("Time", "Batt1 [V]", "C.Oxid") ;
   OxidS2->SetID( OXIDS2 ) ;
   MainSplash->SplashProgress->StepIt() ;
   MainSplash->InitStatus->Caption = "Showing Charts" ;
   MainSplash->InitStatus->Repaint() ;


   // Now display those windows
   Tilt1->Show() ;
   Tilt2->Show() ;
   Tilt1Batt->Show() ;
   Tilt2Batt->Show() ;
   Tilt1Tan->Show() ;
   Tilt2Tan->Show() ;
   TempFum1->Show() ;
   TempFum2->Show() ;
   PO2S1->Show() ;
   PO2S2->Show() ;
   MainSplash->SplashProgress->StepIt() ;
   OxidS1->Show() ;
   OxidS2->Show() ;
   MainSplash->InitStatus->Caption = "Finalizing Initialization" ;
   MainSplash->InitStatus->Repaint() ;

   // Make Tilt1 the default one.
   Tilt1->SetFocus() ;
   MainSplash->SplashProgress->StepIt() ;

   // Init the toolbar itself
   MinimizeButton->Enabled  = false ; // Can't minimize by default
   MaximizeButton->Enabled  = true ;  // Maximize current window is possible
   currentActiveWindow      = TILT1 ; // Make Tilt1 the default win in memory
   currentDisplay           = ALL ;   // Display mode: (o) All ( ) Tilts ( )...

   MainSplash->InitStatus->Caption = "Now reading all data..." ;

   // if we are not in the init mode, this is called by the timer
   // to check whether we have new data or not. If yes, update them!
   // If in init mode, set the timer properties
   if( !initDone ) {
      initDone = true ;
      // Update data files
      ProcessFiles() ;
      MainSplash->Hide() ;
      delete MainSplash ;
      MainSplash = 0 ;
      // And eventually display warning message (if any).
      if( initMessage != "" ) {
         initMessage = "Please define the directories for the " + initMessage +
                  " stations." ;
         MessageBox( 0, initMessage.c_str(), "Data Error", MB_OK ) ;
      }
   }
   else {
      // Setup the timer
      int hr, mn ;
      Configure->RegWorkWith( "Updating" ) ;
      hr = Configure->GetRegInteger( "Hour", 1 ) ;
      mn = Configure->GetRegInteger( "Minute", 0 ) ;
      // Calculate next possible update
      TDateTime voidTime( 0, (WORD)mn, 0, 0 ) ;
      // Get current time
      TDateTime currentDate( Date() ) ;
      TDateTime currentTime( Time() ) ;
      nextUpdate = currentDate+voidTime ;
      while( (currentDate+currentTime)>nextUpdate )
         nextUpdate += 1.0/24.0*hr ;
   }
}



// EXITBUTTONCLICK
// Quit the application
// --------------------------------------------------------
void __fastcall TMainToolBar::ExitButtonClick(TObject *Sender) {
   // Just quit the application
   Close() ;
}

// --------------------------------------------------------
// MAXIMIZEBUTTONCLICK
// Maximize the selected window
// --------------------------------------------------------
void __fastcall TMainToolBar::MaximizeButtonClick(TObject *Sender) {
   // Maximize the current window to full 800x600 screen
   // This will automatically enable the MINIMIZE button
   // by a call from within the maximize() function.
   switch( currentActiveWindow ) {
      case TILT1:
         Tilt1->Maximize() ;
         break ;

      case TILT2:
         Tilt2->Maximize() ;
         break ;

      case TILT1BATT:
         Tilt1Batt->Maximize() ;
         break ;

      case TILT2BATT:
         Tilt2Batt->Maximize() ;
         break ;

      case TILT1TAN:
         Tilt1Tan->Maximize() ;
         break ;

      case TILT2TAN:
         Tilt2Tan->Maximize() ;
         break ;

      case TEMPFUM1:
         TempFum1->Maximize() ;
         break ;

      case TEMPFUM2:
         TempFum2->Maximize() ;
         break ;

      case PO2S1WIN:
         PO2S1->Maximize() ;
         break ;

      case PO2S2WIN:
         PO2S2->Maximize() ;
         break ;

      case OXIDS1:
         OxidS1->Maximize() ;
         break ;

      case OXIDS2:
         OxidS2->Maximize() ;
         break ;
   }
}



// --------------------------------------------------------
// TILTMETERSBUTTONCLICK
// We have to hide the screen into the 4 windows
// displaying specific information on Tiltmeters
// --------------------------------------------------------
void __fastcall TMainToolBar::TiltmetersButtonClick(TObject *Sender) {
   // Enlarge the 4 Tiltmeters ones
   // Locally store current mode of display
   currentDisplay = TILTMETERS ;

   // Show up the tile and horizontal windows button.
   TileButton->Visible = true ;
   HorizontalButton->Visible = true ;

   // Now display the new 4 windows accordingly to the
   // chosen style (tiled or horizontally).
   bool type = TileButton->Down ;
   Tilt1->SelfPosition( 1, type ) ; // 1 is the 1st window
   Tilt2->SelfPosition( 2, type ) ; // 2 is the 2nd, etc
   Tilt1Tan->SelfPosition( 3, type ) ;
   Tilt2Tan->SelfPosition( 4, type ) ;
   // Make Tilt1 the default window for that view.
   Tilt1->SetFocus() ;
}


// --------------------------------------------------------
// ARRANGEBUTTONCLICK
// Reorganize the windows by moving them all to their
// default positions and sizes
// --------------------------------------------------------
void __fastcall TMainToolBar::ArrangeButtonClick(TObject *Sender) {
   // Locally store local display mode
   currentDisplay = ALL ;

   // Just tell each window to minimize itself. They will
   // all resize and position to their defaults.
   Tilt1->Minimize() ;
   Tilt2->Minimize() ;
   PO2S1->Minimize() ;
   PO2S2->Minimize() ;
   Tilt1Tan->Minimize() ;
   Tilt2Tan->Minimize() ;
   TempFum1->Minimize() ;
   TempFum2->Minimize() ;
   Tilt1Batt->Minimize() ;
   Tilt2Batt->Minimize() ;
   OxidS1->Minimize() ;
   OxidS2->Minimize() ;

   // Update toolbar buttons status
   TiltmetersButton->Down = false ;
   OxidationButton->Down  = false ;
   TemperaturesButton->Down = false ;
   TileButton->Visible = false ;
   HorizontalButton->Visible = false ;
}



// --------------------------------------------------------
// TILEBUTTONCLICK
// This button is down by default. It tells the application
// to display the windows in a tiled manner if the user
// clicks on either Titlemeters, P(O2) or Misc.
// Also it immediately updates the windows if clicked.
// It is used for both a click on the tile or horizontal
// buttons. Since they do the same thing and the configuration
// is based on the tile button state (up or down).
// --------------------------------------------------------
void __fastcall TMainToolBar::TileButtonClick(TObject *Sender) {
   // Tell each window to tile or stack up themselves
   switch( currentDisplay) {
      case TILTMETERS:
         TiltmetersButtonClick( Sender ) ;
         break ;
      case PO2:
         OxidationButtonClick( Sender ) ;
         break ;
      case TEMPERATURES:
         TemperaturesButtonClick( Sender ) ;
         break ;
   }
}


// --------------------------------------------------------
// HORIZONTALBUTTONCLICK
// This button is down by default. It tells the application
// to display the windows in a stacked manner if the user
// clicks on either Titlemeters, P(O2) or Misc.
// Also it immediately updates the windows if clicked.
// Just call TileButtonClick as it does the same as this one.
// --------------------------------------------------------
void __fastcall TMainToolBar::HorizontalButtonClick(TObject *Sender) {
   // Tell each tilt window to stackup themselves
   TileButtonClick( Sender ) ;
}


// --------------------------------------------------------
// MINIMIZEBUTTONCLICK
// Minimize the current window to its default position.
// This means it is either a small one or a 1/4 of screen
// if we have a specific view like Tilts only, Misc only or
// Oxid only.
// --------------------------------------------------------
void __fastcall TMainToolBar::MinimizeButtonClick(TObject *Sender) {
   int type = TileButton->Down ;
   switch( currentDisplay ) {
      // Are we in ALL WINDOWS mode?
      case ALL:
         switch( currentActiveWindow ) {
            case TILT1:
               Tilt1->Minimize() ;
               break ;
            case TILT2:
               Tilt2->Minimize() ;
               break ;
            case PO2S1WIN:
               PO2S1->Minimize() ;
               break ;
            case PO2S2WIN:
               PO2S2->Minimize() ;
               break ;
            case TILT1TAN:
               Tilt1Tan->Minimize() ;
               break ;
            case TILT2TAN:
               Tilt2Tan->Minimize() ;
               break ;
            case TEMPFUM1:
               TempFum1->Minimize() ;
               break ;
            case TEMPFUM2:
               TempFum2->Minimize() ;
               break ;
            case TILT1BATT:
               Tilt1Batt->Minimize() ;
               break ;
            case TILT2BATT:
               Tilt2Batt->Minimize() ;
               break ;
            case OXIDS1:
               OxidS1->Minimize() ;
               break ;
            case OXIDS2:
               OxidS2->Minimize() ;
               break ;
         }
         break ;

      // We're in TILTMETERS only mode
      case TILTMETERS:
         switch( currentActiveWindow ) {
            case TILT1:
               Tilt1->SelfPosition(1, type) ;
               break ;
            case TILT2:
               Tilt2->SelfPosition(2, type) ;
               break ;
            case TILT1TAN:
               Tilt1Tan->SelfPosition(3, type) ;
               break ;
            case TILT2TAN:
               Tilt2Tan->SelfPosition(4, type) ;
               break ;
         }
         break ;

      // We are in PO2 only mode
      case PO2:
         switch( currentActiveWindow ) {
            case PO2S1WIN:
               PO2S1->SelfPosition(1, type) ;
               break ;
            case PO2S2WIN:
               PO2S2->SelfPosition(2, type) ;
               break ;
            case OXIDS1:
               OxidS1->SelfPosition(3, type) ;
               break ;
            case OXIDS2:
               OxidS2->SelfPosition(4, type) ;
               break ;
         }
         break ;

      // We are in TEMPERATURES only mode
      case TEMPERATURES:
         switch( currentActiveWindow ) {
            case TEMPFUM1:
               TempFum1->SelfPosition(1, type) ;
               break ;
            case TEMPFUM2:
               TempFum2->SelfPosition(2, type) ;
               break ;
            case TILT1BATT:
               Tilt1Batt->SelfPosition(3, type) ;
               break ;
            case TILT2BATT:
               Tilt2Batt->SelfPosition(4, type) ;
               break ;
         }
         break ;
   }
}


// --------------------------------------------------------
// Set Current Window
// Called by all charts once they are activated to
// keep the main process informed on which window is
// currently active.
// --------------------------------------------------------
void TMainToolBar::SetCurrentWindow( int winID ) {
   currentActiveWindow = winID ;
}


// --------------------------------------------------------
// OXIDATIONBUTTONCLICK
// We have to show only the 4 charts with oxidation and
// fumarolic pressures. Grow them and put them on top
// of all others.
// --------------------------------------------------------
void __fastcall TMainToolBar::OxidationButtonClick(TObject *Sender) {
   // Then enlarge the 4 Tiltmeters ones
   currentDisplay = PO2 ;

   TileButton->Visible = true ;
   HorizontalButton->Visible = true ;
   bool type = TileButton->Down ;
   PO2S1->SelfPosition( 1, type ) ;
   PO2S2->SelfPosition( 2, type ) ;
   OxidS1->SelfPosition( 3, type ) ;
   OxidS2->SelfPosition( 4, type ) ;
   PO2S1->SetFocus() ;
}


// --------------------------------------------------------
// TEMPERATURESBUTTONCLICK
// We have to show only the 4 charts with temperature.
// Grow them and put them on top of all others.
// --------------------------------------------------------
void __fastcall TMainToolBar::TemperaturesButtonClick(TObject *Sender) {
   // Then enlarge the 4 Tiltmeters ones
   currentDisplay = TEMPERATURES ;

   TileButton->Visible = true ;
   HorizontalButton->Visible = true ;
   bool type = TileButton->Down ;
   Tilt1Batt->SelfPosition( 1, type ) ;
   Tilt2Batt->SelfPosition( 2, type ) ;
   TempFum1->SelfPosition( 3, type ) ;
   TempFum2->SelfPosition( 4, type ) ;
   Tilt1Batt->SetFocus() ;
}



// --------------------------------------------------------
// HELPBUTTONCLICK
// Display the About box.
// --------------------------------------------------------
void __fastcall TMainToolBar::HelpButtonClick(TObject *Sender) {
   AboutForm->ShowModal() ;
}


// --------------------------------------------------------
// CONFIGBUTTONCLICK
// Open the configuration window.
// If the user pressed ok, save data to the registry.
// If cancel, do nothing.
// --------------------------------------------------------
void __fastcall TMainToolBar::ConfigButtonClick(TObject *Sender) {
   Configure->ShowModal() ;
   // Should we update the info?
   if( Configure->ModalResult == mrOk) {
      // Update the formulas if they have been changed
      // Be carefull with variable dependencies!!!
      // Was VBat1 changed?
      //   Yes? Update TiltBatt, Oxid Coef
      if( Configure->IsVBat1Dirty() ) {
         InfoForm->Show() ;
         InfoForm->InfoLabel->Caption = "Updating formulas for TILT 1 BATT" ;
         InfoForm->InfoLabel->Repaint() ;
         Tilt1Batt->PrepareDataUpdate() ;
         Tilt1Batt->UpdateData( false ) ;
         Tilt1Batt->UpdateChart() ; // Refresh
         InfoForm->InfoLabel->Caption = "Updating formulas for OXIDATION 1" ;
         InfoForm->InfoLabel->Repaint() ;
         OxidS1->PrepareDataUpdate() ;
         OxidS1->UpdateData( false ) ;
         OxidS1->UpdateChart() ; // Refresh
      }
      if( Configure->IsVBat2Dirty() ) {
         if( !InfoForm->IsShown() )
            InfoForm->Show() ;
         InfoForm->InfoLabel->Caption = "Updating formulas for TILT 2 BATT" ;
         InfoForm->InfoLabel->Repaint() ;
         Tilt2Batt->PrepareDataUpdate() ;
         Tilt2Batt->UpdateData( false ) ;
         Tilt2Batt->UpdateChart() ; // Refresh
         InfoForm->InfoLabel->Caption = "Updating formulas for OXIDATION 2" ;
         InfoForm->InfoLabel->Repaint() ;
         OxidS2->PrepareDataUpdate() ;
         OxidS2->UpdateData( false ) ;
         OxidS2->UpdateChart() ; // Refresh
      }

      // Was Incl changed?
      //   Yes? Update Tilt and TiltTan
      if( Configure->IsIncl1Dirty() ) {
         if( !InfoForm->IsShown() )
            InfoForm->Show() ;
         InfoForm->InfoLabel->Caption = "Updating formulas for TILT 1" ;
         InfoForm->InfoLabel->Repaint() ;
         Tilt1->PrepareDataUpdate() ;
         Tilt1->UpdateData( false ) ;
         Tilt1->UpdateChart() ; // Refresh
         InfoForm->InfoLabel->Caption = "Updating formulas for TILT 1 TANG" ;
         InfoForm->InfoLabel->Repaint() ;
         Tilt1Tan->PrepareDataUpdate() ;
         Tilt1Tan->UpdateData( false ) ;
         Tilt1Tan->UpdateChart() ; // Refresh
      }
      if( Configure->IsIncl2Dirty() ) {
         if( !InfoForm->IsShown() )
            InfoForm->Show() ;
         InfoForm->InfoLabel->Caption = "Updating formulas for TILT 2" ;
         InfoForm->InfoLabel->Repaint() ;
         Tilt2->PrepareDataUpdate() ;
         Tilt2->UpdateData( false ) ;
         Tilt2->UpdateChart() ; // Refresh
         InfoForm->InfoLabel->Caption = "Updating formulas for TILT 2 TANG" ;
         InfoForm->InfoLabel->Repaint() ;
         Tilt2Tan->PrepareDataUpdate() ;
         Tilt2Tan->UpdateData( false ) ;
         Tilt2Tan->UpdateChart() ; // Refresh
      }

      // TA, UT, TF0, E, PO2 or Coeff was changed?
      // Impact is on TiltBatt, TempFum, PO2, and Coeff
      if( Configure->IsTA1Dirty()  || Configure->IsUT1Dirty() ||
          Configure->IsTF01Dirty() || Configure->IsE1Dirty()  ||
          Configure->IsPO21Dirty() || Configure->IsCoeff1Dirty() ) {
         if( !InfoForm->IsShown() )
            InfoForm->Show() ;
         InfoForm->InfoLabel->Caption = "Updating formulas for TILT 1 BATT" ;
         InfoForm->InfoLabel->Repaint() ;
         Tilt1Batt->PrepareDataUpdate() ;
         Tilt1Batt->UpdateData( false ) ;
         Tilt1Batt->UpdateChart() ; // Refresh
         InfoForm->InfoLabel->Caption = "Updating formulas for TEMP FUMAROLS 1" ;
         InfoForm->InfoLabel->Repaint() ;
         TempFum1->PrepareDataUpdate() ;
         TempFum1->UpdateData( false ) ;
         TempFum1->UpdateChart() ; // Refresh
         InfoForm->InfoLabel->Caption = "Updating formulas for PO2 DRILL 1" ;
         InfoForm->InfoLabel->Repaint() ;
         PO2S1->PrepareDataUpdate() ;
         PO2S1->UpdateData( false ) ;
         PO2S1->UpdateChart() ; // Refresh
         InfoForm->InfoLabel->Caption = "Updating formulas for OXIDATION 1" ;
         InfoForm->InfoLabel->Repaint() ;
         OxidS1->PrepareDataUpdate() ;
         OxidS1->UpdateData( false ) ;
         OxidS1->UpdateChart() ; // Refresh
      }

      if( Configure->IsTA2Dirty()  || Configure->IsUT2Dirty() ||
          Configure->IsTF02Dirty() || Configure->IsE2Dirty()  ||
          Configure->IsPO22Dirty() || Configure->IsCoeff2Dirty() ) {
         if( !InfoForm->IsShown() )
            InfoForm->Show() ;
         InfoForm->InfoLabel->Caption = "Updating formulas for TILT 2 BATT" ;
         InfoForm->InfoLabel->Repaint() ;
         Tilt2Batt->PrepareDataUpdate() ;
         Tilt2Batt->UpdateData( false ) ;
         Tilt2Batt->UpdateChart() ; // Refresh
         InfoForm->InfoLabel->Caption = "Updating formulas for TEMP FUMAROLS 2" ;
         InfoForm->InfoLabel->Repaint() ;
         TempFum2->PrepareDataUpdate() ;
         TempFum2->UpdateData( false ) ;
         TempFum2->UpdateChart() ; // Refresh
         InfoForm->InfoLabel->Caption = "Updating formulas for PO2 DRILL 2" ;
         InfoForm->InfoLabel->Repaint() ;
         PO2S2->PrepareDataUpdate() ;
         PO2S2->UpdateData( false ) ;
         PO2S2->UpdateChart() ; // Refresh
         InfoForm->InfoLabel->Caption = "Updating formulas for OXIDATION 2" ;
         InfoForm->InfoLabel->Repaint() ;
         OxidS2->PrepareDataUpdate() ;
         OxidS2->UpdateData( false ) ;
         OxidS2->UpdateChart() ; // Refresh
      }

      // Update the axis if required
      // (log or linear?)
      Tilt1->CheckAxisLog() ;
      Tilt2->CheckAxisLog() ;
      Tilt1Tan->CheckAxisLog() ;
      Tilt2Tan->CheckAxisLog() ;
      TempFum1->CheckAxisLog() ;
      TempFum2->CheckAxisLog() ;
      Tilt1Batt->CheckAxisLog() ;
      Tilt2Batt->CheckAxisLog() ;
      OxidS1->CheckAxisLog() ;
      OxidS2->CheckAxisLog() ;
      PO2S1->CheckAxisLog() ;
      PO2S2->CheckAxisLog() ;

   if( InfoForm->IsShown() )
      InfoForm->Close() ;
   }


   // Reset the timer
   int hr, mn ;
   Configure->RegWorkWith( "Updating" ) ;
   hr = Configure->GetRegInteger( "Hour", 1 ) ;
   mn = Configure->GetRegInteger( "Minute", 0 ) ;
   // Calculate next possible update
   TDateTime voidTime( 0, (WORD)mn, 0, 0 ) ;
   // Get current time
   TDateTime currentDate( Date() ) ;
   TDateTime currentTime( Time() ) ;
   nextUpdate = currentDate+voidTime ;
   while( (currentDate+currentTime)>nextUpdate )
      nextUpdate += 1.0/24.0*hr ;
}



// --------------------------------------------------------
// DATABUTTONCLICK
// Display the data associated to selected window
// --------------------------------------------------------
void __fastcall TMainToolBar::DataButtonClick(TObject *Sender) {
   SpreadForm->ShowModal() ;
}


// --------------------------------------------------------
// PROCESSFILES
// This is called to update data. It is called first from
// FORMPAINT to update the data at launch time.
// But also, the timer calls it regularly to update the new
// data!
// Files are prefixed as follows:
//    CRA for Crater stations
//    TILTx for tiltmeters
// --------------------------------------------------------
void TMainToolBar::ProcessFiles() {
      craSucceed    = false ;
      tilt1Succeed  = false ;
      tilt2Succeed  = false ;
      prefix = "" ;
      initMessage = "" ;
      dir    = "" ;
      
      // Read all data from the CR files
      Configure->RegWorkWith("Directories") ;

      // Read Crater data
      MainSplash->InitStatus->Caption = "Updating CRATER data" ;
      MainSplash->InitStatus->Repaint() ;
      prefix = Configure->GetRegString( "CraterPrefix", "CR" ) ;
      dir = Configure->GetRegString( "CRA", "none" ) ;
      firstCompare = true ;
      ReadFiles( dir, prefix, CRATER ) ;

      // Read TILT1 data
      MainSplash->InitStatus->Caption = "Updating TILT1 data" ;
      MainSplash->InitStatus->Repaint() ;
      dir = Configure->GetRegString( "TILT1", "none" ) ;
      prefix = Configure->GetRegString( "Tilt1Prefix", "T1" ) ;
      firstCompare = true ;
      ReadFiles( dir, prefix, TILT1 ) ;

      // Read TILT2 data
      MainSplash->InitStatus->Caption = "Updating TILT2 data" ;
      MainSplash->InitStatus->Repaint() ;
      dir = Configure->GetRegString( "TILT2", "none" ) ;
      prefix = Configure->GetRegString( "Tilt2Prefix", "T2" ) ;
      firstCompare = true ;
      ReadFiles( dir, prefix, TILT2 ) ;

      // Did we read anything without problem?
      initMessage = "" ;
      if( !craSucceed )
         initMessage += "Crater " ;
      if( !tilt1Succeed )
         initMessage += "Tilt1 " ;
      if( !tilt2Succeed )
         initMessage += "Tilt2 " ;

      // Add the data to the charts
      MainSplash->SplashProgress->Max = 12 ;
      MainSplash->SplashProgress->Position = 0 ;
      MainSplash->InitStatus->Caption = "Passing data to TILT1" ;
      MainSplash->InitStatus->Repaint() ;
      MainSplash->SplashProgress->StepIt() ;
      Tilt1->UpdateData( true ) ; // Both x and Y
      MainSplash->InitStatus->Caption = "Passing data to TILT2" ;
      MainSplash->InitStatus->Repaint() ;
      MainSplash->SplashProgress->StepIt() ;
      Tilt2->UpdateData( true ) ;
      MainSplash->InitStatus->Caption = "Passing data to TILT1TAN" ;
      MainSplash->InitStatus->Repaint() ;
      MainSplash->SplashProgress->StepIt() ;
      Tilt1Tan->UpdateData( true ) ;
      MainSplash->InitStatus->Caption = "Passing data to TILT2TAN" ;
      MainSplash->InitStatus->Repaint() ;
      MainSplash->SplashProgress->StepIt() ;
      Tilt2Tan->UpdateData( true ) ;
      MainSplash->InitStatus->Caption = "Passing data to TILT1BATT" ;
      MainSplash->InitStatus->Repaint() ;
      MainSplash->SplashProgress->StepIt() ;
      Tilt1Batt->UpdateData( true ) ;
      MainSplash->InitStatus->Caption = "Passing data to TILT2BATT" ;
      MainSplash->InitStatus->Repaint() ;
      MainSplash->SplashProgress->StepIt() ;
      Tilt2Batt->UpdateData( true ) ;
      MainSplash->InitStatus->Caption = "Passing data to TEMP FUM 1" ;
      MainSplash->InitStatus->Repaint() ;
      MainSplash->SplashProgress->StepIt() ;
      TempFum1->UpdateData( true ) ;
      MainSplash->InitStatus->Caption = "Passing data to TEMP FUM 2" ;
      MainSplash->InitStatus->Repaint() ;
      MainSplash->SplashProgress->StepIt() ;
      TempFum2->UpdateData( true ) ;
      MainSplash->InitStatus->Caption = "Passing data to P(O2) 1" ;
      MainSplash->InitStatus->Repaint() ;
      MainSplash->SplashProgress->StepIt() ;
      PO2S1->UpdateData( true ) ;
      MainSplash->InitStatus->Caption = "Passing data to P(O2) 2" ;
      MainSplash->InitStatus->Repaint() ;
      MainSplash->SplashProgress->StepIt() ;
      PO2S2->UpdateData( true ) ;
      MainSplash->InitStatus->Caption = "Passing data to Oxid 1" ;
      MainSplash->InitStatus->Repaint() ;
      MainSplash->SplashProgress->StepIt() ;
      OxidS1->UpdateData( true ) ;
      MainSplash->InitStatus->Caption = "Passing data to Oxid 2" ;
      MainSplash->InitStatus->Repaint() ;
      MainSplash->SplashProgress->StepIt() ;
      OxidS2->UpdateData( true ) ;
      MainSplash->InitStatus->Caption = "-- DONE --" ;
      MainSplash->InitStatus->Repaint() ;

      // Enable timer
      ClockTimer->Enabled = true ;
}




// --------------------------------------------------------
// CLOCKTIMERTIMER
// The function called each second by thetimer
// It is here that we check whether one should update the
// charts or not by comparing the update time from the
// configuration settings and the current time.
// --------------------------------------------------------
void __fastcall TMainToolBar::ClockTimerTimer(TObject *Sender) {
   // Update time and also check whether one should update the data
   // Get current time and compare it with with the next update time
   TDateTime currentDate( Date() ) ;
   TDateTime currentTime( Time() ) ;
   if( (currentDate+currentTime) <= nextUpdate )
      return ; // Do nothing

   // Otherwise, calculate the next update
   int hr ;
   UpdateBanner->Visible = true ;
   UpdateBanner->Repaint() ;

   Configure->RegWorkWith( "Updating" ) ;
   hr = Configure->GetRegInteger( "Hour", 0 ) ;
   // Calculate next possible update
   nextUpdate += (double)(1.0/24.0*(double)hr) ;

   if( nextUpdate >= 1.0 )
      nextUpdate -= 1.0;   // Get only fractional part.

   // Finally, perform what we were supposed to do.
   // Two possibilities.
   // We have a concurrent application that prepares the data
   // for us and we just update the info
   // Or we have to launch the data updating application, wait
   // for it to give up, and then only update the data.

   // So check if we have to run an application
   if( Configure->GetRegBool( "UseExternal", false ) ) { //YES!!
      AnsiString fileToExecute = Configure->GetRegString( "ExternalApplication", "none" );
      if( fileToExecute != "none" ) {
         // CAll it and WAIT FOR IT TO FINISH!
         STARTUPINFO si ;
         PROCESS_INFORMATION pi ;
         ZeroMemory( &si, sizeof( si ) ) ;
         si.cb = sizeof( si ) ;
         if( !CreateProcess( NULL, fileToExecute.c_str(),
               NULL, NULL, false, 0, NULL, NULL, &si, &pi ) )

            // Error
            MessageBox( 0 , "Can't execute external application",
               "Updating Error", MB_OK ) ;
         else
            // Ok, now just wait for the application to close
            WaitForSingleObject( pi.hProcess, INFINITE ) ;
      }
   }



   // Read all data from the CR files
   Configure->RegWorkWith("Directories") ;

   // Update data this time
   newData = true ;

   // Read Crater data
   prefix = Configure->GetRegString( "CraterPrefix", "CR" ) ;
   dir = Configure->GetRegString( "CRA", "none" ) ;
   ReadFiles( dir, prefix, CRATER ) ;

   // Read TILT1 data
   dir = Configure->GetRegString( "TILT1", "none" ) ;
   prefix = Configure->GetRegString( "Tilt1Prefix", "T1" ) ;
   ReadFiles( dir, prefix, TILT1 ) ;

   // Read TILT2 data
   dir = Configure->GetRegString( "TILT2", "none" ) ;
   prefix = Configure->GetRegString( "Tilt2Prefix", "T2" ) ;
   ReadFiles( dir, prefix, TILT2 ) ;

   // Finally update data
   Tilt1->UpdateData( true, true ) ;
   Tilt2->UpdateData( true, true ) ;
   PO2S1->UpdateData( true, true ) ;
   PO2S2->UpdateData( true, true ) ;
   Tilt1Tan->UpdateData( true, true ) ;
   Tilt2Tan->UpdateData( true, true ) ;
   TempFum1->UpdateData( true, true ) ;
   TempFum2->UpdateData( true, true ) ;
   Tilt1Batt->UpdateData( true, true ) ;
   Tilt2Batt->UpdateData( true, true ) ;
   OxidS1->UpdateData( true, true ) ;
   OxidS2->UpdateData( true, true ) ;

   UpdateBanner->Visible = false ;
   UpdateBanner->Repaint() ;
}



// --------------------------------------------------------
// ReadFiles
// Read the data from each file.
// How do we proceed? It is simple. We get the current date
// and time and start from next month.
// If we have a monthly file, read it,
//    else look if we have daily files and read them
//       else look for any hourly files.
// This way we are sure to read all of the files, including
// the ones that may suddenly appear in the middle of
// nowhere, for example, after the acquisition was down
// for several days.
// If we already have our own .dat file, just start the
// search the same way, but start with last date and time
// (which is stored in the registy) so you scan just for
// new files.
// --------------------------------------------------------
void TMainToolBar::ReadFiles( AnsiString dir, AnsiString prefix, int type ) {
   if( dir == "none" )
      return ;

   AnsiString outName ;
   AnsiString newName ; // For new data

   lastHour = lastMinute = lastDay = lastMonth = lastYear = 0 ;

   dataType = type ;

   // Retrieve default directory
   AnsiString mainPath ;
   mainPath = ExtractFilePath( Application->ExeName ) ;

   bool  appending = false ;  // Only appending data?

   switch( dataType ) {
      case CRATER:
         craSucceed = true ;
         outName = mainPath + "Crater.dat" ;
         if( newData )
            newName = mainPath + "NewCra.dat"  ;
         break ;
      case TILT1:
         tilt1Succeed = true ;
         outName = mainPath + "Tilt1.dat" ;
         if( newData )
            newName = mainPath + "NewT1.dat"  ;
         break ;
      case TILT2:
         tilt2Succeed = true ;
         outName = mainPath + "Tilt2.dat" ;
         if( newData )
            newName = mainPath + "NewT2.dat"  ;
         break ;
   }

   fileName = dir + "\\" + prefix + "_*.*" ; // CR_*.* etc...
   if( !FileExists( fileName ) ) {
      switch( dataType ) {
         case CRATER:
            MainSplash->InitStatus->Caption = "No CRATER data" ;
            MainSplash->InitStatus->Repaint() ;
            break ;
         case TILT1:
            MainSplash->InitStatus->Caption = "No TILT1 data" ;
            MainSplash->InitStatus->Repaint() ;
            break ;
         case TILT2:
            MainSplash->InitStatus->Caption = "No TILT2 data" ;
            MainSplash->InitStatus->Repaint() ;
            break ;
      }
      return ; // Exit
   }

   // Do we already have a .dat file???
   if( FileExists( outName ) ) {
      // Position ourselves on the last file in the dir list
      GetLastInfo() ; // From Registry
      appending = true ;
      newData = true ;
      firstCompare = false ;
      fileName = UpperCase( fileName ) ;
   }
   else {
      // Get current date and time
      DecodeDate( Date(), lastYear, lastMonth, lastDay ) ;
      DecodeTime( Time(), lastHour, Minute, Second, MSec ) ;
      lastMonth++ ;
      if( lastMonth > 12 )
            lastMonth = 1 ;
   }


   // Create data file or open it for append.
   if( appending )
      outFile.open( outName.c_str(), ios::out | ios::app ) ; // Append
   else
      outFile.open( outName.c_str(), ios::out ) ; // Create

   if( newData )
      newFile.open( newName.c_str(), ios::out ) ;

   // Start searching for each file. If found, open, read data and
   // store new values.
   // Get current date and time
   DecodeDate( Date(), Year, Month, Day ) ;
   DecodeTime( Time(), Hour, Minute, Second, MSec ) ;

   /// Start with next month!
   if( lastMonth == 0 ) {
      lastMonth = Month+1 ;
      if( lastMonth > 12 )
         lastMonth = 1 ;
   }

   UpdateFiles() ;

   // Close the file
   outFile.close() ;
   if( newFile )
      newFile.close() ;
}




// --------------------------------------------------------
// Display the battery form
// It just shows the latest value read from the batteries
// --------------------------------------------------------
void __fastcall TMainToolBar::BatteryButtonClick(TObject *Sender) {
   BatteryForm->ShowModal() ;

}


// --------------------------------------------------------
// ScanData
// This actually adds the content of the specified file to
// the currently opened .dat file.
// If newData is true, it means that it is an update. We
// also store the new info into a different file which will
// contain only that new addition. It is not an appended
// file!
// --------------------------------------------------------
void TMainToolBar::ScanData( char* fln ) {
   char  theLine[255] ;
   fstream inFile ; // Used to read data

   // Read file
   inFile.open( fln, ios::in ) ;

   while( !inFile.eof() && inFile.good() ) {
      inFile.getline( theLine, 255 ) ;
      if( strlen(theLine) ) { // If blank line, do nothing
         if( dataType == CRATER )
            sscanf( theLine, "%d %d %d %d %d %d %d %d %d %d %d %d %d", &VBat2, &VBat1, &TElec,
               &TJct, &TJauge2, &TThermo2, &TJauge1, &TThermo1, &dataHour, &dataMinute, &dataDay,
               &dataMonth, &dataYear ) ;
         if( dataType == TILT1 || dataType == TILT2 )
            sscanf( theLine, "%d %d %d %d %d %d %d %d %d %d %d %d %d %d", &VBat2, &VBat1, &TElec,
               &TAmb, &TJct, &YTang, &Y10Tang, &XRad, &X10Rad, &dataHour, &dataMinute, &dataDay,
               &dataMonth, &dataYear ) ;
         // H h! Don't fall into the 2000 millenia bug!!!
         // Assume that all years lower than 90 are over 2000
         // and all over 90 are all 1900. So we are cool until 2089!
         if( dataYear > 1000 )
               MessageBeep( -1 ) ;
         if( dataYear < 90 )
            dataYear += (unsigned short)2000 ;
         else
            dataYear += (unsigned short)1900 ;

         // Accept data only if same day but greater time or
         // greater day
         TDateTime* newDate1 = 0 ;
         try {
            newDate1 = new TDateTime( (unsigned short)dataYear, (unsigned short)dataMonth, (unsigned short)dataDay ) ;
         }
         catch(...) {
            dataDay = 28 ;
            newDate1 = new TDateTime( (unsigned short)dataYear, (unsigned short)dataMonth, (unsigned short)dataDay ) ;
         }
         TDateTime* newDate2 = 0;
         newDate2 = new TDateTime( (unsigned short)dataHour, (unsigned short)dataMinute, 0, 0 ) ;
         if( firstCompare ) {
            firstCompare   = false ;
            lastHour       = (Word)dataHour ;
            lastMinute     = (Word)dataMinute ;
            lastDay        = (Word)dataDay ;
            lastMonth      = (Word)dataMonth ;
            lastYear       = (Word)(dataYear-1) ;
         }
         TDateTime lastDate1( (unsigned short)lastYear, (unsigned short)lastMonth, (unsigned short)lastDay ) ;
         TDateTime lastDate2( (unsigned short)lastHour, (unsigned short)lastMinute, 0, 0 ) ;
         if( (*newDate2+*newDate1) > (lastDate1+lastDate2) ) {// New data
            if( dataType == CRATER ) {
               outFile << VBat2 << " " << VBat1 << " " << TElec << " " << TJct << " " << TJauge2
                       << " " << TThermo2 << " " << TJauge1 << " " << TThermo1 << " " << dataHour
                       << " " << dataMinute << " " << dataDay << " " << dataMonth << " " << dataYear << endl;
               if( newData )
                  newFile << VBat2 << " " << VBat1 << " " << TElec << " " << TJct << " " << TJauge2
                       << " " << TThermo2 << " " << TJauge1 << " " << TThermo1 << " " << dataHour
                       << " " << dataMinute << " " << dataDay << " " << dataMonth << " " << dataYear << endl;
            }

            if( dataType == TILT1 || dataType == TILT2 ) {
               outFile << VBat2 << " " << VBat1 << " " << TElec << " " << "" << TAmb << " " << TJct << " " << YTang
                       << " " << Y10Tang << " " << XRad << " " << X10Rad << " " << dataHour
                       << " " << dataMinute << " " << dataDay << " " << dataMonth << " " << dataYear << endl ;

               if( newData )
                  newFile << VBat2 << " " << VBat1 << " " << TElec << " " << "" << TAmb << " " << TJct << " " << YTang
                       << " " << Y10Tang << " " << XRad << " " << X10Rad << " " << dataHour
                       << " " << dataMinute << " " << dataDay << " " << dataMonth << " " << dataYear << endl ;
            }
            lastHour       = (Word)dataHour ;
            lastMinute     = (Word)dataMinute ;
            lastDay        = (Word)dataDay ;
            lastMonth      = (Word)dataMonth ;
            lastYear       = (Word)dataYear ;
            if( newDate1 )
               delete newDate1 ;
            if( newDate2 )
               delete newDate2 ;
         }
      }
   }
   inFile.close() ;
}




// --------------------------------------------------------
// SEARCHDAY
// This part searches for available daily files for the
// specified month. If the daily files don't exist,
// tries to look for hourly files.
// --------------------------------------------------------
void TMainToolBar::SearchDay( short month, short maxDay ) {
   // Search for days
   short day = 1 ;
   // Do we have actually CR_MM-?? files? If no exit
   char fName[255] ;
   sprintf( fName, "%s\\%s_%02d-??.", dir.c_str(), prefix.c_str(), month ) ;
   if( !FileExists( fName ) ) {
      SearchHour( month, -31, 24 ) ; /// Check if 31 or current day!!!
      return ;
   }
   while( day < maxDay ) {
      sprintf( fName, "%s\\%s_%02d-%02d.", dir.c_str(), prefix.c_str(), month, day ) ;
      if( FileExists( fName ) ) {
         ScanData( fName ) ;
         lastMonthFound = month ;
      }
      else {
         // Month doesn't exist. In order not to miss data
         // Do we at least have day files? (which implicitely
         // searches for hours too.
         SearchHour( month, day, 24 ) ;
      }

      day++ ;
   }
}



// --------------------------------------------------------
// SEARCHHOUR
// This part searches for available hourly files for the
// specified day of month.
// --------------------------------------------------------
void TMainToolBar::SearchHour( short month, short day, short maxHour ) {
   // Search for hours
   short hour = 0 ;
   // If day = 0, search to see if we have hourly files
   char fName[255] ;
   sprintf( fName, "%s\\%s_%02d-??.??", dir.c_str(), prefix.c_str(), month ) ;
   if( !FileExists( fName ) )
      return ;

   // Do we have actually CR_MM-DD.?? files? If no exit
   if( day < 0 ) {
      // The maxDay value is actually -day
      int maxDay = - day ;
      // Cycle through the days up to maxDay
      for( short i=1; i<=maxDay; i++ ) {
         sprintf( fName, "%s\\%s_%02d-%02d.??", dir.c_str(), prefix.c_str(), month, i ) ;
         if( FileExists( fName ) ) {    ///// != or == ?????
            for( hour =0; hour<24; hour++ ) {
               sprintf( fName,  "%s\\%s_%02d-%02d.%02d",dir.c_str(), prefix.c_str(), month, i, hour ) ;
               if( FileExists( fName ) ) {
                  ScanData( fName ) ;
                  lastMonthFound = month ;
               }
            }
         }
      }
   }
   else {
      while( hour <= maxHour ) {
         sprintf( fName,  "%s\\%s_%02d-%02d.%02d",dir.c_str(), prefix.c_str(), month, day, hour ) ;
         if( FileExists( fName ) ) {
            ScanData( fName ) ;
            lastMonthFound = month ;
         }
         hour++ ;
      }
   }
}

// --------------------------------------------------------
// UPDATE FILES
// Update data from files starting from specified time.
// --------------------------------------------------------
void TMainToolBar::UpdateFiles() {
   // Start with monthly files.
   char fName[255] ;
   short monthIndex = 0 ;
   Word iterations = 12 ;
   lastMonthFound = 1 ;

   // Start with monthly files
   Word mnt = (Word)(lastMonth + 1 ) ;
   if( mnt > 12 )
      mnt = 1 ;

   if( newData ) { // Go only from last data up to now
      if( Month >= lastMonth ) {
         iterations = (Word)(Month - lastMonth + 1) ;
         mnt = lastMonth ;
      }
      else {
         iterations = (Word)(12-lastMonth + Month + 1);
         mnt = Month ;
      }
   }

   if( MainSplash ) {
      MainSplash->SplashProgress->Min  =  0 ;
      MainSplash->SplashProgress->Max  = iterations ;
      MainSplash->SplashProgress->Step =  1 ;
      MainSplash->SplashProgress->Position = 0 ;
   }

   while( monthIndex++ < iterations ) {
      sprintf( fName, "%s\\%s_%02d.", dir.c_str(), prefix.c_str(), mnt ) ;
      if( FileExists( fName ) ) {
         ScanData( fName ) ;
         lastMonthFound = mnt ;
      }
      else
         // Month doesn't exist. In order not to miss data
         // Do we at least have day files? (which implicitely
         // searches for hours too.
         SearchDay( mnt, 31 ) ;

      mnt++ ;
      if( mnt > 12 )
         mnt = 1 ;
      // Update progress bar
      if( MainSplash )
         MainSplash->SplashProgress->StepIt() ;
   }
   // DONE

   // Save the last info to the registry and update batteries
   char header[35] ;

   // Save last file accessed to registry
   switch( dataType ) {

      case CRATER:
         Configure->SetRegInteger( "CRALastHour",   lastHour ) ;
         Configure->SetRegInteger( "CRALastMinute", lastMinute ) ;
         Configure->SetRegInteger( "CRALastDay",    lastDay ) ;
         Configure->SetRegInteger( "CRALastMonth",  lastMonthFound ) ;
         Configure->SetRegInteger( "CRALastYear",   lastYear ) ;
         // Just take a moment to fill-out the battery status
         // Battery 1
         sprintf( header, "CRA #1 - %02d/%02d/%04d at %02d:%02d",
            dataMonth, dataDay, dataYear, dataHour, dataMinute ) ;
         BatteryForm->CRA1BattLabel->Caption = header ;
         BatteryForm->CRA1Meter->Value = VBat1/6553.4*5.26 ;
         sprintf( header, "%.1f V", VBat1/6553.4*5.26 ) ;
         BatteryForm->CRA1BattValue->Caption = header ;
         // Battery 2
         sprintf( header, "CRA #2 - %02d/%02d/%04d at %02d:%02d",
            dataMonth, dataDay, dataYear, dataHour, dataMinute ) ;
         BatteryForm->CRA2BattLabel->Caption = header ;
         BatteryForm->CRA2Meter->Value = VBat2/6553.4*5.26 ;
         sprintf( header, "%.1f V", VBat2/6553.4*5.26 ) ;
         BatteryForm->CRA2BattValue->Caption = header ;
         break ;

      case TILT1:
         Configure->SetRegInteger( "T1LastHour",   lastHour ) ;
         Configure->SetRegInteger( "T1LastMinute", lastMinute ) ;
         Configure->SetRegInteger( "T1LastDay",    lastDay ) ;
         Configure->SetRegInteger( "T1LastMonth",  lastMonthFound ) ;
         Configure->SetRegInteger( "T1LastYear",   lastYear ) ;
         // Just take a moment to fill-out the battery status
         // Tilt1 Batt 1
         sprintf( header, "T1 #1 - %02d/%02d/%04d at %02d:%02d",
            dataMonth, dataDay, dataYear, dataHour, dataMinute ) ;
         BatteryForm->T11BattLabel->Caption = header ;
         BatteryForm->T11Meter->Value = VBat1/6553.4*5.26 ;
         sprintf( header, "%.1f V", VBat1/6553.4*5.26 ) ;
         BatteryForm->T11BattValue->Caption = header ;
         // Tilt1 Batt 2
         sprintf( header, "T1 #2 - %02d/%02d/%04d at %02d:%02d",
            dataMonth, dataDay, dataYear, dataHour, dataMinute ) ;
         BatteryForm->T12BattLabel->Caption = header ;
         BatteryForm->T12Meter->Value = VBat2/6553.4*5.26 ;
         sprintf( header, "%.1f V", VBat2/6553.4*5.26 ) ;
         BatteryForm->T12BattValue->Caption = header ;
         break ;

      case TILT2:
         Configure->SetRegInteger( "T2LastHour",   lastHour ) ;
         Configure->SetRegInteger( "T2LastMinute", lastMinute ) ;
         Configure->SetRegInteger( "T2LastDay",    lastDay ) ;
         Configure->SetRegInteger( "T2LastMonth",  lastMonthFound  ) ;
         Configure->SetRegInteger( "T2LastYear",   lastYear ) ;
         // Just take a moment to fill-out the battery status
         // T2 Batt1
         sprintf( header, "T2 #1 - %02d/%02d/%04d at %02d:%02d",
            dataMonth, dataDay, dataYear, dataHour, dataMinute ) ;
         BatteryForm->T21BattLabel->Caption = header ;
         BatteryForm->T21Meter->Value = VBat1/6553.4*5.26 ;
         sprintf( header, "%.1f V", VBat1/6553.4*5.26 ) ;
         BatteryForm->T21BattValue->Caption = header ;
         // T2 Batt2
         sprintf( header, "T2 #2 - %02d/%02d/%04d at %02d:%02d",
            dataMonth, dataDay, dataYear, dataHour, dataMinute ) ;
         BatteryForm->T22BattLabel->Caption = header ;
         BatteryForm->T22Meter->Value = VBat2/6553.4*5.26 ;
         sprintf( header, "%.1f V", VBat2/6553.4*5.26 ) ;
         BatteryForm->T22BattValue->Caption = header ;
         break ;
   }
}

// --------------------------------------------------------
// GETLASTINFO
// Get the last time and date of data.
// Called at startup to generate files
// Also called each time the timer decides we have to
// update the data.
// --------------------------------------------------------
void TMainToolBar::GetLastInfo() {

   switch( dataType ) {

      case CRATER:
         fileName   = Configure->GetRegString( "LastCRA", "none" ) ;
         lastHour   = (Word)Configure->GetRegInteger( "CRALastHour", 0      ) ;
         lastMinute = (Word)Configure->GetRegInteger( "CRALastMinute", 0  ) ;
         lastDay    = (Word)Configure->GetRegInteger( "CRALastDay", 1     ) ;
         lastMonth  = (Word)Configure->GetRegInteger( "CRALastMonth", 1   ) ;
         lastYear   = (Word)Configure->GetRegInteger( "CRALastYear", 1990 ) ;
         break ;

      case TILT1:
         fileName   = Configure->GetRegString(  "LastTilt1", "none" ) ;
         lastHour   = (Word)Configure->GetRegInteger( "T1LastHour", 0      ) ;
         lastMinute = (Word)Configure->GetRegInteger( "T1LastMinute", 0  ) ;
         lastDay    = (Word)Configure->GetRegInteger( "T1LastDay", 1     ) ;
         lastMonth  = (Word)Configure->GetRegInteger( "T1LastMonth", 1   ) ;
         lastYear   = (Word)Configure->GetRegInteger( "T1LastYear", 1990 ) ;
         break ;

      case TILT2:
         fileName   = Configure->GetRegString(  "LastTilt2", "none" ) ;
         lastHour   = (Word)Configure->GetRegInteger( "T2LastHour", 0      ) ;
         lastMinute = (Word)Configure->GetRegInteger( "T2LastMinute", 0  ) ;
         lastDay    = (Word)Configure->GetRegInteger( "T2LastDay", 1     ) ;
         lastMonth  = (Word)Configure->GetRegInteger( "T2LastMonth", 1   ) ;
         lastYear   = (Word)Configure->GetRegInteger( "T2LastYear", 1990 ) ;
         break ;
   }
}

// --------------------------------------------------------
// PrintBUTTONCLICK
// Print the currently selected chart
// --------------------------------------------------------
void __fastcall TMainToolBar::PrintButtonClick(TObject *Sender) {
   switch( currentActiveWindow ) {
      case TILT1:
         Tilt1->PrintChart() ;
         break ;

      case TILT2:
         Tilt2->PrintChart() ;
         break ;

      case TILT1BATT:
         Tilt1Batt->PrintChart() ;
         break ;

      case TILT2BATT:
         Tilt2Batt->PrintChart() ;
         break ;

      case TILT1TAN:
         Tilt1Tan->PrintChart() ;
         break ;

      case TILT2TAN:
         Tilt2Tan->PrintChart() ;
         break ;

      case TEMPFUM1:
         TempFum1->PrintChart() ;
         break ;

      case TEMPFUM2:
         TempFum2->PrintChart() ;
         break ;

      case PO2S1WIN:
         PO2S1->PrintChart() ;
         break ;

      case PO2S2WIN:
         PO2S2->PrintChart() ;
         break ;

      case OXIDS1:
         OxidS1->PrintChart() ;
         break ;

      case OXIDS2:
         OxidS2->PrintChart() ;
         break ;
   }
}


// --------------------------------------------------------
// Called when the main form is created!
// Just launch and init processes
// --------------------------------------------------------
void __fastcall TMainToolBar::FormCreate(TObject *Sender)
{
   // Is initialization done?
   // No!
   if( !initDone ) {
      initDone = true ;
      // Update data files
      ProcessFiles() ;
      MainSplash->Hide() ;
      delete MainSplash ;
      MainSplash = 0 ;
      // And eventually display warning message (if any).
      if( initMessage != "" ) {
         initMessage = "Please define the directories for the " + initMessage +
                  " stations." ;
         MessageBox( 0, initMessage.c_str(), "Data Error", MB_OK ) ;
      }
   }
   else {
      // Yes
      // Setup the timer
      int hr, mn ;
      Configure->RegWorkWith( "Updating" ) ;
      hr = Configure->GetRegInteger( "Hour", 0 ) ;
      mn = Configure->GetRegInteger( "Minute", 40 ) ;
      // Calculate next possible update
      TDateTime voidTime( 0, (WORD)mn, 0, 0 ) ;
      nextUpdate = voidTime ;
      // Get current time
      TDateTime currentDate( Date() ) ;
      TDateTime currentTime( Time() ) ;
      while( (currentDate+currentTime)>nextUpdate )
         nextUpdate += 1/24.0*hr ;
   }
}
//---------------------------------------------------------------------------

