// CLASS SEARCH
// This class is used to search for data files. The goal here is
// to either detect the file with the oldest stored values.
// The directory is filled with event files for a whole year, so
// we need to get the next file just after the current date and
// time one.
//
// (C) Steve JORDI - 1996-1999  
// steve@sjordi.com - www.sjordi.com
//
// All this code is copyrighted by Steve Jordi and
// cannot be reproduced without the consent of the author.
// The executable may be freely distributed.
//
// MAKE  SURE  TO  UNDERSTAND  AND  ACCEPT  THE STATEMENTS BELOW
// BEFORE USING THE SOFTWARE:
//
// THIS PROGRAM  IS  PROVIDED  "AS IS"  WITHOUT  WARRANTY OF ANY
// KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED
// TO  THE  IMPLIED  WARRANTIES  OF MERCHANTABILITY  AND FITNESS
// FOR  A  PARTICULAR PURPOSE. THE ENTIRE RISK AS TO THE QUALITY
// AND   PERFORMANCE  OF  THE  PROGRAM  IS  WITH YOU. SHOULD THE
// PROGRAM  PROVE   DEFECTIVE,  YOU   (NOT  STEVE JORDI)  ASSUME
// THE ENTIRE COST OF ALL  NECESSARY   SERVICING,   REPAIR,   OR
// CORRECTION. IN NO EVENT WILL STEVE JORDI BE LIABLE TO YOU FOR
// ANY  DAMAGES ,  INCLUDING ANY LOST PROFITS,  LOST SAVINGS, OR
// OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES  ARISING OUT OF THE
// USE OR INABILITY TO USE  SUCH  PROGRAM  EVEN  IF  STEVE JORDI
// HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES,   OR FOR
// ANY  CLAIM BY ANY OTHER PARTY.
// ALSO, USE IT IN LIFE THREATENING SITUATIONS AT YOUR OWN RISKS
// (SUCH AS VOLCANO MONITORING). STEVE JORDI CANNOT BE ACCOUNTED
// AS RESPONSIBLE FOR ANY LIFE OR PROPERTY LOST IN CASE AN ALARM
// SYSTEM HAD A MALFUNCTION OR WAS NOT ABLE TO CORRECTLY  BEHAVE
// IN A CRISIS ENVIRONMENT.
//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop

#include "vcl\sysutils.hpp"
#include "SearchFile.h"


// --------------------------------------------------------
// CONSTRUCTOR
// Store the current date and time and default to CR file.
// --------------------------------------------------------
Search::Search() {
   // First thing, get current time and date
   DecodeDate( Date(), Year, Month, Day ) ;
   DecodeTime( Time(), Hour, Minute, Second, MSec ) ;
   SetFileName( "CR" ) ;
}


// --------------------------------------------------------
// ADDONEHOUR
// Prepare date/time to reflect the current time + 1 hour.
// This is used to see if a file for the next hour exists.
// It is called by the FILEFINDNOW function.
// --------------------------------------------------------
void Search::AddOneHour() {
   TDateTime date1( Year, Month, Day ) ;
   TDateTime date2( Hour, Minute, Second, MSec ) ;
   TDateTime newTime = date1+date2 +(double)(1.0/24.0);
   DecodeDate( newTime, Year, Month, Day ) ;
   DecodeTime( newTime, Hour, Minute, Second, MSec ) ;
}

// --------------------------------------------------------
// MINUSONEHOUR
// Remove one hour to current files name
// --------------------------------------------------------
void Search::MinusOneHour() {
   TDateTime date1( Year, Month, Day ) ;
   TDateTime date2( Hour, Minute, Second, MSec ) ;
   TDateTime newTime = date1+date2 +(double)(1.0/24.0);
   newTime -= 1.0/24.0 ;
   DecodeDate( newTime, Year, Month, Day ) ;
   DecodeTime( newTime, Hour, Minute, Second, MSec ) ;
}



// --------------------------------------------------------
// ADDONEDAY
// Used to prepare date/time to reflect the current date
// + 1 day.
// --------------------------------------------------------
void Search::AddOneDay() {
   TDateTime date1( Year, Month, Day ) ;
   TDateTime date2( Hour, Minute, Second, MSec ) ;
   TDateTime newTime = date1+1+date2 ;
   DecodeDate( newTime, Year, Month, Day ) ;
   DecodeTime( newTime, Hour, Minute, Second, MSec ) ;
}

// --------------------------------------------------------
// SEARCHFILE
// This function searches the specified directory for the
// file with the oldest data.
// --------------------------------------------------------
AnsiString Search::SearchFile() {
   bool findRegular = true ;
   while( !FileExists( "d:\\tiltmeters\\data\\CR\\" + fileName ) ) {
      AddOneDay() ;
      TDateTime date1( Year, Month, Day ) ;
      TDateTime date2( Hour, Minute, Second, MSec ) ;
      fileName = "CR_" + IntToStr( Month ) +
             "-" + IntToStr( Day ) + ".*" ;
      if( ( date1 == Date() ) && (date2>=Time()) ) {
         findRegular = false ;
         break ;
      }
   }
   Hour = 0 ;
   SetFileName( "CR" ) ;
   if( findRegular )
      while( !FileExists( "d:\\tiltmeters\\data\\CR\\" + fileName ) ) {
         AddOneHour() ;
         SetFileName( "CR" ) ;
//       if( ( date1 == Date() ) && (date2>=Time()) )
//          break ;
      }
   return fileName ;
}

// --------------------------------------------------------
// SETFILENAME
// Set the filename to the prefix + the new date/time we
// just have processed.
// The file name format is PP_MM-DD.HH
//    where:   PP is the Prefix
//             MM is the month on 2 digits
//             DD is the day on 2 digits
//             HH is the hour on 2 digits
// --------------------------------------------------------
void Search::SetFileName( AnsiString prefix ) {
   TDateTime date1( Year, Month, Day ) ;
   TDateTime date2( Hour, Minute, Second, MSec ) ;
   fileName = UpperCase(prefix) + "_" ;
   if( Month<10 )
      fileName += "0" ;
   fileName += IntToStr( Month ) + "-" ;
   if( Day <10 )
      fileName += "0" ;
   fileName += IntToStr( Day ) + "." ;
   if( Hour < 10 )
      fileName += "0" ;
   fileName += IntToStr( Hour ) ;
}


// --------------------------------------------------------
// GETFILENAME
// Return the current filename
// --------------------------------------------------------
AnsiString Search::GetFileName() {
   return fileName ;
}

// --------------------------------------------------------
// LASTFILE
// Returns if we currently are on the file valid for current
// date and time. In which case you don't open it. You should
// always stop at the file before this one.
// --------------------------------------------------------
bool Search::LastFile() {
   // Current time
   Word  CYear, CMonth, CDay, CHour, CMinute, CSecond, CMSec ;
   DecodeDate( Date(), CYear, CMonth, CDay ) ;
   DecodeTime( Time(), CHour, CMinute, CSecond, CMSec ) ;
   // Compare last filename retrieved to current time
   if( CMonth == Month && CDay == Day && CHour == Hour &&
       CMinute == Minute && CSecond == Second && CMSec == MSec )
      return true ;
   else
      return false ;
}


// --------------------------------------------------------
// FINDNEXTNOW
// Look for the immediately next file from current date
// and time. Progressing by 1 hour steps.
// --------------------------------------------------------
void Search::FindNextNow() {
   DecodeDate( Date(), Year, Month, Day ) ;
   DecodeTime( Time(), Hour, Minute, Second, MSec ) ;
   AddOneHour() ;
   SetFileName( prefix ) ;
   while( !FileExists( dir + "\\" + fileName ) ) {
      AddOneHour() ;
      TDateTime date1( Year, Month, Day ) ;
      TDateTime date2( Hour, Minute, Second, MSec ) ;
      SetFileName( prefix ) ;
   }
}


// --------------------------------------------------------
// SETDIRECTORY
// Set the working directory
// --------------------------------------------------------
void Search::SetDirectory( AnsiString direct) {
   dir = UpperCase(direct) ;
}


// --------------------------------------------------------
// SETPREFIX
// Store the file directory
// --------------------------------------------------------
void Search::SetPrefix( AnsiString prf ) {
   prefix = UpperCase(prf) ;
}


// --------------------------------------------------------
// FINDCURRENTFILE
// Find the file for current time and date
// --------------------------------------------------------
void Search::FindCurrentFile() {
   DecodeDate( Date(), Year, Month, Day ) ;
   DecodeTime( Time(), Hour, Minute, Second, MSec ) ;
   SetFileName( prefix ) ;
   while( !FileExists( dir + "\\" + fileName ) ) {
      MinusOneHour() ;
      TDateTime date1( Year, Month, Day ) ;
      TDateTime date2( Hour, Minute, Second, MSec ) ;
      SetFileName( prefix ) ;
   }
}
