// Class   : TCPMapper
// Author  : Russ Ethington
// Version : 6/20/00 for Java 1.1

import java.util.*;
import java.net.*;
import java.io.*;

public class TCPMapper extends ConcurrentServer
{
    private static int port = 0;
    private static int uniqueId = 0;
    private static Integer uniqueIdMonitor = new Integer(0);
    private static String proxyHost = "webhost.lehman.com";
    private static int proxyPort = 80;
    private static boolean logging = true;  // log by default

    public static void main(String args[])
    {
        if (args.length < 2) usage();

        try
        {
            port = Integer.parseInt(args[0]);
        }
        catch (Exception e)
        {
            print("Invalid port number: " + args[0]);
            usage();
        }

        print("TCPMapper Server 1.2");
        print("  by Russ Ethington (Russ.Eth@ix.netcom.com)\n");
        
        try
        {
            StringTokenizer t = new StringTokenizer(args[1], " :");
            proxyHost = t.nextToken();
            proxyPort = Integer.parseInt(t.nextToken());
        }
        catch (Exception e)
        {
            print("Invalid destination host/port: " + args[1]);
            usage();
        }

        if (args.length == 3 && args[2].equalsIgnoreCase("-nolog"))
        {
            logging = false;
        }
      
        print("Setting listening port to: " + port);
        print("Setting destination to: " + proxyHost + ":" + proxyPort);
        if (logging)
            print("Logging is enabled.");
        else
           print("Logging is disabled.");
      
        print("Ready for connections...");
        new TCPMapper().start(port);
    }

    public static void usage()
    {
        print("Usage: <listeningPort> <destinationHost:port> [-nolog]");
        System.exit(-1);
    }

    public void handleSession(Socket socketFromClient)
    {
        int              id;
        Socket           socketToServer = null;
        InputStream      serverIn = null;
        OutputStream     serverOut = null;
        DataInputStream  clientIn;
        DataOutputStream clientOut;

        synchronized (uniqueIdMonitor)
        {
            uniqueId++;
        }

        String from = socketFromClient.getInetAddress().toString();
        String to = proxyHost + ":" + proxyPort;
        print("" + uniqueId + ": Mapping connection from " + from + " to " + to);

        try
        {
            String line;
            clientIn = new DataInputStream(socketFromClient.getInputStream());
            clientOut = new DataOutputStream(socketFromClient.getOutputStream());

            socketToServer = new Socket(proxyHost, proxyPort);

            serverIn = socketToServer.getInputStream();
            serverOut = socketToServer.getOutputStream();

            ByteCopier copier1 = null;
            ByteCopier copier2 = null;
            DataLogger logger1 = null;
            DataLogger logger2 = null;

            if (logging)
            {
                logger1 = new DataLogger(uniqueId, "To");
                logger2 = new DataLogger(uniqueId, "From");
                copier1 = new ByteCopier(clientIn, serverOut, logger1);
                copier2 = new ByteCopier(serverIn, clientOut, logger2);
            }
            else 
            {
                copier1 = new ByteCopier(clientIn, serverOut);
                copier2 = new ByteCopier(serverIn, clientOut);
            }

            copier1.start();
            copier2.start();

            while (copier1.isAlive() && copier2.isAlive()) waitHere(100);

            if (logging)
            {
                logger1.close();
                logger2.close();
            }
            copier1 = null;
            copier2 = null;

            try
            {
                serverIn.close();
                serverOut.close();
                socketToServer.close();
            }
            catch (Exception e) { }
        }
        catch (IOException e)
        {
            print(uniqueId + ": Cannot connect to destination '" + to + "': " + e);
            return;
        }
    }

    public synchronized void waitHere(long millis)
    {
        try { this.wait(millis); }
        catch (InterruptedException e) { }
    }

    public static void print(String message)
    {
        System.out.println(message);
        System.out.flush();
    }
}